@@ -1,7 +1,7 @@
 /*
  * audio_port.h - base-class for objects providing sound at a port
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -38,15 +38,13 @@
 
 #endif
 
-
-#include "mixer.h"
-#include "effect_board.h"
+#include "effect_chain.h"
 
 
-class audioPort : public engineObject
+class audioPort
 {
 public:
-	audioPort( const QString & _name, engine * _engine );
+	audioPort( const QString & _name );
 	~audioPort();
 
 	inline surroundSampleFrame * firstBuffer( void )
@@ -76,6 +74,11 @@ public:
 		return( m_nextFxChannel );
 	}
 
+	inline effectChain * getEffects( void )
+	{
+		return( m_effects );
+	}
+
 	void setNextFxChannel( const fx_ch_t _chnl )
 	{
 		m_nextFxChannel = _chnl;
@@ -94,7 +97,8 @@ public:
 	{
 		NONE, FIRST, BOTH
 	} m_bufferUsage;
-
+	
+	inline bool processEffects( void ) { return( m_effects->processAudioBuffer( m_firstBuffer, m_frames ) ); };
 
 private:
 	surroundSampleFrame * m_firstBuffer;
@@ -103,6 +107,9 @@ private:
 	fx_ch_t m_nextFxChannel;
 
 	QString m_name;
+	
+	effectChain * m_effects;
+	fpp_t m_frames;
 
 } ;
 
