@@ -4,7 +4,7 @@
  * original file by ??? 
  * modified and enhanced by Tobias Doerffel
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -46,7 +46,6 @@ template<ch_cnt_t CHANNELS = DEFAULT_CHANNELS>
 class basicFilters
 {
 public:
-
 	enum filterTypes
 	{
 		LOWPASS,
@@ -56,19 +55,31 @@ public:
 		NOTCH,
 		ALLPASS,
 		MOOG,
-		SIMPLE_FLT_CNT,
-		DOUBLE_LOWPASS	= 16+LOWPASS,
-		DOUBLE_MOOG	= 16+MOOG
+		SIMPLE_FLT_CNT
 	} ;
 
-	static inline filterTypes getFilterType( const int _idx )
+	static inline float minQ( void )
 	{
-		if( _idx < SIMPLE_FLT_CNT )
+		return( 0.01f );
+	}
+
+	inline void setFilterType( const int _idx )
+	{
+		m_double_filter = _idx >= SIMPLE_FLT_CNT;
+		if( !m_double_filter )
 		{
-			return( static_cast<filterTypes>( _idx ) );
+			m_type = static_cast<filterTypes>( _idx );
+			return;
 		}
-		return( static_cast<filterTypes>( DOUBLE_LOWPASS + _idx -
-							SIMPLE_FLT_CNT ) );
+		m_type = static_cast<filterTypes>( LOWPASS + _idx -
+							SIMPLE_FLT_CNT );
+		if( m_subFilter == NULL )
+		{
+			m_subFilter = new basicFilters<CHANNELS>(
+						static_cast<sample_rate_t>(
+							m_sampleRate ) );
+		}
+		m_subFilter->m_type = m_type;
 	}
 
 	inline basicFilters( const sample_rate_t _sample_rate ) :
@@ -77,7 +88,8 @@ public:
 		m_b2a0( 0.0f ),
 		m_a1a0( 0.0f ),
 		m_a2a0( 0.0f ),
-		m_sampleRate( 1.0f / _sample_rate ),
+		m_double_filter( FALSE ),
+		m_sampleRate( _sample_rate ),
 		m_subFilter( NULL )
 	{
 		// reset in/out history
@@ -104,20 +116,27 @@ public:
 		switch( m_type )
 		{
 			case MOOG:
-			case DOUBLE_MOOG:
 			{
 				sample_t x = _in0 - m_r*m_y4[_chnl];
 
 				// four cascaded onepole filters
 				// (bilinear transform)
-				m_y1[_chnl] = x*m_p + m_oldx[_chnl]*m_p -
-								m_k*m_y1[_chnl];
-				m_y2[_chnl] = m_y1[_chnl]*m_p+m_oldy1[_chnl]*
-							m_p - m_k*m_y2[_chnl];
-				m_y3[_chnl] = m_y2[_chnl]*m_p+m_oldy2[_chnl]*
-							m_p - m_k*m_y3[_chnl];
-				m_y4[_chnl] = m_y3[_chnl]*m_p+m_oldy3[_chnl]*
-							m_p - m_k*m_y4[_chnl];
+				m_y1[_chnl] = tLimit(
+						( x + m_oldx[_chnl] ) * m_p
+							- m_k * m_y1[_chnl],
+								-10.0f, 10.0f );
+				m_y2[_chnl] = tLimit(
+					( m_y1[_chnl] + m_oldy1[_chnl] ) * m_p
+							- m_k * m_y2[_chnl],
+								-10.0f, 10.0f );
+				m_y3[_chnl] = tLimit(
+					( m_y2[_chnl] + m_oldy2[_chnl] ) * m_p
+							- m_k * m_y3[_chnl],
+								-10.0f, 10.0f );
+				m_y4[_chnl] = tLimit(
+					( m_y3[_chnl] + m_oldy3[_chnl] ) * m_p
+							- m_k * m_y4[_chnl],
+								-10.0f, 10.0f );
 
 				m_oldx[_chnl] = x;
 				m_oldy1[_chnl] = m_y1[_chnl];
@@ -209,7 +228,7 @@ public:
 				break;
 		}
 
-		if( m_subFilter != NULL )
+		if( m_double_filter )
 		{
 			return( m_subFilter->update( out, _chnl ) );
 		}
@@ -218,40 +237,31 @@ public:
 		return( out );
 	}
 
-   
-	inline void calcFilterCoeffs( const filterTypes _type, float _freq,
-					const float _q
+
+	inline void calcFilterCoeffs( float _freq, float _q
 				/*, const bool _q_is_bandwidth = FALSE*/ )
 	{
 		// temp coef vars
-		m_type = _type;
-		_freq = tMax( _freq, 0.01f );// limit freq for not getting
+		_freq = tMax( _freq, 0.01f );// limit freq and q for not getting
 					      // bad noise out of the filter...
+		_q = tMax( _q, minQ() );
 
-		switch( m_type )
+		if( m_type == MOOG )
 		{
-			case DOUBLE_MOOG:
-			{
-				if( m_subFilter == NULL )
-				{
-					m_subFilter =
-						new basicFilters<CHANNELS>(
-			static_cast<sample_rate_t>( 1.0f / m_sampleRate ) );
-				}
-				m_subFilter->calcFilterCoeffs( MOOG, _freq,
-									_q );
-			}
+			// [ 0 - 0.5 ]
+			const float f = _freq / m_sampleRate;
+			// (Empirical tunning)
+			m_p = ( 3.6f - 3.2f * f ) * f;
+			m_k = 2.0f * m_p - 1;
+			m_r = _q * powf( M_E, ( 1 - m_p ) * 1.386249f );
 
-			case MOOG:
+			if( m_double_filter )
 			{
-				// [ 0 - 1 ]
-				const float f = 2 * _freq * m_sampleRate;
-				// (Empirical tunning)
-				m_k = 3.6f*f - 1.6f*f*f - 1;
-				m_p = (m_k+1)*0.5f;
-				m_r = _q*powf( M_E, ( ( 1-m_p ) * 1.386249f ) );
-				break;
+				m_subFilter->m_r = m_r;
+				m_subFilter->m_p = m_p;
+				m_subFilter->m_k = m_k;
 			}
+			return;
 
 /*			case DOUBLE_MOOG2:
 			{
@@ -259,7 +269,7 @@ public:
 				{
 					m_subFilter =
 						new basicFilters<CHANNELS>(
-							1.0f / m_sampleRate );
+							m_sampleRate );
 				}
 				m_subFilter->calcFilterCoeffs( MOOG2, _freq,
 									_q );
@@ -267,8 +277,8 @@ public:
 
 			case MOOG2:
 			{
-				const float kfc = 2 * _freq * m_sampleRate;
-				const float kf = _freq * m_sampleRate;
+				const float kfc = 2 * _freq / m_sampleRate;
+				const float kf = _freq / m_sampleRate;
 				const float kfcr = 1.8730 * ( kfc*kfc*kfc ) +
 							0.4955 * ( kfc*kfc ) +
 							0.6490 * kfc + 0.9988;
@@ -279,82 +289,68 @@ public:
 				m_r = 4 * _q * kacr;
 				break;
 			}*/
+		}
 
-			default:
-			{
-				// other filters
-				const float omega	= F_2PI * _freq *
-								m_sampleRate;
-				const float tsin	= sinf( omega );
-				const float tcos	= cosf( omega );
-				//float alpha;
-  
-				//if (q_is_bandwidth)
-				//alpha = tsin*sinhf(logf(2.0f)/2.0f*q*omega/
-				//					tsin);
-				//else
-				const float alpha = 0.5f * tsin / _q;
+		// other filters
+		const float omega = F_2PI * _freq / m_sampleRate;
+		const float tsin = sinf( omega );
+		const float tcos = cosf( omega );
+		//float alpha;
 
-				const float a0 = 1.0f / ( 1.0f + alpha );
-   
-				if( m_type == LOWPASS ||
-						m_type == DOUBLE_LOWPASS )
-				{
-					m_b0a0 = ((1.0f-tcos)*0.5f)*a0;
-					m_b1a0 = (1.0f-tcos)*a0;
-					m_b2a0 = m_b0a0;//((1.0f-tcos)/2.0f)*a0;
-					m_a1a0 = (-2.0f*tcos)*a0;
-					if( m_type == DOUBLE_LOWPASS )
-					{
-						if( m_subFilter == NULL )
-						{
-							m_subFilter =
-		new basicFilters<CHANNELS>( static_cast<sample_rate_t>(
-							1.0f / m_sampleRate ) );
-						}
-						m_subFilter->calcFilterCoeffs(
-							LOWPASS, _freq, _q );
-					}
-				}
-				else if( m_type == HIPASS )
-				{
-					m_b0a0 = ((1.0f+tcos)*0.5f)*a0;
-					m_b1a0 = (-1.0f-tcos)*a0;
-					m_b2a0 = m_b0a0;//((1.0f+tcos)/2.0f)*a0;
-					m_a1a0 = (-2.0f*tcos)*a0;
-				}
-				else if( m_type == BANDPASS_CSG )
-				{
-					m_b0a0 = tsin*0.5f*a0;
-					m_b1a0 = 0.0f;
-					m_b2a0 = -tsin*0.5f*a0;
-					m_a1a0 = (-2.0f*tcos)*a0;
-				}
-				else if( m_type == BANDPASS_CZPG )
-				{
-					m_b0a0 = alpha*a0;
-					m_b1a0 = 0.0f;
-					m_b2a0 = (-alpha)*a0;
-					m_a1a0 = (-2.0f*tcos)*a0;
-				}
-				else if( m_type == NOTCH )
-				{
-					m_b0a0 = a0;
-					m_b1a0 = (-2.0f*tcos)*a0;
-					m_b2a0 = a0;
-					m_a1a0 = m_b1a0;//(-2.0f*tcos)*a0;
-				}
-				else if( m_type == ALLPASS )
-				{
-					m_b0a0 = (1.0f-alpha)*a0;
-					m_b1a0 = (-2.0f*tcos)*a0;
-					m_b2a0 = 1.0;//(1.0f+alpha)*a0;
-					m_a1a0 = m_b1a0;//(-2.0f*tcos)*a0;
-					//m_a2a0 = m_b0a0;//(1.0f-alpha)*a0;
-				}
-				m_a2a0 = (1.0f-alpha)*a0;
+		//if (q_is_bandwidth)
+		//alpha = tsin*sinhf(logf(2.0f)/2.0f*q*omega/
+		//					tsin);
+		//else
+		const float alpha = 0.5f * tsin / _q;
+
+		const float a0 = 1.0f / ( 1.0f + alpha );
+
+		m_a1a0 = -2.0f * tcos * a0;
+		m_a2a0 = ( 1.0f - alpha ) * a0;
+
+		switch( m_type )
+		{
+			case LOWPASS:
+				m_b1a0 = ( 1.0f - tcos ) * a0;
+				m_b0a0 = m_b1a0 * 0.5f;
+				m_b2a0 = m_b0a0;//((1.0f-tcos)/2.0f)*a0;
 				break;
-			}
+			case HIPASS:
+				m_b1a0 = ( -1.0f - tcos ) * a0;
+				m_b0a0 = m_b1a0 * -0.5f;
+				m_b2a0 = m_b0a0;//((1.0f+tcos)/2.0f)*a0;
+				break;
+			case BANDPASS_CSG:
+				m_b1a0 = 0.0f;
+				m_b0a0 = tsin * 0.5f * a0;
+				m_b2a0 = -m_b0a0;
+				break;
+			case BANDPASS_CZPG:
+				m_b1a0 = 0.0f;
+				m_b0a0 = alpha * a0;
+				m_b2a0 = -m_b0a0;
+				break;
+			case NOTCH:
+				m_b1a0 = m_a1a0;
+				m_b0a0 = a0;
+				m_b2a0 = a0;
+				break;
+			case ALLPASS:
+				m_b1a0 = m_a1a0;
+				m_b0a0 = m_a2a0;
+				m_b2a0 = 1.0f;//(1.0f+alpha)*a0;
+				break;
+			default:
+				break;
+		}
+
+		if( m_double_filter )
+		{
+			m_subFilter->m_b0a0 = m_b0a0;
+			m_subFilter->m_b1a0 = m_b1a0;
+			m_subFilter->m_b2a0 = m_b2a0;
+			m_subFilter->m_a1a0 = m_a1a0;
+			m_subFilter->m_a2a0 = m_a2a0;
 		}
 	}
 
@@ -375,9 +371,11 @@ private:
 	frame m_y1, m_y2, m_y3, m_y4, m_oldx, m_oldy1, m_oldy2, m_oldy3;
 
 	filterTypes m_type;
+	bool m_double_filter;
 
 	float m_sampleRate;
 	basicFilters<CHANNELS> * m_subFilter;
+
 } ;
 
 
