@@ -2,7 +2,7 @@
  * dummy_plugin.h - empty plugin which is used as fallback if a plugin couldn't
  *                  be found
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -33,8 +33,8 @@
 class dummyPlugin : public plugin
 {
 public:
-	inline dummyPlugin( engine * _engine = NULL ) :
-		plugin( NULL, _engine )
+	inline dummyPlugin( void ) :
+		plugin( NULL )
 	{
 	}
 
