@@ -1,7 +1,7 @@
 /*
  * file_browser.h - include file for fileBrowser
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -36,11 +36,13 @@
 #ifdef QT4
 
 #include <QtCore/QDir>
+#include <QtCore/QMutex>
 
 #else
 
 #include <qlistview.h>
 #include <qdir.h>
+#include <qmutex.h>
 
 #define cleanPath cleanDirPath
 
@@ -48,39 +50,44 @@
 
 
 #include "side_bar_widget.h"
-#include "engine.h"
 
 
+#ifndef QT3
+class QListViewItem;
+#else
+class Q3ListViewItem;
+#endif
+class QPixmap;
+
 class fileItem;
 class listView;
 class playHandle;
-class QPixmap;
 class trackContainer;
 
 
 
-class fileBrowser : public sideBarWidget, public engineObject
+class fileBrowser : public sideBarWidget
 {
 	Q_OBJECT
 public:
 	fileBrowser( const QString & _directories, const QString & _filter,
 			const QString & _title, const QPixmap & _pm,
-					QWidget * _parent, engine * _engine );
+							QWidget * _parent );
 	virtual ~fileBrowser();
 
+	static bool isDirWithContent( const QString & _path,
+						const QString & _filter );
+
 
 public slots:
 	void reloadTree( void );
 
 
 protected slots:
-#ifdef QT4
 	void contextMenuRequest( Q3ListViewItem * _i, const QPoint & _pos,
 								int _col );
-#else
 	void contextMenuRequest( QListViewItem * _i, const QPoint & _pos,
 								int _col );
-#endif
 	void sendToActiveInstrumentTrack( void );
 	void openInNewInstrumentTrackSE( void );
 	void openInNewInstrumentTrackBBE( void );
@@ -104,11 +111,11 @@ private:
 
 
 
-class listView : public Q3ListView, public engineObject
+class listView : public Q3ListView
 {
 	Q_OBJECT
 public:
-	listView( QWidget * _parent, engine * _engine );
+	listView( QWidget * _parent );
 	virtual ~listView();
 
 
@@ -124,13 +131,23 @@ private:
 	QPoint m_pressPos;
 
 	playHandle * m_previewPlayHandle;
+	QMutex m_pphMutex;
 
 } ;
 
 
 
 
-class directory : public Q3ListViewItem
+class directory :
+#ifndef QT3
+	public Q3ListViewItem
+// trick moc
+#if 0
+		,
+#endif
+#else
+	public QListViewItem
+#endif
 {
 public:
 	directory( Q3ListView * _parent, const QString & _filename,
@@ -184,13 +201,27 @@ private:
 
 
 
-class fileItem : public Q3ListViewItem
+class fileItem :
+#ifndef QT3
+	public Q3ListViewItem
+// trick moc
+#if 0
+		,
+#endif
+#else
+	public QListViewItem
+#endif
 {
 public:
 	fileItem( Q3ListView * _parent, const QString & _name,
-						const QString & _path );
+							const QString & _path );
+#ifndef QT3
 	fileItem( Q3ListViewItem * _parent, const QString & _name,
-						const QString & _path );
+							const QString & _path );
+#else
+	fileItem( QListViewItem * _parent, const QString & _name,
+							const QString & _path );
+#endif
 
 	inline QString fullName( void ) const
 	{
@@ -213,6 +244,9 @@ public:
 		return( m_type );
 	}
 
+	QString extension( void );
+	static QString extension( const QString & _file );
+
 
 private:
 	void initPixmapStuff( void );
