@@ -1,7 +1,7 @@
 /*
  * midi_alsa_seq.h - ALSA-sequencer-client
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -71,7 +71,7 @@ class midiALSASeq :
 {
 	Q_OBJECT
 public:
-	midiALSASeq( engine * _engine );
+	midiALSASeq( void );
 	virtual ~midiALSASeq();
 
 	static QString probeDevice( void );
@@ -79,7 +79,8 @@ public:
 
 	inline static QString name( void )
 	{
-		return( setupWidget::tr( "ALSA-Sequencer (Advanced Linux Sound "
+		return( QT_TRANSLATE_NOOP( "setupWidget",
+			"ALSA-Sequencer (Advanced Linux Sound "
 							"Architecture)" ) );
 	}
 
@@ -169,6 +170,8 @@ private:
 	QStringList m_readablePorts;
 	QStringList m_writeablePorts;
 
+	int m_pipe[2];
+
 
 signals:
 	void readablePortsChanged( void );
