@@ -1,7 +1,7 @@
 /*
  * midi_client.h - base-class for MIDI-clients like ALSA-sequencer-client
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -44,17 +44,16 @@
 #include "midi.h"
 #include "midi_event_processor.h"
 #include "tab_widget.h"
-#include "engine.h"
 
 
 class midiPort;
 
 
 // base-class for all MIDI-clients
-class midiClient : public engineObject
+class midiClient
 {
 public:
-	midiClient( engine * _engine );
+	midiClient( void );
 	virtual ~midiClient();
 
 	// to be implemented by sub-classes
@@ -117,7 +116,11 @@ public:
 	public:
 		setupWidget( const QString & _caption, QWidget * _parent ) :
 			tabWidget( tabWidget::tr( "Settings for %1" ).arg(
-						_caption ).toUpper(), _parent )
+					tr( _caption
+#ifndef QT3
+						.toAscii()
+#endif
+							) ).toUpper(), _parent )
 		{
 		}
 
@@ -127,6 +130,14 @@ public:
 
 		virtual void saveSettings( void ) = 0;
 
+
+	public slots:
+		virtual void show( void )
+		{
+			parentWidget()->show();
+			QWidget::show();
+		}
+
 	} ;
 
 
@@ -144,7 +155,7 @@ const Uint8 RAW_MIDI_PARSE_BUF_SIZE = 16;
 class midiClientRaw : public midiClient
 {
 public:
-	midiClientRaw( engine * _engine );
+	midiClientRaw( void );
 	virtual ~midiClientRaw();
 
 	// we are raw-clients for sure!
