@@ -1,7 +1,7 @@
 /*
  * tempo_sync_knob.h - adds bpm to ms conversion for knob class
  *
- * Copyright (c) 2005 Danny McRae <khjklujn/at/yahoo.com>
+ * Copyright (c) 2005-2007 Danny McRae <khjklujn/at/yahoo.com>
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -26,7 +26,7 @@
 #ifndef _TEMPO_SYNC_KNOB_H
 #define _TEMPO_SYNC_KNOB_H
 
-#ifdef QT4
+#ifndef QT3
 
 #include <QtGui/QPixmap>
 
@@ -37,10 +37,10 @@
 #endif
 
 #include "knob.h"
-#include "types.h"
 
 
 class QAction;
+class meterDialog;
 
 
 class tempoSyncKnob : public knob
@@ -56,15 +56,21 @@ public:
 		QUARTER_NOTE,
 		EIGHTH_NOTE,
 		SIXTEENTH_NOTE,
-		THIRTYSECOND_NOTE
+		THIRTYSECOND_NOTE,
+		CUSTOM
 	} ;
 
 
 	tempoSyncKnob( int _knob_num, QWidget * _parent, const QString & _name,
-					engine * _engine, track * _track,
-					float _scale = 1.0f );
+					track * _track, float _scale = 1.0f );
 	virtual ~tempoSyncKnob();
 
+	virtual void FASTCALL saveSettings( QDomDocument & _doc,
+						QDomElement & _this,
+						const QString & _name );
+	virtual void FASTCALL loadSettings( const QDomElement & _this,
+						const QString & _name );
+
 	tempoSyncMode getSyncMode( void );
 	void setSyncMode( tempoSyncMode _new_mode );
 
@@ -77,7 +83,6 @@ public:
 	const QPixmap & getSyncIcon( void );
 	void setSyncIcon( const QPixmap & _new_pix );
 
-
 signals:
 	void syncModeChanged( tempoSyncMode _new_mode );
 	void scaleChanged( float _new_scale );
@@ -98,7 +103,8 @@ protected:
 
 protected slots:
 	void calculateTempoSyncTime( bpm_t _bpm );
-
+	void updateCustom( int );
+	void showCustom( void );
 
 private:
 	tempoSyncMode m_tempoSyncMode;
@@ -107,6 +113,7 @@ private:
 	QString m_tempoSyncDescription;
 	
 	tempoSyncMode m_tempoLastSyncMode;
+	QPointer<meterDialog> m_custom;
 
 } ;
 
