@@ -2,7 +2,7 @@
  * track.h - declaration of classes concerning tracks -> neccessary for all
  *           track-like objects (beat/bassline, sample-track...)
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -18,8 +18,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -55,7 +55,6 @@
 #include "types.h"
 #include "midi_time.h"
 #include "rubberband.h"
-#include "engine.h"
 #include "journalling_object.h"
 
 
@@ -63,6 +62,7 @@ class QMenu;
 class QPushButton;
 
 class automationPattern;
+class bbTrack;
 class pixmapButton;
 class textFloat;
 class track;
@@ -265,9 +265,11 @@ protected:
 
 private slots:
 	void cloneTrack( void );
+	void disableAutomation( void );
+	void enableAutomation( void );
 	void removeTrack( void );
-	void removeTrackTimer( void );
 	void muteBtnRightClicked( void );
+	void updateMenu( void );
 
 
 private:
@@ -278,6 +280,9 @@ private:
 	QPushButton * m_trackOps;
 	pixmapButton * m_muteBtn;
 
+	bbTrack * currentBBTrack( void );
+	bool inBBEditor( void );
+
 } ;
 
 
@@ -340,6 +345,7 @@ public:
 	}
 	
 	virtual void repaint( void );
+	virtual void update( void );
 
 
 public slots:
@@ -391,12 +397,11 @@ class track : public journallingObject
 public:
 	enum trackTypes
 	{
-		CHANNEL_TRACK,
+		INSTRUMENT_TRACK,
 		BB_TRACK,
 		SAMPLE_TRACK,
 		EVENT_TRACK,
 		VIDEO_TRACK,
-		NULL_TRACK,
 		AUTOMATION_TRACK,
 		TOTAL_TRACK_TYPES
 	} ;
@@ -405,9 +410,9 @@ public:
 	virtual ~track();
 
 	static track * FASTCALL create( trackTypes _tt, trackContainer * _tc );
-	static track * FASTCALL create( const QDomElement & _this,
+	static void FASTCALL create( const QDomElement & _this,
 							trackContainer * _tc );
-	static track * FASTCALL clone( track * _track );
+	void FASTCALL clone( void );
 
 	tact length( void ) const;
 
@@ -426,8 +431,7 @@ public:
 	virtual trackTypes type( void ) const = 0;
 
 	virtual bool FASTCALL play( const midiTime & _start,
-						const f_cnt_t _start_frame,
-						const fpab_t _frames,
+						const fpp_t _frames,
 						const f_cnt_t _frame_base,
 						Sint16 _tco_num = -1 ) = 0;
 
@@ -461,7 +465,7 @@ public:
 		return( m_trackWidget );
 	}
 
-	inline trackContainer * getTrackContainer( void )
+	inline trackContainer * getTrackContainer( void ) const
 	{
 		return( m_trackContainer );
 	}
@@ -501,15 +505,14 @@ public:
 
 
 protected:
-	bool sendMidiTime( const midiTime & _time );
+	void sendMidiTime( const midiTime & _time );
 	QString m_name;
 
 
 private:
 	trackContainer * m_trackContainer;
 	trackWidget * m_trackWidget;
-	QPtrList<automationPattern> m_automation_patterns;
-	midiTime m_last_time_sent;
+	vlist<automationPattern *> m_automation_patterns;
 
 } ;
 
