@@ -1,7 +1,7 @@
 /*
  * vibed.cpp - combination of PluckedStringSynth and BitInvader
  *
- * Copyright (c) 2006 Danny McRae <khjklujn/at/yahoo/com>
+ * Copyright (c) 2006-2007 Danny McRae <khjklujn/at/yahoo/com>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -17,8 +17,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -45,20 +45,20 @@
 
 
 #include "vibed.h"
-#include "note_play_handle.h"
+#include "base64.h"
+#include "engine.h"
 #include "instrument_track.h"
-#include "templates.h"
-#include "buffer_allocator.h"
 #include "knob.h"
-#include "tooltip.h"
+#include "note_play_handle.h"
 #include "oscillator.h"
 #include "string_container.h"
-#include "base64.h"
+#include "templates.h"
+#include "tooltip.h"
+#include "volume.h"
+#include "volume_knob.h"
 
 #undef SINGLE_SOURCE_COMPILE
 #include "embed.cpp"
-#include "volume_knob.h"
-#include "volume.h"
 
 
 extern "C"
@@ -72,8 +72,9 @@ plugin::descriptor vibedstrings_plugin_descriptor =
 				"Vibrating string modeler" ),
 	"Danny McRae <khjklujn/at/yahoo/com>",
 	0x0100,
-	plugin::INSTRUMENT,
-	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) )
+	plugin::Instrument,
+	new QPixmap( PLUGIN_NAME::getIconPixmap( "logo" ) ),
+	NULL
 };
 
 }
@@ -95,8 +96,7 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 
 	for( Uint8 harm = 0; harm < 9; harm++ )
 	{
-		m_editor = new impulseEditor( this, 76, 21, eng(),
-							_channel_track );
+		m_editor = new impulseEditor( this, 76, 21, _channel_track );
 		m_editor->setOn( FALSE );
 		m_editor->hide();
 		m_editors.append( m_editor );
@@ -118,8 +118,8 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 
 "The 'N' button will normalize the waveform.") );
 
-		m_volumeKnob = new volumeKnob( knobBright_26, this, 
-					tr( "Volume" ), eng(), _channel_track );
+		m_volumeKnob = new volumeKnob( knobBright_26, this,
+					tr( "Volume" ), _channel_track );
 		m_volumeKnob->setRange( MIN_VOLUME, MAX_VOLUME, 1.0f );
 		m_volumeKnob->setInitValue( DEFAULT_VOLUME );
 		m_volumeKnob->move( 103, 142 );
@@ -136,7 +136,7 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 	
 		m_stiffnessKnob = new knob( knobBright_26, this, 
 							tr( "String stiffness" ),
-							eng(), _channel_track );
+							_channel_track );
 		m_stiffnessKnob->setRange( 0.0f, 0.05f, 0.001f );
 		m_stiffnessKnob->setInitValue( 0.0f );
 		m_stiffnessKnob->move( 129, 142 );
@@ -157,7 +157,7 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 		
 		m_pickKnob = new knob( knobBright_26, this, 
 							tr( "Pick position" ),
-							eng(), _channel_track );
+							_channel_track );
 		m_pickKnob->setRange( 0.0f, 0.5f, 0.005f );
 		m_pickKnob->setInitValue( 0.0f );
 		m_pickKnob->move( 153, 142 );
@@ -175,7 +175,7 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 	
 		m_pickupKnob = new knob( knobBright_26, this, 
 							tr( "Pickup position" ),
-							eng(), _channel_track );
+							_channel_track );
 		m_pickupKnob->setRange( 0.0f, 0.5f, 0.005f );
 		m_pickupKnob->setInitValue( 0.05f );
 		m_pickupKnob->move( 177, 142 );
@@ -193,8 +193,8 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 "for the selected string.  The lower the setting, the closer the "
 "pickup is to the bridge." ) );
 
- 		m_panKnob = new knob( knobBright_26, this, 
- 					tr( "Pan" ), eng(), _channel_track );
+ 		m_panKnob = new knob( knobBright_26, this, tr( "Pan" ),
+							_channel_track );
 	 	m_panKnob->setRange( -1.0f, 1.0f, 0.01f );
  		m_panKnob->setInitValue( 0.0f );
 	 	m_panKnob->move( 105, 187 );
@@ -210,9 +210,8 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 "The Pan knob determines the location of the selected string in the stereo "
 "field." ) );
 		
-	 	m_detuneKnob = new knob( knobBright_26, this, 
-							tr( "Detune" ),
-		 					eng(), _channel_track );
+	 	m_detuneKnob = new knob( knobBright_26, this, tr( "Detune" ),
+		 					_channel_track );
  		m_detuneKnob->setRange( -0.1f, 0.1f, 0.001f );
  		m_detuneKnob->setInitValue( 0.0f );
  		m_detuneKnob->move( 150, 187 );
@@ -229,9 +228,8 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 "than zero will cause the string to sound flat.  Settings greater than zero "
 "will cause the string to sound sharp." ) );
 	
-		m_randomKnob = new knob( knobBright_26, this, 
-							tr( "Fuzziness" ),
-							eng(), _channel_track );
+		m_randomKnob = new knob( knobBright_26, this, tr( "Fuzziness" ),
+							_channel_track );
 		m_randomKnob->setRange( 0.0f, 0.75f, 0.01f );
 		m_randomKnob->setInitValue( 0.0f );
 		m_randomKnob->move( 194, 187 );
@@ -249,8 +247,8 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 "apparent during the attack, though it can also be used to make the string "
 "sound more 'metallic'.") );
 
-		m_lengthKnob = new knob( knobBright_26, this, 
-					tr( "Length" ), eng(), _channel_track );
+		m_lengthKnob = new knob( knobBright_26, this, tr( "Length" ),
+							_channel_track );
 		m_lengthKnob->setRange( 1, 16, 1 );
 		m_lengthKnob->setInitValue( 1 );
 		m_lengthKnob->move( 23, 193 );
@@ -268,7 +266,7 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 "will both ring longer and sound brighter, however, they will also eat up "
 "more CPU cycles." ) );
 
-		m_impulse = new ledCheckBox( "", this, tr( "Impulse" ), eng(),
+		m_impulse = new ledCheckBox( "", this, tr( "Impulse" ),
 							_channel_track );
 		m_impulse->move( 23, 94 );
 		m_impulse->setChecked( FALSE );
@@ -308,7 +306,6 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 			2,
 			21, 127,
 			this,
-			eng(),
 			NULL );
 		m_harmonic->hide();
 		m_harmonics.append( m_harmonic );
@@ -347,7 +344,6 @@ vibed::vibed( instrumentTrack * _channel_track ) :
 			0,
 			21, 39,
 			this,
-			eng(),
 			NULL );
 	connect( m_stringSelector, SIGNAL( nineButtonSelection( Uint8 ) ),
 			this, SLOT( showString( Uint8 ) ) );
@@ -484,7 +480,8 @@ void vibed::loadSettings( const QDomElement & _this )
 		name = "active" + QString::number( i );
 		m_editors[i]->setOn( _this.attribute( name ).toInt() );
 		
-		if( m_editors[i]->isOn() )
+		if( m_editors[i]->isOn() &&
+			_this.hasAttribute( "volume" + QString::number( i ) ) )
 		{
 			name = "volume" + QString::number( i );
 			m_volumeKnobs[i]->loadSettings( _this, name );
@@ -550,16 +547,13 @@ QString vibed::nodeName( void ) const
 
 
 
-void vibed::playNote( notePlayHandle * _n )
+void vibed::playNote( notePlayHandle * _n, bool )
 {
 	if ( _n->totalFramesPlayed() == 0 )
 	{
-		float freq = getInstrumentTrack()->frequency( _n );
-		
-		_n->m_pluginData = new stringContainer( 
-				freq,
-				eng()->getMixer()->sampleRate(),
-				m_sampleLength );
+		_n->m_pluginData = new stringContainer( _n->frequency(),
+					engine::getMixer()->sampleRate(),
+					m_sampleLength );
 		
 		for( Uint8 i = 0; i < 9; ++i )
 		{
@@ -582,13 +576,13 @@ void vibed::playNote( notePlayHandle * _n )
 		}
 	}
 
-	const fpab_t frames = eng()->getMixer()->framesPerAudioBuffer();
+	const fpp_t frames = _n->framesLeftForCurrentPeriod();
 	stringContainer * ps = static_cast<stringContainer *>(
 			 				_n->m_pluginData );
 	
-	sampleFrame * buf = bufferAllocator::alloc<sampleFrame>( frames );
+	sampleFrame * buf = new sampleFrame[frames];
 
-	for( fpab_t i = 0; i < frames; ++i )
+	for( fpp_t i = 0; i < frames; ++i )
 	{
 		buf[i][0] = 0.0f;
 		buf[i][1] = 0.0f;
@@ -597,14 +591,15 @@ void vibed::playNote( notePlayHandle * _n )
 		{
 			if( ps->exists( string ) )
 			{
+				// pan: 0 -> left, 1 -> right
 				const float pan = ( 
 					m_panKnobs[string]->value() + 1 ) /
 									2.0f;
 				const sample_t sample = ps->getStringSample(
 									s ) *
 					m_volumeKnobs[string]->value() / 100.0f;
-				buf[i][0] += pan * sample;
-				buf[i][1] += ( 1.0f - pan ) * sample;
+				buf[i][0] += ( 1.0f - pan ) * sample;
+				buf[i][1] += pan * sample;
 				s++;
 			}
 		}
@@ -612,7 +607,7 @@ void vibed::playNote( notePlayHandle * _n )
 
 	getInstrumentTrack()->processAudioBuffer( buf, frames, _n );
 	
-	bufferAllocator::free( buf );
+	delete[] buf;
 }
 
 
