@@ -1,9 +1,11 @@
+#ifndef SINGLE_SOURCE_COMPILE
+
 /*
  * audio_sample_recorder.cpp - device-class that implements recording
  *                             surround-audio-buffers into RAM, maybe later
  *                             also harddisk
  *
- * Copyright (c) 2004-2005 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,8 +21,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -28,14 +30,15 @@
 
 #include "audio_sample_recorder.h"
 #include "sample_buffer.h"
-#include "buffer_allocator.h"
 #include "debug.h"
 
 
 
-audioSampleRecorder::audioSampleRecorder( Uint32 _sample_rate, Uint32 _channels,
-							bool & _success_ful ) :
-	audioDevice( _sample_rate, _channels ),
+audioSampleRecorder::audioSampleRecorder( const sample_rate_t _sample_rate,
+						const ch_cnt_t _channels,
+							bool & _success_ful,
+							mixer * _mixer ) :
+	audioDevice( _sample_rate, _channels, _mixer ),
 	m_buffers()
 {
 	_success_ful = TRUE;
@@ -48,7 +51,7 @@ audioSampleRecorder::~audioSampleRecorder()
 {
 	while( !m_buffers.empty() )
 	{
-		bufferAllocator::free( m_buffers.front().first );
+		delete[] m_buffers.front().first;
 		m_buffers.erase( m_buffers.begin() );
 	}
 }
@@ -56,13 +59,13 @@ audioSampleRecorder::~audioSampleRecorder()
 
 
 
-Uint32 audioSampleRecorder::framesRecorded( void ) const
+f_cnt_t audioSampleRecorder::framesRecorded( void ) const
 {
-	Uint32 frames = 0;
-	for( bufferVector::const_iterator it = m_buffers.begin();
+	f_cnt_t frames = 0;
+	for( bufferList::const_iterator it = m_buffers.begin();
 						it != m_buffers.end(); ++it )
 	{
-		frames += it->second;
+		frames += ( *it ).second;
 	}
 	return( frames );
 }
@@ -71,38 +74,40 @@ Uint32 audioSampleRecorder::framesRecorded( void ) const
 
 
 void audioSampleRecorder::createSampleBuffer( sampleBuffer * * _sample_buf )
-									const
 {
-	Uint32 frames = framesRecorded();
+	const f_cnt_t frames = framesRecorded();
 	// create buffer to store all recorded buffers in
-	sampleFrame * data = bufferAllocator::alloc<sampleFrame>( frames );
+	sampleFrame * data = new sampleFrame[frames];
 	// make sure buffer is cleaned up properly at the end...
-	bufferAllocator::autoCleaner<sampleFrame> ac( data );
+	sampleFrame * data_ptr = data;
 
 #ifdef LMMS_DEBUG
 	assert( data != NULL );
 #endif
 	// now copy all buffers into big buffer
-	for( bufferVector::const_iterator it = m_buffers.begin();
+	for( bufferList::const_iterator it = m_buffers.begin();
 						it != m_buffers.end(); ++it )
 	{
-		memcpy( data, it->first, it->second * sizeof( sampleFrame ) );
-		data += it->second;
+		memcpy( data_ptr, ( *it ).first, ( *it ).second *
+							sizeof( sampleFrame ) );
+		data_ptr += ( *it ).second;
 	}
 	// create according sample-buffer out of big buffer
-	*_sample_buf = new sampleBuffer( ac.ptr(), frames );
+	*_sample_buf = new sampleBuffer( data, frames );
+	( *_sample_buf )->setSampleRate( sampleRate() );
+	delete[] data;
 }
 
 
 
 
-void audioSampleRecorder::writeBuffer( surroundSampleFrame * _ab,
-							Uint32 _frames, float )
+void audioSampleRecorder::writeBuffer( const surroundSampleFrame * _ab,
+					const fpp_t _frames, const float )
 {
-	sampleFrame * buf = bufferAllocator::alloc<sampleFrame>( _frames );
-	for( Uint32 frame = 0; frame < _frames; ++frame )
+	sampleFrame * buf = new sampleFrame[_frames];
+	for( fpp_t frame = 0; frame < _frames; ++frame )
 	{
-		for( Uint8 chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+		for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 		{
 			buf[frame][chnl] = _ab[frame][chnl];
 		}
@@ -111,3 +116,5 @@ void audioSampleRecorder::writeBuffer( surroundSampleFrame * _ab,
 }
 
 
+
+#endif
