@@ -4,7 +4,7 @@
  * envelope_and_lfo_widget.cpp - widget which is m_used by envelope/lfo/filter-
  *                               tab of channel-window
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -49,6 +49,7 @@
 #include "envelope_and_lfo_widget.h"
 #include "song_editor.h"
 #include "embed.h"
+#include "engine.h"
 #include "knob.h"
 #include "pixmap_button.h"
 #include "oscillator.h"
@@ -100,17 +101,14 @@ const int LFO_SHAPES_Y = LFO_GRAPH_Y + 50;
 QPixmap * envelopeAndLFOWidget::s_envGraph = NULL;
 QPixmap * envelopeAndLFOWidget::s_lfoGraph = NULL;
 
-QMap<engine *, vvector<envelopeAndLFOWidget *> >
-					envelopeAndLFOWidget::s_EaLWidgets;
+vvector<envelopeAndLFOWidget *> envelopeAndLFOWidget::s_EaLWidgets;
 
 
 
 envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 							QWidget * _parent,
-							engine * _engine,
 							track * _track ) :
 	QWidget( _parent ),
-	journallingObject( _engine ),
 #ifdef QT4
 	specialBgHandlingWidget( palette().color( backgroundRole() ) ),
 #else
@@ -123,9 +121,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	m_lfoFrame( 0 ),
 	m_lfoAmountIsZero( FALSE ),
 	m_lfoShapeData( NULL ),
-	m_userWave( eng() ),
-	m_lfoShape( SIN ),
-	m_busy( FALSE )
+	m_lfoShape( SIN )
 {
 	if( s_envGraph == NULL )
 	{
@@ -137,11 +133,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 		s_lfoGraph = new QPixmap( embed::getIconPixmap( "lfo_graph" ) );
 	}
 
-	s_EaLWidgets[eng()].push_back( this );
+	s_EaLWidgets.push_back( this );
 
 
 	m_predelayKnob = new knob( knobBright_26, this, tr( "Predelay-time" ),
-								eng(), _track );
+								_track );
 	m_predelayKnob->setLabel( tr( "DEL" ) );
 	m_predelayKnob->setRange( 0.0, 1.0, 0.001 );
 	m_predelayKnob->setInitValue( 0.0 );
@@ -159,7 +155,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_attackKnob = new knob( knobBright_26, this, tr( "Attack-time" ),
-								eng(), _track );
+								_track );
 	m_attackKnob->setLabel( tr( "ATT" ) );
 	m_attackKnob->setRange( 0.0, 1.0, 0.001 );
 	m_attackKnob->setInitValue( 0.0 );
@@ -178,8 +174,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 	connect( m_attackKnob, SIGNAL( valueChanged( float ) ), this,
 				SLOT( updateAfterKnobChange( float ) ) );
 
-	m_holdKnob = new knob( knobBright_26, this, tr( "Hold-time" ),
-								eng(), _track );
+	m_holdKnob = new knob( knobBright_26, this, tr( "Hold-time" ), _track );
 	m_holdKnob->setLabel( tr( "HOLD" ) );
 	m_holdKnob->setRange( 0.0, 1.0, 0.001 );
 	m_holdKnob->setInitValue( 0.5 );
@@ -198,7 +193,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_decayKnob = new knob( knobBright_26, this, tr( "Decay-time" ),
-								eng(), _track );
+								_track );
 	m_decayKnob->setLabel( tr( "DEC" ) );
 	m_decayKnob->setRange( 0.0, 1.0, 0.001 );
 	m_decayKnob->setInitValue( 0.5 );
@@ -218,7 +213,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_sustainKnob = new knob( knobBright_26, this, tr( "Sustain-level" ),
-								eng(), _track );
+								_track );
 	m_sustainKnob->setLabel( tr( "SUST" ) );
 	m_sustainKnob->setRange( 0.0, 1.0, 0.001 );
 	m_sustainKnob->setInitValue( 0.5 );
@@ -237,7 +232,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_releaseKnob = new knob( knobBright_26, this, tr( "Release-time" ),
-								eng(), _track );
+								_track );
 	m_releaseKnob->setLabel( tr( "REL" ) );
 	m_releaseKnob->setRange( 0.0, 1.0, 0.001 );
 	m_releaseKnob->setInitValue( 0.1 );
@@ -258,7 +253,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 	m_amountKnob = new knob( knobBright_26, this,
 						tr( "Modulation amount" ),
-						eng(), _track );
+						_track );
 	m_amountKnob->setLabel( tr( "AMT" ) );
 	m_amountKnob->setRange( -1.0, 1.0, 0.005 );
 	m_amountKnob->setInitValue( 0.0 );
@@ -280,7 +275,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 	m_lfoPredelayKnob = new knob( knobBright_26, this,
 						tr( "LFO-predelay-time" ),
-						eng(), _track );
+						_track );
 	m_lfoPredelayKnob->setLabel( tr( "DEL" ) );
 	m_lfoPredelayKnob->setRange( 0.0, 1.0, 0.001 );
 	m_lfoPredelayKnob->setInitValue( 0.0 );
@@ -299,7 +294,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 	m_lfoAttackKnob = new knob( knobBright_26, this,
 						tr( "LFO-attack-time" ),
-						eng(), _track );
+						_track );
 	m_lfoAttackKnob->setLabel( tr( "ATT" ) );
 	m_lfoAttackKnob->setRange( 0.0, 1.0, 0.001 );
 	m_lfoAttackKnob->setInitValue( 0.0 );
@@ -317,8 +312,8 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 	m_lfoSpeedKnob = new tempoSyncKnob( knobBright_26, this,
-						tr( "LFO-speed" ),
-						eng(), _track, 20000.0 );
+							tr( "LFO-speed" ),
+							_track, 20000.0 );
 	m_lfoSpeedKnob->setLabel( tr( "SPD" ) );
 	m_lfoSpeedKnob->setRange( 0.01, 1.0, 0.0001 );
 	m_lfoSpeedKnob->setInitValue( 0.1 );
@@ -337,7 +332,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 	m_lfoAmountKnob = new knob( knobBright_26, this,
 						tr( "LFO-modulation-amount" ),
-						eng(), _track );
+						_track );
 	m_lfoAmountKnob->setLabel( tr( "AMT" ) );
 	m_lfoAmountKnob->setRange( -1.0, 1.0, 0.005 );
 	m_lfoAmountKnob->setInitValue( 0.0 );
@@ -356,8 +351,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 				SLOT( updateAfterKnobChange( float ) ) );
 
 
-	pixmapButton * sin_lfo_btn = new pixmapButton( this, NULL, eng(),
-									NULL );
+	pixmapButton * sin_lfo_btn = new pixmapButton( this, NULL, NULL );
 	sin_lfo_btn->move( LFO_SHAPES_X, LFO_SHAPES_Y );
 	sin_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 							"sin_wave_active" ) );
@@ -371,8 +365,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 		tr( "Click here if you want a sine-wave for current "
 							"oscillator." ) );
 
-	pixmapButton * triangle_lfo_btn = new pixmapButton( this, NULL, eng(),
-									NULL );
+	pixmapButton * triangle_lfo_btn = new pixmapButton( this, NULL, NULL );
 	triangle_lfo_btn->move( LFO_SHAPES_X+15, LFO_SHAPES_Y );
 	triangle_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 						"triangle_wave_active" ) );
@@ -386,8 +379,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 		tr( "Click here if you want a triangle-wave for current "
 							"oscillator." ) );
 
-	pixmapButton * saw_lfo_btn = new pixmapButton( this, NULL, eng(),
-									NULL );
+	pixmapButton * saw_lfo_btn = new pixmapButton( this, NULL, NULL );
 	saw_lfo_btn->move( LFO_SHAPES_X+30, LFO_SHAPES_Y );
 	saw_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 							"saw_wave_active" ) );
@@ -401,8 +393,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 		tr( "Click here if you want a saw-wave for current "
 							"oscillator." ) );
 
-	pixmapButton * sqr_lfo_btn = new pixmapButton( this, NULL, eng(),
-									NULL );
+	pixmapButton * sqr_lfo_btn = new pixmapButton( this, NULL, NULL );
 	sqr_lfo_btn->move( LFO_SHAPES_X+45, LFO_SHAPES_Y );
 	sqr_lfo_btn->setActiveGraphic( embed::getIconPixmap(
 						"square_wave_active" ) );
@@ -416,7 +407,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 		tr( "Click here if you want a square-wave for current "
 							"oscillator." ) );
 
-	m_userLfoBtn = new pixmapButton( this, NULL, eng(), NULL );
+	m_userLfoBtn = new pixmapButton( this, NULL, NULL );
 	m_userLfoBtn->move( LFO_SHAPES_X+60, LFO_SHAPES_Y );
 	m_userLfoBtn->setActiveGraphic( embed::getIconPixmap(
 							"usr_wave_active" ) );
@@ -436,7 +427,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 	m_lfoWaveBtnGrp = new automatableButtonGroup( this,
 							tr( "LFO wave shape" ),
-							eng(), _track );
+							_track );
 	m_lfoWaveBtnGrp->addButton( sin_lfo_btn );
 	m_lfoWaveBtnGrp->addButton( triangle_lfo_btn );
 	m_lfoWaveBtnGrp->addButton( saw_lfo_btn );
@@ -448,7 +439,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 						SLOT( lfoWaveCh( int ) ) );
 
 	m_x100Cb = new ledCheckBox( tr( "FREQ x 100" ), this,
-					tr( "Freq x 100" ), eng(), _track );
+						tr( "Freq x 100" ), _track );
 	m_x100Cb->setFont( pointSize<6>( m_x100Cb->font() ) );
 	m_x100Cb->move( LFO_PREDELAY_KNOB_X, LFO_GRAPH_Y + 36 );
 #ifdef QT4
@@ -465,7 +456,7 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 
 	m_controlEnvAmountCb = new ledCheckBox( tr( "MODULATE ENV-AMOUNT" ),
 					this, tr( "Modulate Env-Amount" ),
-					eng(), _track );
+					_track );
 	m_controlEnvAmountCb->move( LFO_PREDELAY_KNOB_X, LFO_GRAPH_Y + 54 );
 	m_controlEnvAmountCb->setFont( pointSize<6>(
 					m_controlEnvAmountCb->font() ) );
@@ -486,9 +477,11 @@ envelopeAndLFOWidget::envelopeAndLFOWidget( float _value_for_zero_amount,
 #endif
 	setAcceptDrops( TRUE );
 
-	connect( eng()->getMixer(), SIGNAL( sampleRateChanged() ), this,
+	connect( engine::getMixer(), SIGNAL( sampleRateChanged() ), this,
 						SLOT( updateSampleVars() ) );
 
+	m_lfoShapeData =
+		new sample_t[engine::getMixer()->framesPerPeriod()];
 	updateSampleVars();
 }
 
@@ -501,7 +494,7 @@ envelopeAndLFOWidget::~envelopeAndLFOWidget()
 	delete[] m_rEnv;
 	delete[] m_lfoShapeData;
 
-	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets[eng()];
+	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets;
 	if( qFind( v.begin(), v.end(), this ) != v.end() )
 	{
 		v.erase( qFind( v.begin(), v.end(), this ) );
@@ -511,76 +504,136 @@ envelopeAndLFOWidget::~envelopeAndLFOWidget()
 
 
 
-void envelopeAndLFOWidget::triggerLFO( engine * _engine )
+inline sample_t envelopeAndLFOWidget::lfoShapeSample( fpp_t _frame_offset )
 {
-	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets[_engine];
+	f_cnt_t frame = ( m_lfoFrame + _frame_offset ) % m_lfoOscillationFrames;
+	const float phase = frame / static_cast<float>(
+						m_lfoOscillationFrames );
+	sample_t shape_sample;
+	switch( m_lfoShape  )
+	{
+		case TRIANGLE:
+			shape_sample = oscillator::triangleSample( phase );
+			break;
+		case SQUARE:
+			shape_sample = oscillator::squareSample( phase );
+			break;
+		case SAW:
+			shape_sample = oscillator::sawSample( phase );
+			break;
+		case USER:
+			shape_sample = m_userWave.userWaveSample( phase );
+			break;
+		case SIN:
+		default:
+			shape_sample = oscillator::sinSample( phase );
+			break;
+	}
+	return( shape_sample * m_lfoAmount );
+}
+
+
+
+
+void envelopeAndLFOWidget::updateLFOShapeData( void )
+{
+	const fpp_t frames = engine::getMixer()->framesPerPeriod();
+	for( fpp_t offset = 0; offset < frames; ++offset )
+	{
+		m_lfoShapeData[offset] = lfoShapeSample( offset );
+	}
+	m_bad_lfoShapeData = FALSE;
+}
+
+
+
+
+void envelopeAndLFOWidget::triggerLFO( void )
+{
+	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets;
 	for( vvector<envelopeAndLFOWidget *>::iterator it = v.begin();
 							it != v.end(); ++it )
 	{
 		( *it )->m_lfoFrame +=
-				_engine->getMixer()->framesPerAudioBuffer();
+				engine::getMixer()->framesPerPeriod();
+		( *it )->m_bad_lfoShapeData = TRUE;
 	}
 }
 
 
 
 
-void envelopeAndLFOWidget::resetLFO( engine * _engine )
+void envelopeAndLFOWidget::resetLFO( void )
 {
-	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets[_engine];
+	vvector<envelopeAndLFOWidget *> & v = s_EaLWidgets;
 	for( vvector<envelopeAndLFOWidget *>::iterator it = v.begin();
 							it != v.end(); ++it )
 	{
 		( *it )->m_lfoFrame = 0;
+		( *it )->m_bad_lfoShapeData = TRUE;
 	}
 }
 
 
 
 
-inline float FASTCALL envelopeAndLFOWidget::lfoLevel( f_cnt_t _frame,
-					const f_cnt_t _frame_offset ) const
+inline void FASTCALL envelopeAndLFOWidget::fillLFOLevel( float * _buf,
+							f_cnt_t _frame,
+							const fpp_t _frames )
 {
-	if( m_lfoAmountIsZero == FALSE && _frame > m_lfoPredelayFrames )
+	if( m_lfoAmountIsZero || _frame <= m_lfoPredelayFrames )
 	{
-#ifdef LMMS_DEBUG
-		assert( m_lfoShapeData != NULL );
-#endif
-		_frame -= m_lfoPredelayFrames;
-		if( _frame > m_lfoAttackFrames )
+		for( fpp_t offset = 0; offset < _frames; ++offset )
 		{
-			return( m_lfoShapeData[( m_lfoFrame + _frame_offset ) %
-						m_lfoOscillationFrames] );
+			*_buf++ = 0.0f;
 		}
-		return( m_lfoShapeData[( m_lfoFrame + _frame_offset ) %
-						m_lfoOscillationFrames] *
-						_frame / m_lfoAttackFrames );
+		return;
+	}
+	_frame -= m_lfoPredelayFrames;
+
+	if( m_bad_lfoShapeData )
+	{
+		updateLFOShapeData();
+	}
+
+	fpp_t offset = 0;
+	for( ; offset < _frames && _frame < m_lfoAttackFrames; ++offset,
+								++_frame )
+	{
+		*_buf++ = m_lfoShapeData[offset] * _frame / m_lfoAttackFrames;
+	}
+	for( ; offset < _frames; ++offset )
+	{
+		*_buf++ = m_lfoShapeData[offset];
 	}
-	return( 0.0f );
 }
 
 
 
 
-float FASTCALL envelopeAndLFOWidget::level( f_cnt_t _frame,
+void FASTCALL envelopeAndLFOWidget::fillLevel( float * _buf, f_cnt_t _frame,
 					const f_cnt_t _release_begin,
-					const f_cnt_t _frame_offset ) const
+					const fpp_t _frames )
 {
-	if( m_busy )
+	fillLFOLevel( _buf, _frame, _frames );
+
+	for( fpp_t offset = 0; offset < _frames; ++offset, ++_buf, ++_frame )
 	{
-		return( 0.0f );
-	}
-	const float lfo_level = lfoLevel( _frame, _frame_offset );
-	float env_level;
-	if( _frame < _release_begin && _frame < m_pahdFrames )
-	{
-		env_level = m_pahdEnv[_frame];
-	}
-	else if( _frame >= _release_begin )
-	{
-		if( ( _frame -= _release_begin ) < m_rFrames )
+		float env_level;
+		if( _frame < _release_begin )
 		{
-			env_level = m_rEnv[_frame] *
+			if( _frame < m_pahdFrames )
+			{
+				env_level = m_pahdEnv[_frame];
+			}
+			else
+			{
+				env_level = m_sustainLevel;
+			}
+		}
+		else if( ( _frame - _release_begin ) < m_rFrames )
+		{
+			env_level = m_rEnv[_frame - _release_begin] *
 				( ( _release_begin < m_pahdFrames ) ?
 				m_pahdEnv[_release_begin] : m_sustainLevel );
 		}
@@ -588,15 +641,12 @@ float FASTCALL envelopeAndLFOWidget::level( f_cnt_t _frame,
 		{
 			env_level = 0.0f;
 		}
+
+		// at this point, *_buf is LFO level
+		*_buf = m_controlEnvAmountCb->isChecked() ?
+			env_level * ( 0.5f + *_buf ) :
+			env_level + *_buf;
 	}
-	else
-	{
-		env_level = m_sustainLevel;
-	}
-	return( m_controlEnvAmountCb->isChecked() ?
-			env_level * ( 0.5f + lfo_level )
-		:
-			env_level + lfo_level );
 }
 
 
@@ -619,8 +669,6 @@ void envelopeAndLFOWidget::saveSettings( QDomDocument & _doc,
 	m_lfoAmountKnob->saveSettings( _doc, _parent, "lamt" );
 	m_x100Cb->saveSettings( _doc, _parent, "x100" );
 	m_controlEnvAmountCb->saveSettings( _doc, _parent, "ctlenvamt" );
-	_parent.setAttribute( "lfosyncmode",
-					( int ) m_lfoSpeedKnob->getSyncMode() );
 	_parent.setAttribute( "userwavefile", m_userWave.audioFile() );
 }
 
@@ -629,8 +677,6 @@ void envelopeAndLFOWidget::saveSettings( QDomDocument & _doc,
 
 void envelopeAndLFOWidget::loadSettings( const QDomElement & _this )
 {
-	m_busy = TRUE;
-
 	m_predelayKnob->loadSettings( _this, "pdel" );
 	m_attackKnob->loadSettings( _this, "att" );
 	m_holdKnob->loadSettings( _this, "hold" );
@@ -645,15 +691,18 @@ void envelopeAndLFOWidget::loadSettings( const QDomElement & _this )
 	m_lfoAmountKnob->loadSettings( _this, "lamt" );
 	m_x100Cb->loadSettings( _this, "x100" );
 	m_controlEnvAmountCb->loadSettings( _this, "ctlenvamt" );
-	m_lfoSpeedKnob->setSyncMode(
+	
+	// Keep compatibility with version 2.1 file format
+	if( _this.hasAttribute( "lfosyncmode" ) )
+	{
+		m_lfoSpeedKnob->setSyncMode(
 		( tempoSyncKnob::tempoSyncMode ) _this.attribute(
 						"lfosyncmode" ).toInt() );
+	}
+	
 	m_userWave.setAudioFile( _this.attribute( "userwavefile" ) );
 
-	m_busy = FALSE;
-
 	updateSampleVars();
-	update();
 }
 
 
@@ -734,8 +783,6 @@ void envelopeAndLFOWidget::dropEvent( QDropEvent * _de )
 
 void envelopeAndLFOWidget::paintEvent( QPaintEvent * )
 {
-	updateSampleVars();
-
 #ifdef QT4
 	QPainter p( this );
 	p.setRenderHint( QPainter::Antialiasing );
@@ -822,7 +869,7 @@ void envelopeAndLFOWidget::paintEvent( QPaintEvent * )
 	int graph_y_base = LFO_GRAPH_Y + 3 + LFO_GRAPH_H / 2;
 
 	const float frames_for_graph = SECS_PER_LFO_OSCILLATION *
-					eng()->getMixer()->sampleRate() / 10;
+					engine::getMixer()->sampleRate() / 10;
 
 	const float lfo_gray_amount = 1.0f - fabsf( m_lfoAmountKnob->value() );
 	p.setPen( QPen( QColor( static_cast<int>( 96 * lfo_gray_amount ),
@@ -910,176 +957,126 @@ void envelopeAndLFOWidget::paintEvent( QPaintEvent * )
 
 void envelopeAndLFOWidget::updateSampleVars( void )
 {
-	if( !m_busy )
-	{
-		m_busy = TRUE;
+	engine::getMixer()->lock();
 
-		const float frames_per_env_seg = SECS_PER_ENV_SEGMENT *
-						eng()->getMixer()->sampleRate();
-		const f_cnt_t predelay_frames = static_cast<f_cnt_t>(
+	const float frames_per_env_seg = SECS_PER_ENV_SEGMENT *
+					engine::getMixer()->sampleRate();
+	// TODO: Remove the expKnobVals, time should be linear
+	const f_cnt_t predelay_frames = static_cast<f_cnt_t>(
 							frames_per_env_seg *
 					expKnobVal( m_predelayKnob->value() ) );
 
-		const f_cnt_t attack_frames = static_cast<f_cnt_t>(
-							frames_per_env_seg *
+	const f_cnt_t attack_frames = static_cast<f_cnt_t>( frames_per_env_seg *
 					expKnobVal( m_attackKnob->value() ) );
 
-		const f_cnt_t hold_frames = static_cast<f_cnt_t>(
-							frames_per_env_seg *
+	const f_cnt_t hold_frames = static_cast<f_cnt_t>( frames_per_env_seg *
 					expKnobVal( m_holdKnob->value() ) );
 
-		const f_cnt_t decay_frames = static_cast<f_cnt_t>(
-							frames_per_env_seg *
+	const f_cnt_t decay_frames = static_cast<f_cnt_t>( frames_per_env_seg *
 					expKnobVal( m_decayKnob->value() *
 						m_sustainKnob->value() ) );
 
-		m_sustainLevel = 1.0f - m_sustainKnob->value();
-		m_amount = m_amountKnob->value();
-		if( m_amount >= 0 )
-		{
-			m_amountAdd = ( 1.0f - m_amount ) *
-							m_valueForZeroAmount;
-		}
-		else
-		{
-			m_amountAdd = m_valueForZeroAmount;
-		}
+	m_sustainLevel = 1.0f - m_sustainKnob->value();
+	m_amount = m_amountKnob->value();
+	if( m_amount >= 0 )
+	{
+		m_amountAdd = ( 1.0f - m_amount ) * m_valueForZeroAmount;
+	}
+	else
+	{
+		m_amountAdd = m_valueForZeroAmount;
+	}
 
-		m_pahdFrames = predelay_frames + attack_frames + hold_frames +
+	m_pahdFrames = predelay_frames + attack_frames + hold_frames +
 								decay_frames;
-		m_rFrames = static_cast<f_cnt_t>( frames_per_env_seg *
+	m_rFrames = static_cast<f_cnt_t>( frames_per_env_seg *
 					expKnobVal( m_releaseKnob->value() ) );
 
-		if( static_cast<int>( floorf( m_amount * 1000.0f ) ) == 0 )
-		{
-			//m_pahdFrames = 0;
-			m_rFrames = 0;
-		}
+	if( static_cast<int>( floorf( m_amount * 1000.0f ) ) == 0 )
+	{
+		//m_pahdFrames = 0;
+		m_rFrames = 0;
+	}
 
-		sample_t * new_pahd_env = new sample_t[m_pahdFrames];
-		sample_t * new_r_env = new sample_t[m_rFrames];
+	delete[] m_pahdEnv;
+	delete[] m_rEnv;
 
-		for( f_cnt_t i = 0; i < predelay_frames; ++i )
-		{
-			new_pahd_env[i] = m_amountAdd;
-		}
+	m_pahdEnv = new sample_t[m_pahdFrames];
+	m_rEnv = new sample_t[m_rFrames];
 
-		f_cnt_t add = predelay_frames;
+	for( f_cnt_t i = 0; i < predelay_frames; ++i )
+	{
+		m_pahdEnv[i] = m_amountAdd;
+	}
 
-		for( f_cnt_t i = 0; i < attack_frames; ++i )
-		{
-			new_pahd_env[add+i] = ( (float)i / attack_frames ) *
+	f_cnt_t add = predelay_frames;
+
+	for( f_cnt_t i = 0; i < attack_frames; ++i )
+	{
+		m_pahdEnv[add + i] = ( (float)i / attack_frames ) *
 							m_amount + m_amountAdd;
-		}
+	}
 
-		add += attack_frames;
-		for( f_cnt_t i = 0; i < hold_frames; ++i )
-		{
-			new_pahd_env[add+i] = m_amount + m_amountAdd;
-		}
+	add += attack_frames;
+	for( f_cnt_t i = 0; i < hold_frames; ++i )
+	{
+		m_pahdEnv[add + i] = m_amount + m_amountAdd;
+	}
 
-		add += hold_frames;
-		for( f_cnt_t i = 0; i < decay_frames; ++i )
-		{
-			new_pahd_env[add+i] = ( m_sustainLevel + ( 1.0f -
+	add += hold_frames;
+	for( f_cnt_t i = 0; i < decay_frames; ++i )
+	{
+		m_pahdEnv[add + i] = ( m_sustainLevel + ( 1.0f -
 						(float)i / decay_frames ) *
 						( 1.0f - m_sustainLevel ) ) *
 							m_amount + m_amountAdd;
-		}
+	}
 
-		delete[] m_pahdEnv;
-		delete[] m_rEnv;
-
-		m_pahdEnv = new_pahd_env;
-		m_rEnv = new_r_env;
-
-		for( f_cnt_t i = 0; i < m_rFrames; ++i )
-		{
-			new_r_env[i] = ( (float)( m_rFrames - i ) / m_rFrames
+	for( f_cnt_t i = 0; i < m_rFrames; ++i )
+	{
+		m_rEnv[i] = ( (float)( m_rFrames - i ) / m_rFrames
 					// * m_sustainLevel
 					 		) * m_amount;
-		}
+	}
 
-		// save this calculation in real-time-part
-		m_sustainLevel = m_sustainLevel * m_amount + m_amountAdd;
-		
+	// save this calculation in real-time-part
+	m_sustainLevel = m_sustainLevel * m_amount + m_amountAdd;
 
-		const float frames_per_lfo_oscillation =
-						SECS_PER_LFO_OSCILLATION *
-						eng()->getMixer()->sampleRate();
-		m_lfoPredelayFrames = static_cast<f_cnt_t>(
-						frames_per_lfo_oscillation *
+
+	const float frames_per_lfo_oscillation = SECS_PER_LFO_OSCILLATION *
+					engine::getMixer()->sampleRate();
+	m_lfoPredelayFrames = static_cast<f_cnt_t>( frames_per_lfo_oscillation *
 				expKnobVal( m_lfoPredelayKnob->value() ) );
-		m_lfoAttackFrames = static_cast<f_cnt_t>(
-						frames_per_lfo_oscillation *
+	m_lfoAttackFrames = static_cast<f_cnt_t>( frames_per_lfo_oscillation *
 				expKnobVal( m_lfoAttackKnob->value() ) );
-		m_lfoOscillationFrames = static_cast<f_cnt_t>(
+	m_lfoOscillationFrames = static_cast<f_cnt_t>(
 						frames_per_lfo_oscillation *
 						m_lfoSpeedKnob->value() );
-		if( m_x100Cb->isChecked() )
-		{
-			m_lfoOscillationFrames /= 100;
-		}
-		m_lfoAmount = m_lfoAmountKnob->value() * 0.5f;
-
-		m_used = TRUE;
-		if( static_cast<int>( floorf( m_lfoAmount * 1000.0f ) ) == 0 )
-		{
-			m_lfoAmountIsZero = TRUE;
-			if( static_cast<int>( floorf( m_amount * 1000.0f ) ) ==
-									0 )
-			{
-				m_used = FALSE;
-			}
-		}
-		else
-		{
-			m_lfoAmountIsZero = FALSE;
-		}
-
-		if( m_lfoAmountIsZero == FALSE )
-		{
-			delete[] m_lfoShapeData;
-			m_lfoShapeData = new sample_t[m_lfoOscillationFrames];
-			for( f_cnt_t frame = 0; frame < m_lfoOscillationFrames;
-								++frame )
-			{
-				const float phase = frame / static_cast<float>(
-						m_lfoOscillationFrames );
-				// hope, compiler optimizes well and places
-				// branches out of loop and generates one loop
-				// for each branch...
-				switch( m_lfoShape  )
-				{
-					case TRIANGLE:
-						m_lfoShapeData[frame] =
-				oscillator::triangleSample( phase );
-						break;
-
-					case SQUARE:
-						m_lfoShapeData[frame] =
-				oscillator::squareSample( phase );
-						break;
-
-					case SAW:
-						m_lfoShapeData[frame] =
-				oscillator::sawSample( phase );
-						break;
-					case USER:
-						m_lfoShapeData[frame] =
-				m_userWave.userWaveSample( phase );
-						break;
-					case SIN:
-					default:
-						m_lfoShapeData[frame] =
-				oscillator::sinSample( phase );
-						break;
-				}
-				m_lfoShapeData[frame] *= m_lfoAmount;
-			}
-		}
-		m_busy = FALSE;
+	if( m_x100Cb->isChecked() )
+	{
+		m_lfoOscillationFrames /= 100;
 	}
+	m_lfoAmount = m_lfoAmountKnob->value() * 0.5f;
+
+	m_used = TRUE;
+	if( static_cast<int>( floorf( m_lfoAmount * 1000.0f ) ) == 0 )
+	{
+		m_lfoAmountIsZero = TRUE;
+		if( static_cast<int>( floorf( m_amount * 1000.0f ) ) == 0 )
+		{
+			m_used = FALSE;
+		}
+	}
+	else
+	{
+		m_lfoAmountIsZero = FALSE;
+	}
+
+	m_bad_lfoShapeData = TRUE;
+
+	update();
+
+	engine::getMixer()->unlock();
 }
 
 
@@ -1087,7 +1084,7 @@ void envelopeAndLFOWidget::updateSampleVars( void )
 
 void envelopeAndLFOWidget::x100Toggled( bool )
 {
-	eng()->getSongEditor()->setModified();
+	engine::getSongEditor()->setModified();
 	updateSampleVars();
 }
 
@@ -1096,7 +1093,7 @@ void envelopeAndLFOWidget::x100Toggled( bool )
 
 void envelopeAndLFOWidget::updateAfterKnobChange( float )
 {
-	update();
+	updateSampleVars();
 }
 
 
@@ -1105,7 +1102,7 @@ void envelopeAndLFOWidget::updateAfterKnobChange( float )
 void envelopeAndLFOWidget::lfoWaveCh( int _val )
 {
 	m_lfoShape = static_cast<lfoShapes>( _val );
-	update();
+	updateSampleVars();
 }
 
 
@@ -1124,9 +1121,9 @@ void envelopeAndLFOWidget::lfoUserWaveCh( bool _on )
 		}
 		m_lfoShape = USER;
 	}
-	eng()->getSongEditor()->setModified();
+	engine::getSongEditor()->setModified();
 
-	update();
+	updateSampleVars();
 }
 
 
