@@ -4,7 +4,7 @@
  * envelope_tab_widget.cpp - widget for use in envelope/lfo/filter-tab of
  *                           instrument-track-window
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -41,16 +41,16 @@
 
 
 #include "envelope_tab_widget.h"
-#include "envelope_and_lfo_widget.h"
-#include "note_play_handle.h"
-#include "knob.h"
-#include "pixmap_button.h"
-#include "group_box.h"
-#include "tab_widget.h"
+#include "combobox.h"
 #include "embed.h"
+#include "engine.h"
+#include "envelope_and_lfo_widget.h"
+#include "group_box.h"
 #include "gui_templates.h"
 #include "instrument_track.h"
-#include "combobox.h"
+#include "knob.h"
+#include "note_play_handle.h"
+#include "tab_widget.h"
 
 
 
@@ -84,7 +84,7 @@ static const QString targetNames[envelopeTabWidget::TARGET_COUNT][2] =
 
 envelopeTabWidget::envelopeTabWidget( instrumentTrack * _instrument_track ) :
 	QWidget( _instrument_track->tabWidgetParent() ),
-	journallingObject( _instrument_track->eng() )
+	m_instrumentTrack( _instrument_track )
 {
 
 	m_targetsTabWidget = new tabWidget( tr( "TARGET" ), this );
@@ -121,7 +121,6 @@ envelopeTabWidget::envelopeTabWidget( instrumentTrack * _instrument_track ) :
 		m_envLFOWidgets[i] = new envelopeAndLFOWidget(
 							value_for_zero_amount,
 							m_targetsTabWidget,
-							eng(),
 							_instrument_track );
 		m_targetsTabWidget->addTab( m_envLFOWidgets[i],
 						tr( targetNames[i][0]
@@ -137,14 +136,14 @@ envelopeTabWidget::envelopeTabWidget( instrumentTrack * _instrument_track ) :
 	}
 	
 	
-	m_filterGroupBox = new groupBox( tr( "FILTER" ), this, eng(),
+	m_filterGroupBox = new groupBox( tr( "FILTER" ), this,
 							_instrument_track );
 	m_filterGroupBox->setGeometry( FILTER_GROUPBOX_X, FILTER_GROUPBOX_Y,
 						FILTER_GROUPBOX_WIDTH,
 						FILTER_GROUPBOX_HEIGHT );
 
 	m_filterComboBox = new comboBox( m_filterGroupBox, tr( "Filter type" ),
-						eng(), _instrument_track );
+							_instrument_track );
 	m_filterComboBox->setGeometry( 14, 22, 120, 22 );
 	m_filterComboBox->setFont( pointSize<8>( m_filterComboBox->font() ) );
 
@@ -178,7 +177,7 @@ envelopeTabWidget::envelopeTabWidget( instrumentTrack * _instrument_track ) :
 
 	m_filterCutKnob = new knob( knobBright_26, m_filterGroupBox,
 						tr( "cutoff-frequency" ),
-						eng(), _instrument_track );
+						_instrument_track );
 	m_filterCutKnob->setLabel( tr( "CUTOFF" ) );
 	m_filterCutKnob->setRange( 0.0, 14000.0, 1.0 );
 	m_filterCutKnob->move( 140, 18 );
@@ -198,10 +197,10 @@ envelopeTabWidget::envelopeTabWidget( instrumentTrack * _instrument_track ) :
 			"frequencies below cutoff-frequency and so on..." ) );
 
 	m_filterResKnob = new knob( knobBright_26, m_filterGroupBox,
-						tr( "Q/Resonance" ),
-						eng(), _instrument_track );
+							tr( "Q/Resonance" ),
+							_instrument_track );
 	m_filterResKnob->setLabel( tr( "Q/RESO" ) );
-	m_filterResKnob->setRange( 0.01, 10.0, 0.01 );
+	m_filterResKnob->setRange( basicFilters<>::minQ(), 10.0, 0.01 );
 	m_filterResKnob->move( 190, 18 );
 	m_filterResKnob->setInitValue( 0.5 );
 	m_filterResKnob->setHintText( tr( "Q/Resonance:" ) + " ", "" );
@@ -235,146 +234,103 @@ float FASTCALL envelopeTabWidget::volumeLevel( notePlayHandle * _n,
 
 	if( _n->released() == FALSE )
 	{
-		release_begin += eng()->getMixer()->framesPerAudioBuffer();
+		release_begin += engine::getMixer()->framesPerPeriod();
 	}
 
-	return( m_envLFOWidgets[VOLUME]->level( _frame, release_begin, 0 ) );
+	float volume_level;
+	m_envLFOWidgets[VOLUME]->fillLevel( &volume_level, _frame,
+							release_begin, 1 );
+
+	return( volume_level );
 }
 
 
 
 
 void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab,
-							const fpab_t _frames,
+							const fpp_t _frames,
 							notePlayHandle * _n )
 {
-	f_cnt_t total_frames = _n->totalFramesPlayed();
+	const f_cnt_t total_frames = _n->totalFramesPlayed();
 	f_cnt_t release_begin = total_frames - _n->releaseFramesDone() +
 						_n->framesBeforeRelease();
 
 	if( _n->released() == FALSE )
 	{
-		release_begin += eng()->getMixer()->framesPerAudioBuffer();
+		release_begin += engine::getMixer()->framesPerPeriod();
 	}
 
 	// because of optimizations, there's special code for several cases:
-	// 	- volume-, cut- and res-lfo/envelope active
-	// 	- volume- and cut-lfo/envelope active
-	// 	- volume- and res-lfo/envelope active
+	// 	- cut- and res-lfo/envelope active
 	// 	- cut-lfo/envelope active
 	// 	- res-lfo/envelope active
-	// 	- volume-lfo/envelope active
 	//	- no lfo/envelope active but filter is used
-	// now there's a lot of similar code but I didn't found a way to
-	// generalize it yet... may be later we could do that
-	// by using preprocessor and macro-expansion... (like in oscillator.cpp)
 
 	// only use filter, if it is really needed
 
-	if( _n->m_filter == NULL )
-	{
-		_n->m_filter = new basicFilters<>(
-					eng()->getMixer()->sampleRate() );
-	}
-
 	if( m_filterGroupBox->isActive() )
 	{
 		int old_filter_cut = 0;
 		int old_filter_res = 0;
 
-		basicFilters<>::filterTypes filter =
-				basicFilters<>::getFilterType(
-					m_filterComboBox->value() );
+		if( _n->m_filter == NULL )
+		{
+			_n->m_filter = new basicFilters<>(
+					engine::getMixer()->sampleRate() );
+		}
+		_n->m_filter->setFilterType( m_filterComboBox->value() );
 
-		if( m_envLFOWidgets[VOLUME]->used() &&
-			m_envLFOWidgets[CUT]->used() &&
+		float * cut_buf = NULL;
+		float * res_buf = NULL;
+
+		if( m_envLFOWidgets[CUT]->used() )
+		{
+			cut_buf = new float[_frames];
+			m_envLFOWidgets[CUT]->fillLevel( cut_buf, total_frames,
+						release_begin, _frames );
+		}
+		if( m_envLFOWidgets[RES]->used() )
+		{
+			res_buf = new float[_frames];
+			m_envLFOWidgets[RES]->fillLevel( res_buf, total_frames,
+						release_begin, _frames );
+		}
+
+		if( m_envLFOWidgets[CUT]->used() &&
 			m_envLFOWidgets[RES]->used() )
 		{
-			for( fpab_t frame = 0; frame < _frames;
-						++frame, ++total_frames )
+			for( fpp_t frame = 0; frame < _frames; ++frame )
 			{
-				float new_cut_val = m_envLFOWidgets[CUT]->level( total_frames, release_begin, frame );
-				new_cut_val = envelopeAndLFOWidget::expKnobVal( new_cut_val ) * CUT_FREQ_MULTIPLIER +
+				float new_cut_val = envelopeAndLFOWidget::expKnobVal( cut_buf[frame] ) * CUT_FREQ_MULTIPLIER +
 						m_filterCutKnob->value();
 
 				float new_res_val = m_filterResKnob->value() + RES_MULTIPLIER *
-							m_envLFOWidgets[RES]->level( total_frames, release_begin, frame );
+							res_buf[frame];
 
 				if( static_cast<int>( new_cut_val ) != old_filter_cut ||
 					static_cast<int>( new_res_val*RES_PRECISION ) != old_filter_res )
 				{
-					_n->m_filter->calcFilterCoeffs( filter, new_cut_val, new_res_val );
+					_n->m_filter->calcFilterCoeffs( new_cut_val, new_res_val );
 					old_filter_cut = static_cast<int>( new_cut_val );
 					old_filter_res = static_cast<int>( new_res_val*RES_PRECISION );
 				}
 
-				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
-				vol_level = vol_level*vol_level;
-
 				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
-					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
+					_ab[frame][chnl] = _n->m_filter->update( _ab[frame][chnl], chnl );
 				}
 			}
 		}
-		else if( m_envLFOWidgets[VOLUME]->used() && m_envLFOWidgets[CUT]->used() )
-		{
-			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
-			{
-				float new_cut_val = m_envLFOWidgets[CUT]->level( total_frames, release_begin, frame );
-				new_cut_val = envelopeAndLFOWidget::expKnobVal( new_cut_val ) * CUT_FREQ_MULTIPLIER +
-						m_filterCutKnob->value();
-
-				if( static_cast<int>( new_cut_val ) != old_filter_cut )
-				{
-					_n->m_filter->calcFilterCoeffs( filter, new_cut_val, m_filterResKnob->value() );
-					old_filter_cut = static_cast<int>( new_cut_val );
-				}
-
-				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
-				vol_level = vol_level*vol_level;
-
-				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
-				{
-					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
-				}
-			}
-		}
-		else if( m_envLFOWidgets[VOLUME]->used() && m_envLFOWidgets[RES]->used() )
-		{
-
-			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
-			{
-				float new_res_val = m_filterResKnob->value() + RES_MULTIPLIER *
-							m_envLFOWidgets[RES]->level( total_frames, release_begin, frame );
-
-				if( static_cast<int>( new_res_val*RES_PRECISION ) != old_filter_res )
-				{
-					_n->m_filter->calcFilterCoeffs( filter, m_filterCutKnob->value(), new_res_val );
-					old_filter_res = static_cast<int>( new_res_val*RES_PRECISION );
-				}
-
-				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
-				vol_level = vol_level*vol_level;
-
-				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
-				{
-					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
-				}
-			}
-
-		}
 		else if( m_envLFOWidgets[CUT]->used() )
 		{
-			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpp_t frame = 0; frame < _frames; ++frame )
 			{
-				float new_cut_val = m_envLFOWidgets[CUT]->level( total_frames, release_begin, frame );
-				new_cut_val = envelopeAndLFOWidget::expKnobVal( new_cut_val ) * CUT_FREQ_MULTIPLIER +
+				float new_cut_val = envelopeAndLFOWidget::expKnobVal( cut_buf[frame] ) * CUT_FREQ_MULTIPLIER +
 						m_filterCutKnob->value();
 
 				if( static_cast<int>( new_cut_val ) != old_filter_cut )
 				{
-					_n->m_filter->calcFilterCoeffs( filter, new_cut_val, m_filterResKnob->value() );
+					_n->m_filter->calcFilterCoeffs( new_cut_val, m_filterResKnob->value() );
 					old_filter_cut = static_cast<int>( new_cut_val );
 				}
 
@@ -386,14 +342,14 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab,
 		}
 		else if( m_envLFOWidgets[RES]->used() )
 		{
-			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpp_t frame = 0; frame < _frames; ++frame )
 			{
 				float new_res_val = m_filterResKnob->value() + RES_MULTIPLIER *
-							m_envLFOWidgets[RES]->level( total_frames, release_begin, frame );
+							res_buf[frame];
 
 				if( static_cast<int>( new_res_val*RES_PRECISION ) != old_filter_res )
 				{
-					_n->m_filter->calcFilterCoeffs( filter, m_filterCutKnob->value(), new_res_val );
+					_n->m_filter->calcFilterCoeffs( m_filterCutKnob->value(), new_res_val );
 					old_filter_res = static_cast<int>( new_res_val*RES_PRECISION );
 				}
 
@@ -403,26 +359,11 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab,
 				}
 			}
 		}
-		else if( m_envLFOWidgets[VOLUME]->used() )
-		{
-			_n->m_filter->calcFilterCoeffs( filter, m_filterCutKnob->value(), m_filterResKnob->value() );
-
-			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
-			{
-				float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
-				vol_level = vol_level*vol_level;
-
-				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
-				{
-					_ab[frame][chnl] = vol_level * _n->m_filter->update( _ab[frame][chnl], chnl );
-				}
-			}
-		}
 		else
 		{
-			_n->m_filter->calcFilterCoeffs( filter, m_filterCutKnob->value(), m_filterResKnob->value() );
+			_n->m_filter->calcFilterCoeffs( m_filterCutKnob->value(), m_filterResKnob->value() );
 
-			for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
+			for( fpp_t frame = 0; frame < _frames; ++frame )
 			{
 				for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 				{
@@ -430,26 +371,36 @@ void envelopeTabWidget::processAudioBuffer( sampleFrame * _ab,
 				}
 			}
 		}
+
+		delete[] cut_buf;
+		delete[] res_buf;
 	}
-	else if( m_envLFOWidgets[VOLUME]->used() /*&& m_envLFOWidgets[PANNING]->used() == FALSE*/ )
+
+	if( m_envLFOWidgets[VOLUME]->used() )
 	{
-		// only use volume-envelope...
-		for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
+		float * vol_buf = new float[_frames];
+		m_envLFOWidgets[VOLUME]->fillLevel( vol_buf, total_frames,
+						release_begin, _frames );
+
+		for( fpp_t frame = 0; frame < _frames; ++frame )
 		{
-			float vol_level = m_envLFOWidgets[VOLUME]->level( total_frames, release_begin, frame );
-			vol_level = vol_level*vol_level;
-			for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
+			float vol_level = vol_buf[frame];
+			vol_level = vol_level * vol_level;
+			for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS;
+									++chnl )
 			{
 				_ab[frame][chnl] = vol_level * _ab[frame][chnl];
 			}
 		}
+		delete[] vol_buf;
 	}
+
 /*	else if( m_envLFOWidgets[VOLUME]->used() == FALSE && m_envLFOWidgets[PANNING]->used() )
 	{
 		// only use panning-envelope...
-		for( fpab_t frame = 0; frame < _frames; ++frame, ++total_frames )
+		for( fpp_t frame = 0; frame < _frames; ++frame )
 		{
-			float vol_level = m_envLFOWidgets[PANNING]->level( total_frames, release_begin, frame );
+			float vol_level = pan_buf[frame];
 			vol_level = vol_level*vol_level;
 			for( ch_cnt_t chnl = 0; chnl < DEFAULT_CHANNELS; ++chnl )
 			{
@@ -487,6 +438,13 @@ f_cnt_t envelopeTabWidget::releaseFrames( const bool _only_vol )
 {
 	f_cnt_t ret_val = m_envLFOWidgets[VOLUME]->used() ?
 					m_envLFOWidgets[VOLUME]->m_rFrames : 0;
+	if( m_instrumentTrack->getInstrument()->desiredReleaseFrames() >
+								ret_val )
+	{
+		ret_val = m_instrumentTrack->getInstrument()->
+							desiredReleaseFrames();
+	}
+
 	if( m_envLFOWidgets[VOLUME]->used() == FALSE )
 	{
 		for( int i = VOLUME+1; i < TARGET_COUNT; ++i )
@@ -509,7 +467,7 @@ void envelopeTabWidget::saveSettings( QDomDocument & _doc, QDomElement & _this )
 	m_filterComboBox->saveSettings( _doc, _this, "ftype" );
 	m_filterCutKnob->saveSettings( _doc, _this, "fcut" );
 	m_filterResKnob->saveSettings( _doc, _this, "fres" );
-	_this.setAttribute( "fwet", m_filterGroupBox->isActive() );
+	m_filterGroupBox->saveSettings( _doc, _this, "fwet" );
 
 	for( int i = 0; i < TARGET_COUNT; ++i )
 	{
@@ -527,8 +485,7 @@ void envelopeTabWidget::loadSettings( const QDomElement & _this )
 	m_filterComboBox->loadSettings( _this, "ftype" );
 	m_filterCutKnob->loadSettings( _this, "fcut" );
 	m_filterResKnob->loadSettings( _this, "fres" );
-/*	m_filterState->setChecked( _this.attribute( "fwet" ).toInt() );*/
-	m_filterGroupBox->setState( _this.attribute( "fwet" ).toInt() );
+	m_filterGroupBox->loadSettings( _this, "fwet" );
 
 	QDomNode node = _this.firstChild();
 	while( !node.isNull() )
