@@ -3,7 +3,7 @@
 /*
  * instrument.cpp - base-class for all instrument-plugins (synths, samplers etc)
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -28,14 +28,14 @@
 #include "instrument.h"
 #include "instrument_track.h"
 #include "dummy_instrument.h"
+#include "note_play_handle.h"
 
 
 instrument::instrument( instrumentTrack * _instrument_track,
 					const descriptor * _descriptor ) :
 	QWidget( _instrument_track->tabWidgetParent() ),
-	plugin( _descriptor, _instrument_track->eng() ),
-	m_instrumentTrack( _instrument_track ),
-	m_valid( TRUE )
+	plugin( _descriptor ),
+	m_instrumentTrack( _instrument_track )
 {
 	setFixedSize( 250, 250 );
 	m_instrumentTrack->setWindowIcon( *getDescriptor()->logo );
@@ -51,14 +51,14 @@ instrument::~instrument()
 
 
 
-void instrument::play( void )
+void instrument::play( bool )
 {
 }
 
 
 
 
-void instrument::playNote( notePlayHandle * )
+void instrument::playNote( notePlayHandle *, bool )
 {
 }
 
@@ -97,4 +97,34 @@ instrument * instrument::instantiate( const QString & _plugin_name,
 }
 
 
+
+
+bool instrument::isFromTrack( const track * _track ) const
+{
+	return( m_instrumentTrack == _track );
+}
+
+
+
+void instrument::applyRelease( sampleFrame * buf, const notePlayHandle * _n )
+{
+	const fpp_t frames = _n->framesLeftForCurrentPeriod();
+	const fpp_t fpp = engine::getMixer()->framesPerPeriod();
+	const f_cnt_t fl = _n->framesLeft();
+	if( fl <= desiredReleaseFrames()+fpp )
+	{
+		for( fpp_t f = fl > desiredReleaseFrames() ?
+				( tMax( fpp - desiredReleaseFrames(), 0 ) +
+					fl % fpp ) : 0; f < frames; ++f )
+		{
+			const float fac = (float)( fl-f-1 ) /
+							desiredReleaseFrames();
+			for( ch_cnt_t ch = 0; ch < DEFAULT_CHANNELS; ++ch )
+			{
+				buf[f][ch] *= fac;
+			}
+		}
+	}
+}
+
 #endif
