@@ -3,7 +3,7 @@
 /*
  * main_window.cpp - implementation of LMMS-main-window
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -67,6 +67,7 @@
 #include "song_editor.h"
 #include "piano_roll.h"
 #include "embed.h"
+#include "engine.h"
 #include "about_dialog.h"
 #include "file_browser.h"
 #include "plugin_browser.h"
@@ -74,12 +75,13 @@
 #include "config_mgr.h"
 #include "mixer.h"
 #include "project_notes.h"
-#include "buffer_allocator.h"
 #include "setup_dialog.h"
 #include "audio_dummy.h"
+#include "tool.h"
 #include "tool_button.h"
 #include "project_journal.h"
 #include "automation_editor.h"
+#include "templates.h"
 
 
 #if QT_VERSION >= 0x030100
@@ -89,15 +91,16 @@ extern int splash_alignment_flags;
 
 
 
-mainWindow::mainWindow( engine * _engine ) :
+mainWindow::mainWindow( void ) :
 	QMainWindow(
 #ifndef QT4
 			0 , NULL, WDestructiveClose
 #endif
 		),
-	engineObject( _engine ),
 	m_workspace( NULL ),
-	m_templatesMenu( NULL )
+	m_templatesMenu( NULL ),
+	m_recentlyOpenedProjectsMenu( NULL ),
+	m_tools_menu( NULL )
 {
 #ifdef QT4
 	setAttribute( Qt::WA_DeleteOnClose );
@@ -126,43 +129,44 @@ mainWindow::mainWindow( engine * _engine ) :
 	splitter->setChildrenCollapsible( FALSE );
 #endif
 
+	QString sample_filter;
+	vlist<QString> ext_keys = engine::sampleExtensions().keys();
+	for( vlist<QString>::iterator it = ext_keys.begin();
+						it != ext_keys.end(); ++it )
+	{
+		sample_filter += " *." + *it;
+	}
+
 	int id = 0;
 	QString wdir = configManager::inst()->workingDir();
-	side_bar->appendTab( new pluginBrowser( splitter, eng() ), ++id );
+	side_bar->appendTab( new pluginBrowser( splitter ), ++id );
 	side_bar->appendTab( new fileBrowser(
 			configManager::inst()->factoryProjectsDir() + "*" +
 				configManager::inst()->userProjectsDir(),
-						"*.mmp *.xml *.mid *.flp",
+					"*.mmp *.mmpz *.xml *.mid *.flp",
 							tr( "My projects" ),
 					embed::getIconPixmap( "project_file" ),
-							splitter, eng() ),
-									++id );
+							splitter ), ++id );
 	side_bar->appendTab( new fileBrowser(
 			configManager::inst()->factorySamplesDir() + "*" +
 					configManager::inst()->userSamplesDir(),
-					"*.wav *.ogg *.au"
-					"*.voc *.aif *.aiff *.flac *.raw",
-							tr( "My samples" ),
+					sample_filter, tr( "My samples" ),
 					embed::getIconPixmap( "sound_file" ),
-							splitter, eng() ),
-									++id );
+							splitter ), ++id );
 	side_bar->appendTab( new fileBrowser(
 			configManager::inst()->factoryPresetsDir() + "*" +
 					configManager::inst()->userPresetsDir(),
 						"*.cs.xml", tr( "My presets" ),
 					embed::getIconPixmap( "preset_file" ),
-							splitter, eng() ),
-									++id );
+							splitter ), ++id );
 	side_bar->appendTab( new fileBrowser( QDir::homePath(), "*",
 							tr( "My home" ),
 					embed::getIconPixmap( "home" ),
-							splitter, eng() ),
-									++id );
+							splitter ), ++id );
 	side_bar->appendTab( new fileBrowser( QDir::rootPath(), "*",
 							tr( "Root directory" ),
 					embed::getIconPixmap( "root" ),
-							splitter, eng() ),
-									++id );
+							splitter ), ++id );
 
 	if( no_mdi == FALSE )
 	{
@@ -215,21 +219,8 @@ mainWindow::mainWindow( engine * _engine ) :
 
 mainWindow::~mainWindow()
 {
-/*	// first make sure, there're no mixing/audio-device-threads any more
-	eng()->getMixer()->stopProcessing();
-
-	// destroy editors with all their children
-	delete eng()->getSongEditor();
-	delete eng()->getBBEditor();
-
-
-
-	// destroy mixer
-	delete eng()->getMixer();
-
-	// destroy config-manager (which automatically saves config-file)
-*/
-	eng()->close();
+	// destroy engine which will do further cleanups etc.
+	engine::destroy();
 }
 
 
@@ -271,8 +262,13 @@ void mainWindow::finalize( void )
 	m_templatesMenu = new QMenu( project_new );
 	connect( m_templatesMenu, SIGNAL( aboutToShow( void ) ),
 				this, SLOT( fillTemplatesMenu( void ) ) );
+#ifdef QT4
+	connect( m_templatesMenu, SIGNAL( triggered( QAction * ) ),
+		this, SLOT( createNewProjectFromTemplate( QAction * ) ) );
+#else
 	connect( m_templatesMenu, SIGNAL( activated( int ) ),
 			this, SLOT( createNewProjectFromTemplate( int ) ) );
+#endif
 	project_new->setMenu( m_templatesMenu );
 #ifdef QT4
 	project_new->setPopupMode( toolButton::MenuButtonPopup );
@@ -297,7 +293,7 @@ void mainWindow::finalize( void )
 	toolButton * project_export = new toolButton( 
 				embed::getIconPixmap( "project_export" ),
 					tr( "Export current project" ),
-					eng()->getSongEditor(),
+					engine::getSongEditor(),
 							SLOT( exportProject() ),
 								m_toolBar );
 
@@ -429,6 +425,10 @@ void mainWindow::finalize( void )
 	m_toolBarLayout->setColumnStretch( 100, 1 );
 
 
+	m_recentlyOpenedProjectsMenu = new QMenu( NULL );
+	connect( m_recentlyOpenedProjectsMenu, SIGNAL( activated( int ) ),
+			this, SLOT( openRecentlyOpenedProject( int ) ) );
+	updateRecentlyOpenedProjectsMenu();
 
 	// project-popup-menu
 	QMenu * project_menu = new QMenu( this );
@@ -447,6 +447,10 @@ void mainWindow::finalize( void )
 					this, SLOT( openProject() ),
 					Qt::CTRL + Qt::Key_O );	
 
+	project_menu->addAction( embed::getIconPixmap( "project_open" ),
+					tr( "Recently opened projects" ),
+					m_recentlyOpenedProjectsMenu );
+
 	project_menu->addAction( embed::getIconPixmap( "project_save" ),
 					tr( "&Save" ),
 					this, SLOT( saveProject() ),
@@ -462,17 +466,12 @@ void mainWindow::finalize( void )
 	project_menu->insertSeparator();
 #endif
 	project_menu->addAction( /*embed::getIconPixmap( "project_import" ),*/
-					tr( "Import file" ),
-					eng()->getSongEditor(),
+					tr( "Import..." ),
+					engine::getSongEditor(),
 					SLOT( importProject() ) );
-#ifdef QT4
-	project_menu->addSeparator();
-#else
-	project_menu->insertSeparator();
-#endif
 	project_menu->addAction( embed::getIconPixmap( "project_export" ),
-					tr( "E&xport" ),
-					eng()->getSongEditor(),
+					tr( "E&xport..." ),
+					engine::getSongEditor(),
 					SLOT( exportProject() ),
 					Qt::CTRL + Qt::Key_E );
 #ifdef QT4
@@ -500,7 +499,7 @@ void mainWindow::finalize( void )
 					this, SLOT( redo() ),
 					Qt::CTRL + Qt::Key_R );
 
-	
+
 	QMenu * settings_menu = new QMenu( this );
 #ifdef QT4
 	menuBar()->addMenu( settings_menu )->setText( tr( "&Settings" ) );
@@ -514,7 +513,39 @@ void mainWindow::finalize( void )
 					tr( "Show setup wizard" ),
 					configManager::inst(), SLOT( exec() ) );
 
-	
+
+	m_tools_menu = new QMenu( this );
+	vvector<plugin::descriptor> pluginDescriptors;
+	plugin::getDescriptorsOfAvailPlugins( pluginDescriptors );
+	for( vvector<plugin::descriptor>::iterator it =
+						pluginDescriptors.begin();
+					it != pluginDescriptors.end(); ++it )
+	{
+		if( it->type == plugin::Tool )
+		{
+			m_tools_menu->addAction( *it->logo, it->public_name );
+			m_tools.push_back( tool::instantiate( it->name,
+								this ) );
+		}
+	}
+#ifndef QT3
+	if( !m_tools_menu->isEmpty() )
+#else
+	if( m_tools_menu->count() )
+#endif
+	{
+#ifdef QT4
+		menuBar()->addMenu( m_tools_menu )->setText( tr( "&Tools" ) );
+		connect( m_tools_menu, SIGNAL( triggered( QAction * ) ),
+					this, SLOT( showTool( QAction * ) ) );
+#else
+		menuBar()->insertItem( tr( "&Tools" ), m_tools_menu );
+		connect( m_tools_menu, SIGNAL( activated( int ) ),
+						this, SLOT( showTool( int ) ) );
+#endif
+	}
+
+
 	// help-popup-menu
 	QMenu * help_menu = new QMenu( this );
 #ifdef QT4
@@ -522,32 +553,43 @@ void mainWindow::finalize( void )
 #else
 	menuBar()->insertItem( tr( "&Help" ), help_menu );
 #endif
-	help_menu->addAction( embed::getIconPixmap( "help" ), tr( "Help" ),
+	if( have_www_browser() )
+	{
+		help_menu->addAction( embed::getIconPixmap( "help" ),
+						tr( "Online help" ),
+						this, SLOT( browseHelp() ) );
+	}
+	else
+	{
+		help_menu->addAction( embed::getIconPixmap( "help" ),
+							tr( "Help" ),
 							this, SLOT( help() ) );
+	}
 	help_menu->addAction( embed::getIconPixmap( "whatsthis" ),
 					tr( "What's this?" ),
 					this, SLOT( enterWhatsThisMode() ) );
+
 #ifdef QT4
 	help_menu->addSeparator();
 #else
 	help_menu->insertSeparator();
 #endif
 	help_menu->addAction( embed::getIconPixmap( "icon" ), tr( "About" ),
-						this, SLOT( aboutLMMS() ) );
+			      this, SLOT( aboutLMMS() ) );
 
 	// setup-dialog opened before?
 	if( !configManager::inst()->value( "app", "configured" ).toInt() )
 	{
 		configManager::inst()->setValue( "app", "configured", "1" );
 		// no, so show it that user can setup everything
-		setupDialog sd( eng() );
+		setupDialog sd;
 		sd.exec();
 	}
 	// look whether mixer could use a audio-interface beside audioDummy
-	else if( eng()->getMixer()->audioDevName() == audioDummy::name() )
+	else if( engine::getMixer()->audioDevName() == audioDummy::name() )
 	{
 		// no, so we offer setup-dialog with audio-settings...
-		setupDialog sd( eng(), setupDialog::AUDIO_SETTINGS );
+		setupDialog sd( setupDialog::AUDIO_SETTINGS );
 		sd.exec();
 	}
 }
@@ -588,9 +630,9 @@ void mainWindow::addSpacingToToolBar( int _size )
 void mainWindow::resetWindowTitle( const QString & _add )
 {
 	QString title = _add;
-	if( _add == "" && eng()->getSongEditor()->projectFileName() != "" )
+	if( _add == "" && engine::getSongEditor()->projectFileName() != "" )
 	{
-		title = QFileInfo( eng()->getSongEditor()->projectFileName()
+		title = QFileInfo( engine::getSongEditor()->projectFileName()
 #ifdef QT4
 						).completeBaseName();
 #else
@@ -639,7 +681,8 @@ void mainWindow::saveWidgetState( QWidget * _w, QDomElement & _de )
 
 void mainWindow::restoreWidgetState( QWidget * _w, const QDomElement & _de )
 {
-	QRect r( _de.attribute( "x" ).toInt(), _de.attribute( "y" ).toInt(),
+	QRect r( tMax( 0, _de.attribute( "x" ).toInt() ),
+			tMax( 0, _de.attribute( "y" ).toInt() ),
 			_de.attribute( "width" ).toInt(),
 			_de.attribute( "height" ).toInt() );
 	if( !r.isNull() && _w->parentWidget() != NULL )
@@ -659,32 +702,51 @@ void mainWindow::restoreWidgetState( QWidget * _w, const QDomElement & _de )
 
 void mainWindow::createNewProject( void )
 {
-	if( eng()->getSongEditor()->mayChangeProject() == TRUE )
+	if( engine::getSongEditor()->mayChangeProject() == TRUE )
 	{
-		eng()->getSongEditor()->createNewProject();
+		engine::getSongEditor()->createNewProject();
 	}
 }
 
 
 
 
-void mainWindow::createNewProjectFromTemplate( int _idx )
-{
 #ifdef QT4
-	// TODO!!!
+void mainWindow::createNewProjectFromTemplate( QAction * _idx )
 #else
+void mainWindow::createNewProjectFromTemplate( int _idx )
+#endif
+{
 	if( m_templatesMenu != NULL &&
-			eng()->getSongEditor()->mayChangeProject() == TRUE )
+			engine::getSongEditor()->mayChangeProject() == TRUE )
 	{
+#ifdef QT4
+		QString dir_base = m_templatesMenu->actions().indexOf( _idx )
+#else
 		QString dir_base = m_templatesMenu->indexOf( _idx )
+#endif
 						>= m_custom_templates_count ?
 				configManager::inst()->factoryProjectsDir() :
 				configManager::inst()->userProjectsDir();
-		eng()->getSongEditor()->createNewProjectFromTemplate(
+		engine::getSongEditor()->createNewProjectFromTemplate(
 				dir_base + "templates/" +
+#ifdef QT4
+							_idx->text() + ".mpt" );
+#else
 				m_templatesMenu->text( _idx ) + ".mpt" );
-	}
 #endif
+	}
+}
+
+
+
+
+#ifdef QT4
+void mainWindow::createNewProjectFromTemplate( int _idx )
+#else
+void mainWindow::createNewProjectFromTemplate( QAction * _idx )
+#endif
+{
 }
 
 
@@ -692,14 +754,14 @@ void mainWindow::createNewProjectFromTemplate( int _idx )
 
 void mainWindow::openProject( void )
 {
-	if( eng()->getSongEditor()->mayChangeProject() == TRUE )
+	if( engine::getSongEditor()->mayChangeProject() == TRUE )
 	{
 #ifdef QT4
 		QFileDialog ofd( this, tr( "Open project" ), "",
-				tr( "MultiMedia Project (*.mmp *.xml)" ) );
+			tr( "MultiMedia Project (*.mmp *.mmpz *.xml)" ) );
 #else
 		QFileDialog ofd( QString::null,
-				tr( "MultiMedia Project (*.mmp *.xml)" ),
+			tr( "MultiMedia Project (*.mmp *.mmpz *.xml)" ),
 							this, "", TRUE );
 		ofd.setWindowTitle( tr( "Open project" ) );
 #endif
@@ -708,7 +770,7 @@ void mainWindow::openProject( void )
 		if( ofd.exec () == QDialog::Accepted &&
 						!ofd.selectedFiles().isEmpty() )
 		{
-			eng()->getSongEditor()->loadProject(
+			engine::getSongEditor()->loadProject(
 						ofd.selectedFiles()[0] );
 		}
 	}
@@ -717,15 +779,39 @@ void mainWindow::openProject( void )
 
 
 
+void mainWindow::updateRecentlyOpenedProjectsMenu( void )
+{
+	m_recentlyOpenedProjectsMenu->clear();
+	QStringList rup = configManager::inst()->recentlyOpenedProjects();
+	for( QStringList::iterator it = rup.begin(); it != rup.end(); ++it )
+	{
+		m_recentlyOpenedProjectsMenu->addAction( *it );
+	}
+}
+
+
+
+
+void mainWindow::openRecentlyOpenedProject( int _id )
+{
+	const QString & f = m_recentlyOpenedProjectsMenu->text( _id );
+	engine::getSongEditor()->loadProject( f );
+	configManager::inst()->addRecentlyOpenedProject( f );
+	updateRecentlyOpenedProjectsMenu();
+}
+
+
+
+
 bool mainWindow::saveProject( void )
 {
-	if( eng()->getSongEditor()->projectFileName() == "" )
+	if( engine::getSongEditor()->projectFileName() == "" )
 	{
 		return( saveProjectAs() );
 	}
 	else
 	{
-		eng()->getSongEditor()->saveProject();
+		engine::getSongEditor()->saveProject();
 	}
 	return( TRUE );
 }
@@ -737,17 +823,17 @@ bool mainWindow::saveProjectAs( void )
 {
 #ifdef QT4
 	QFileDialog sfd( this, tr( "Save project" ), "",
-				tr( "MultiMedia Project (*.mmp);;"
+			tr( "MultiMedia Project (*.mmp *.mmpz);;"
 				"MultiMedia Project Template (*.mpt)" ) );
 #else
 	QFileDialog sfd( QString::null,
-				tr( "MultiMedia Project (*.mmp);;"
+			tr( "MultiMedia Project (*.mmp *.mmpz);;"
 					"MultiMedia Project Template (*.mpt)" ),
 							this, "", TRUE );
 	sfd.setWindowTitle( tr( "Save project" ) );
 #endif
 	sfd.setFileMode( QFileDialog::AnyFile );
-	QString f = eng()->getSongEditor()->projectFileName();
+	QString f = engine::getSongEditor()->projectFileName();
 	if( f != "" )
 	{
 		sfd.selectFile( QFileInfo( f ).fileName() );
@@ -771,9 +857,10 @@ bool mainWindow::saveProjectAs( void )
  		)
 	{
 #ifdef QT4
-		eng()->getSongEditor()->saveProjectAs( sfd.selectedFiles()[0] );
+		engine::getSongEditor()->saveProjectAs(
+						sfd.selectedFiles()[0] );
 #else
-		eng()->getSongEditor()->saveProjectAs( sfd.selectedFile() );
+		engine::getSongEditor()->saveProjectAs( sfd.selectedFile() );
 #endif
 		return( TRUE );
 	}
@@ -785,7 +872,7 @@ bool mainWindow::saveProjectAs( void )
 
 void mainWindow::showSettingsDialog( void )
 {
-	setupDialog sd( eng() );
+	setupDialog sd;
 	sd.exec();
 }
 
@@ -803,12 +890,12 @@ void mainWindow::aboutLMMS( void )
 void mainWindow::help( void )
 {
 	QMessageBox::information( this, tr( "Help not available" ),
-					tr( "Currently there's no help "
-						"available in LMMS.\n"
-						"Please visit "
-						"http://wiki.mindrules.net "
-						"for documentation on LMMS." ),
-					QMessageBox::Ok );
+				  tr( "Currently there's no help "
+						  "available in LMMS.\n"
+						  "Please visit "
+						  "http://wiki.mindrules.net "
+						  "for documentation on LMMS." ),
+				  QMessageBox::Ok );
 }
 
 
@@ -816,16 +903,16 @@ void mainWindow::help( void )
 
 void mainWindow::toggleBBEditorWin( void )
 {
-	if( eng()->getBBEditor()->isHidden() == TRUE ||
+	if( engine::getBBEditor()->isHidden() == TRUE ||
 		( m_workspace != NULL &&
-		  	m_workspace->activeWindow() != eng()->getBBEditor() ) )
+		  	m_workspace->activeWindow() != engine::getBBEditor() ) )
 	{
-		eng()->getBBEditor()->show();
-		eng()->getBBEditor()->setFocus();
+		engine::getBBEditor()->show();
+		engine::getBBEditor()->setFocus();
 	}
 	else
 	{
-		eng()->getBBEditor()->hide();
+		engine::getBBEditor()->hide();
 	}
 }
 
@@ -834,16 +921,16 @@ void mainWindow::toggleBBEditorWin( void )
 
 void mainWindow::toggleSongEditorWin( void )
 {
-	if( eng()->getSongEditor()->isHidden() == TRUE ||
-		( m_workspace != NULL &&
-			m_workspace->activeWindow() != eng()->getSongEditor() ) )
+	if( engine::getSongEditor()->isHidden() == TRUE ||
+		( m_workspace != NULL && m_workspace->activeWindow()
+						!= engine::getSongEditor() ) )
 	{
-		eng()->getSongEditor()->show();
-		eng()->getSongEditor()->setFocus();
+		engine::getSongEditor()->show();
+		engine::getSongEditor()->setFocus();
 	}
 	else
 	{
-		eng()->getSongEditor()->hide();
+		engine::getSongEditor()->hide();
 	}
 }
 
@@ -852,16 +939,16 @@ void mainWindow::toggleSongEditorWin( void )
 
 void mainWindow::toggleProjectNotesWin( void )
 {
-	if( eng()->getProjectNotes()->isHidden() == TRUE ||
+	if( engine::getProjectNotes()->isHidden() == TRUE ||
 		( m_workspace != NULL && m_workspace->activeWindow() !=
-						eng()->getProjectNotes() ) )
+						engine::getProjectNotes() ) )
 	{
-		eng()->getProjectNotes()->show();
-		eng()->getProjectNotes()->setFocus();
+		engine::getProjectNotes()->show();
+		engine::getProjectNotes()->setFocus();
 	}
 	else
 	{
-		eng()->getProjectNotes()->hide();
+		engine::getProjectNotes()->hide();
 	}
 }
 
@@ -870,16 +957,16 @@ void mainWindow::toggleProjectNotesWin( void )
 
 void mainWindow::togglePianoRollWin( void )
 {
-	if( eng()->getPianoRoll()->isHidden() == TRUE ||
-		( m_workspace != NULL &&
-			m_workspace->activeWindow() != eng()->getPianoRoll() ) )
+	if( engine::getPianoRoll()->isHidden() == TRUE ||
+		( m_workspace != NULL && m_workspace->activeWindow()
+						!= engine::getPianoRoll() ) )
 	{
-		eng()->getPianoRoll()->show();
-		eng()->getPianoRoll()->setFocus();
+		engine::getPianoRoll()->show();
+		engine::getPianoRoll()->setFocus();
 	}
 	else
 	{
-		eng()->getPianoRoll()->hide();
+		engine::getPianoRoll()->hide();
 	}
 }
 
@@ -888,16 +975,16 @@ void mainWindow::togglePianoRollWin( void )
 
 void mainWindow::toggleAutomationEditorWin( void )
 {
-	if( eng()->getAutomationEditor()->isHidden() == TRUE ||
+	if( engine::getAutomationEditor()->isHidden() == TRUE ||
 		( m_workspace != NULL && m_workspace->activeWindow()
-					!= eng()->getAutomationEditor() ) )
+					!= engine::getAutomationEditor() ) )
 	{
-		eng()->getAutomationEditor()->show();
-		eng()->getAutomationEditor()->setFocus();
+		engine::getAutomationEditor()->show();
+		engine::getAutomationEditor()->setFocus();
 	}
 	else
 	{
-		eng()->getAutomationEditor()->hide();
+		engine::getAutomationEditor()->hide();
 	}
 }
 
@@ -906,7 +993,7 @@ void mainWindow::toggleAutomationEditorWin( void )
 
 void mainWindow::undo( void )
 {
-	eng()->getProjectJournal()->undo();
+	engine::getProjectJournal()->undo();
 }
 
 
@@ -914,7 +1001,7 @@ void mainWindow::undo( void )
 
 void mainWindow::redo( void )
 {
-	eng()->getProjectJournal()->redo();
+	engine::getProjectJournal()->redo();
 }
 
 
@@ -922,7 +1009,7 @@ void mainWindow::redo( void )
 
 void mainWindow::closeEvent( QCloseEvent * _ce )
 {
-	if( eng()->getSongEditor()->mayChangeProject() == TRUE )
+	if( engine::getSongEditor()->mayChangeProject() == TRUE )
 	{
 		_ce->accept();
 	}
@@ -1024,6 +1111,70 @@ void mainWindow::fillTemplatesMenu( void )
 
 
 
+#ifndef QT3
+void mainWindow::showTool( QAction * _idx )
+#else
+void mainWindow::showTool( int _idx )
+#endif
+{
+#ifndef QT3
+	tool * t = m_tools[m_tools_menu->actions().indexOf( _idx )];
+#else
+	tool * t = m_tools[m_tools_menu->indexOf( _idx )];
+#endif
+	t->show();
+	t->setFocus();
+}
+
+
+
+
+#ifndef QT3
+void mainWindow::showTool( int _idx )
+#else
+void mainWindow::showTool( QAction * _idx )
+#endif
+{
+}
+
+
+
+
+bool mainWindow::have_www_browser( void )
+{
+	int ret = system( "which x-www-browser > /dev/null" );
+	return( WIFEXITED( ret ) && WEXITSTATUS( ret ) == EXIT_SUCCESS );
+}
+
+
+
+
+void mainWindow::browseHelp( void )
+{
+	pid_t pid = fork();
+	if( pid == -1 )
+	{
+		perror( "fork" );
+	}
+	else if( pid == 0 )
+	{
+// TODO: use QDesktopService with Qt4
+		QString url = "http://lmms.sf.net/wiki/index.php?title=Main_Page";
+		execlp( "x-www-browser", "x-www-browser", url.
+#ifdef QT4
+							toAscii().constData(),
+#else
+							ascii(),
+#endif
+									NULL );
+		perror( "execlp" );
+		exit( EXIT_FAILURE );
+	}
+}
+
+
+
+
 #ifndef QT4
 #undef addSeparator
 #endif
