@@ -4,7 +4,7 @@
  * preset_preview_play_handle.cpp - implementation of class
  *                                  presetPreviewPlayHandle
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -41,13 +41,14 @@
 
 
 #include "preset_preview_play_handle.h"
-#include "note_play_handle.h"
-#include "instrument_track.h"
-#include "track_container.h"
-#include "mmp.h"
 #include "debug.h"
+#include "engine.h"
+#include "instrument_track.h"
 #include "midi_port.h"
+#include "mmp.h"
+#include "note_play_handle.h"
 #include "project_journal.h"
+#include "track_container.h"
 
 
 
@@ -55,15 +56,14 @@
 class previewTrackContainer : public trackContainer
 {
 public:
-	previewTrackContainer( engine * _engine ) :
-		trackContainer( _engine ),
+	previewTrackContainer( void ) :
 		m_previewInstrumentTrack( NULL ),
 		m_previewNote( NULL ),
 		m_dataMutex()
 	{
 		setJournalling( FALSE );
-		m_previewInstrumentTrack =  dynamic_cast<instrumentTrack *>(
-					track::create( track::CHANNEL_TRACK,
+		m_previewInstrumentTrack = dynamic_cast<instrumentTrack *>(
+					track::create( track::INSTRUMENT_TRACK,
 								this ) );
 		m_previewInstrumentTrack->setJournalling( FALSE );
 		hide();
@@ -119,58 +119,50 @@ private:
 } ;
 
 
-QMap<const engine *, previewTrackContainer *>
-					presetPreviewPlayHandle::s_previewTCs;
+previewTrackContainer * presetPreviewPlayHandle::s_previewTC;
 
 
 
 presetPreviewPlayHandle::presetPreviewPlayHandle(
-						const QString & _preset_file,
-						engine * _engine ) :
-	playHandle( PRESET_PREVIEW_PLAY_HANDLE ),
-	engineObject( _engine ),
+						const QString & _preset_file ) :
+	playHandle( PresetPreviewHandle ),
 	m_previewNote( NULL )
 {
-	if( s_previewTCs.contains( _engine ) == FALSE )
-	{
-		s_previewTCs[_engine] = new previewTrackContainer( eng() );
-	}
+	s_previewTC->lockData();
 
-	previewTC()->lockData();
-
-	if( previewTC()->previewNote() != NULL )
+	if( s_previewTC->previewNote() != NULL )
 	{
-		previewTC()->previewNote()->mute();
+		s_previewTC->previewNote()->mute();
 	}
 
 
-	const bool j = eng()->getProjectJournal()->isJournalling();
-	eng()->getProjectJournal()->setJournalling( FALSE );
+	const bool j = engine::getProjectJournal()->isJournalling();
+	engine::getProjectJournal()->setJournalling( FALSE );
 
 	multimediaProject mmp( _preset_file );
-	previewTC()->previewInstrumentTrack()->loadTrackSpecificSettings(
+	s_previewTC->previewInstrumentTrack()->loadTrackSpecificSettings(
 				mmp.content().firstChild().toElement() );
 	// preset also contains information about window-states etc. that's why
 	// here we have to make sure that the instrument-track-window is hidden
-	previewTC()->previewInstrumentTrack()->hide();
+	s_previewTC->previewInstrumentTrack()->hide();
 
 	// make sure, our preset-preview-track does not appear in any MIDI-
 	// devices list, so just disable receiving/sending MIDI-events at all
-	previewTC()->previewInstrumentTrack()->m_midiPort->setMode(
+	s_previewTC->previewInstrumentTrack()->m_midiPort->setMode(
 							midiPort::DUMMY );
 
 	// create note-play-handle for it
 	m_previewNote = new notePlayHandle(
-			previewTC()->previewInstrumentTrack(), 0,
-			valueRanges<f_cnt_t>::max,
-		note( NULL, 0, 0, static_cast<tones>( A ),
+			s_previewTC->previewInstrumentTrack(), 0,
+			valueRanges<f_cnt_t>::max / 2,
+		note( 0, 0, static_cast<tones>( A ),
 			static_cast<octaves>( DEFAULT_OCTAVE - 1 ), 100 ) );
 
 
-	previewTC()->setPreviewNote( m_previewNote );
+	s_previewTC->setPreviewNote( m_previewNote );
 
-	previewTC()->unlockData();
-	eng()->getProjectJournal()->setJournalling( j );
+	s_previewTC->unlockData();
+	engine::getProjectJournal()->setJournalling( j );
 }
 
 
@@ -178,23 +170,23 @@ presetPreviewPlayHandle::presetPreviewPlayHandle(
 
 presetPreviewPlayHandle::~presetPreviewPlayHandle()
 {
-	previewTC()->lockData();
+	s_previewTC->lockData();
 	// not muted by other preset-preview-handle?
 	if( m_previewNote->muted() == FALSE )
 	{
 		// then set according state
-		previewTC()->setPreviewNote( NULL );
+		s_previewTC->setPreviewNote( NULL );
 	}
 	delete m_previewNote;
-	previewTC()->unlockData();
+	s_previewTC->unlockData();
 }
 
 
 
 
-void presetPreviewPlayHandle::play( void )
+void presetPreviewPlayHandle::play( bool _try_parallelizing )
 {
-	m_previewNote->play();
+	m_previewNote->play( _try_parallelizing );
 }
 
 
@@ -208,33 +200,45 @@ bool presetPreviewPlayHandle::done( void ) const
 
 
 
-void presetPreviewPlayHandle::cleanUp( engine * _engine )
+bool presetPreviewPlayHandle::isFromTrack( const track * _track ) const
 {
-	if( s_previewTCs.contains( _engine ) == TRUE )
+	return( s_previewTC->previewInstrumentTrack() == _track );
+}
+
+
+
+
+void presetPreviewPlayHandle::init( void )
+{
+	if( !s_previewTC )
 	{
-		delete s_previewTCs[_engine];
-		s_previewTCs.remove( _engine );
+		s_previewTC = new previewTrackContainer;
 	}
 }
 
 
 
 
+void presetPreviewPlayHandle::cleanUp( void )
+{
+	delete s_previewTC;
+	s_previewTC = NULL;
+}
+
+
+
+
 constNotePlayHandleVector presetPreviewPlayHandle::nphsOfInstrumentTrack(
 						const instrumentTrack * _it )
 {
 	constNotePlayHandleVector cnphv;
-	if( s_previewTCs.contains( _it->eng() ) == TRUE )
+	s_previewTC->lockData();
+	if( s_previewTC->previewNote() != NULL &&
+		s_previewTC->previewNote()->getInstrumentTrack() == _it )
 	{
-		previewTrackContainer * tc = s_previewTCs[_it->eng()];
-		tc->lockData();
-		if( tc->previewNote() != NULL &&
-			tc->previewNote()->getInstrumentTrack() == _it )
-		{
-			cnphv.push_back( tc->previewNote() );
-		}
-		tc->unlockData();
+		cnphv.push_back( s_previewTC->previewNote() );
 	}
+	s_previewTC->unlockData();
 	return( cnphv );
 }
 
