@@ -3,7 +3,7 @@
 /*
  * setup_dialog.cpp - dialog for setting up LMMS
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -58,11 +58,14 @@
 #include "tab_widget.h"
 #include "gui_templates.h"
 #include "mixer.h"
+#include "project_journal.h"
 #include "config_mgr.h"
 #include "embed.h"
+#include "engine.h"
 #include "debug.h"
 #include "tooltip.h"
 #include "led_checkbox.h"
+#include "lcd_spinbox.h"
 
 
 // platform-specific audio-interface-classes
@@ -79,7 +82,6 @@
 #include "midi_dummy.h"
 
 
-
 inline void labelWidget( QWidget * _w, const QString & _txt )
 {
 	QLabel * title = new QLabel( _txt, _w );
@@ -98,10 +100,9 @@ inline void labelWidget( QWidget * _w, const QString & _txt )
 
 
 
-setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
-	QDialog(),
-	engineObject( _engine ),
-	m_bufferSize( eng()->getMixer()->framesPerAudioBuffer() ),
+setupDialog::setupDialog( configTabs _tab_to_open ) :
+	m_bufferSize( configManager::inst()->value( "mixer",
+					"framesperaudiobuffer" ).toInt() ),
 	m_disableToolTips( configManager::inst()->value( "tooltips",
 							"disabled" ).toInt() ),
 	m_classicalKnobUsability( configManager::inst()->value( "knobs",
@@ -113,20 +114,30 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 						"nomsgaftersetup" ).toInt() ),
 	m_displaydBV( configManager::inst()->value( "app", 
 		      				"displaydbv" ).toInt() ),
+	m_noMMPZ( configManager::inst()->value( "app",
+						"nommpz" ).toInt() ),
 	m_workingDir( configManager::inst()->workingDir() ),
 	m_vstDir( configManager::inst()->vstDir() ),
 	m_artworkDir( configManager::inst()->artworkDir() ),
 	m_flDir( configManager::inst()->flDir() ),
+	m_ladDir( configManager::inst()->ladspaDir() ),
+#ifdef HAVE_STK_H
+	m_stkDir( configManager::inst()->stkDir() ),
+#endif
 	m_disableChActInd( configManager::inst()->value( "ui",
 				"disablechannelactivityindicators" ).toInt() ),
 	m_manualChPiano( configManager::inst()->value( "ui",
-					"manualchannelpiano" ).toInt() )
+					"manualchannelpiano" ).toInt() ),
+	m_parLevel( configManager::inst()->value( "mixer",
+				"parallelizinglevel" ).toInt() )
 					
 {
 	setWindowIcon( embed::getIconPixmap( "setup_general" ) );
 	setWindowTitle( tr( "Setup LMMS" ) );
 	setModal( TRUE );
 
+	engine::getProjectJournal()->setJournalling( FALSE );
+
 	QVBoxLayout * vlayout = new QVBoxLayout( this );
 	vlayout->setSpacing( 0 );
 	vlayout->setMargin( 0 );
@@ -140,8 +151,11 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 	m_tabBar->setFixedWidth( 72 );
 
 	QWidget * ws = new QWidget( settings );
-	ws->setFixedSize( 360, 300 );
-
+#ifdef HAVE_STK_H
+	ws->setFixedSize( 360, 422 );
+#else
+	ws->setFixedSize( 360, 366 );
+#endif
 	QWidget * general = new QWidget( ws );
 	general->setFixedSize( 360, 240 );
 	QVBoxLayout * gen_layout = new QVBoxLayout( general );
@@ -159,10 +173,10 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 #else
 	m_bufSizeSlider->setMinimum( 1 );
 	m_bufSizeSlider->setMaximum( 256 );
-	m_bufSizeSlider->setLineStep( 4 );
+	m_bufSizeSlider->setLineStep( 8 );
 	m_bufSizeSlider->setTickmarks( QSlider::Below );
 #endif
-	m_bufSizeSlider->setPageStep( 4 );
+	m_bufSizeSlider->setPageStep( 8 );
 	m_bufSizeSlider->setTickInterval( 8 );
 	m_bufSizeSlider->setGeometry( 10, 16, 340, 18 );
 	m_bufSizeSlider->setValue( m_bufferSize / 64 );
@@ -189,12 +203,12 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 
 	tabWidget * misc_tw = new tabWidget( tr( "MISC" ), general );
-	misc_tw->setFixedHeight( 128 );
+	misc_tw->setFixedHeight( 150 );
 
 	ledCheckBox * disable_tooltips = new ledCheckBox(
 					tr( "Disable tooltips (no spurious "
 						"interrupts while playing)" ),
-						misc_tw, NULL, eng(), NULL );
+							misc_tw, NULL, NULL );
 	disable_tooltips->move( 10, 18 );
 	disable_tooltips->setChecked( m_disableToolTips );
 	connect( disable_tooltips, SIGNAL( toggled( bool ) ),
@@ -205,7 +219,7 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 					tr( "Classical knob usability (move "
 						"cursor around knob to change "
 						"value)" ),
-						misc_tw, NULL, eng(), NULL );
+							misc_tw, NULL, NULL );
 	classical_knob_usability->move( 10, 36 );
 	classical_knob_usability->setChecked( m_classicalKnobUsability );
 	connect( classical_knob_usability, SIGNAL( toggled( bool ) ),
@@ -214,7 +228,7 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 	ledCheckBox * gimp_like_windows = new ledCheckBox(
 					tr( "GIMP-like windows (no MDI)" ),
-						misc_tw, NULL, eng(), NULL );
+							misc_tw, NULL, NULL );
 	gimp_like_windows->move( 10, 54 );
 	gimp_like_windows->setChecked( m_gimpLikeWindows );
 	connect( gimp_like_windows, SIGNAL( toggled( bool ) ),
@@ -223,7 +237,7 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 	ledCheckBox * no_wizard = new ledCheckBox(
 				tr( "Do not show wizard after up-/downgrade" ),
-						misc_tw, NULL, eng(), NULL );
+							misc_tw, NULL, NULL );
 	no_wizard->move( 10, 72 );
 	no_wizard->setChecked( m_noWizard );
 	connect( no_wizard, SIGNAL( toggled( bool ) ),
@@ -232,22 +246,31 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 	ledCheckBox * no_msg = new ledCheckBox(
 					tr( "Do not show message after "
-						"closing this dialog" ),
-						misc_tw, NULL, eng(), NULL );
+							"closing this dialog" ),
+							misc_tw, NULL, NULL );
 	no_msg->move( 10, 90 );
 	no_msg->setChecked( m_noMsgAfterSetup );
 	connect( no_msg, SIGNAL( toggled( bool ) ),
 				this, SLOT( toggleNoMsgAfterSetup( bool ) ) );
 
 
-	ledCheckBox * dbv = new ledCheckBox(
-					tr( "Display volume as dbV " ),
-						misc_tw, NULL, eng(), NULL );
+	ledCheckBox * dbv = new ledCheckBox( tr( "Display volume as dbV " ),
+							misc_tw, NULL, NULL );
 	dbv->move( 10, 108 );
 	dbv->setChecked( m_displaydBV );
 	connect( dbv, SIGNAL( toggled( bool ) ),
 				this, SLOT( toggleDisplaydBV( bool ) ) );
 
+
+	ledCheckBox * no_mmpz = new ledCheckBox(
+			tr( "Do not compress project files per default" ),
+							misc_tw, NULL, NULL );
+	no_mmpz->move( 10, 126 );
+	no_mmpz->setChecked( m_noMMPZ );
+	connect( no_mmpz, SIGNAL( toggled( bool ) ),
+				this, SLOT( toggleNoMMPZ( bool ) ) );
+
+
 	gen_layout->addWidget( bufsize_tw );
 	gen_layout->addSpacing( 10 );
 	gen_layout->addWidget( misc_tw );
@@ -256,7 +279,11 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 
 	QWidget * directories = new QWidget( ws );
-	directories->setFixedSize( 360, 260 );
+#ifdef HAVE_STK_H
+	directories->setFixedSize( 360, 372 );
+#else
+	directories->setFixedSize( 360, 326 );
+#endif
 	QVBoxLayout * dir_layout = new QVBoxLayout( directories );
 	dir_layout->setSpacing( 0 );
 	dir_layout->setMargin( 0 );
@@ -337,7 +364,45 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 	fldir_select_btn->move( 320, 20 );
 	connect( fldir_select_btn, SIGNAL( clicked() ), this,
 						SLOT( openFLDir() ) );
+	// LADSPA-dir
+	tabWidget * lad_tw = new tabWidget( tr(
+			"LADSPA plugin directories" ).toUpper(),
+							directories );
+	lad_tw->setFixedHeight( 56 );
 
+	m_ladLineEdit = new QLineEdit( m_ladDir, lad_tw );
+	m_ladLineEdit->setGeometry( 10, 20, 300, 16 );
+	connect( m_ladLineEdit, SIGNAL( textChanged( const QString & ) ), this,
+		 		SLOT( setLADSPADir( const QString & ) ) );
+
+	QPushButton * laddir_select_btn = new QPushButton(
+				embed::getIconPixmap( "project_open", 16, 16 ),
+								"", lad_tw );
+	laddir_select_btn->setFixedSize( 24, 24 );
+	laddir_select_btn->move( 320, 20 );
+	connect( laddir_select_btn, SIGNAL( clicked() ), this,
+				 		SLOT( openLADSPADir() ) );
+
+#ifdef HAVE_STK_H
+	// STK-dir
+	tabWidget * stk_tw = new tabWidget( tr(
+			"STK rawwave directory" ).toUpper(),
+							directories );
+	stk_tw->setFixedHeight( 56 );
+
+	m_stkLineEdit = new QLineEdit( m_stkDir, stk_tw );
+	m_stkLineEdit->setGeometry( 10, 20, 300, 16 );
+	connect( m_stkLineEdit, SIGNAL( textChanged( const QString & ) ), this,
+		 SLOT( setSTKDir( const QString & ) ) );
+
+	QPushButton * stkdir_select_btn = new QPushButton(
+			embed::getIconPixmap( "project_open", 16, 16 ),
+								"", stk_tw );
+	stkdir_select_btn->setFixedSize( 24, 24 );
+	stkdir_select_btn->move( 320, 20 );
+	connect( stkdir_select_btn, SIGNAL( clicked() ), this,
+		 SLOT( openSTKDir() ) );
+#endif
 
 	dir_layout->addWidget( lmms_wd_tw );
 	dir_layout->addSpacing( 10 );
@@ -346,6 +411,12 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 	dir_layout->addWidget( artwork_tw );
 	dir_layout->addSpacing( 10 );
 	dir_layout->addWidget( fl_tw );
+	dir_layout->addSpacing( 10 );
+	dir_layout->addWidget( lad_tw );
+#ifdef HAVE_STK_H
+	dir_layout->addSpacing( 10 );
+	dir_layout->addWidget( stk_tw );
+#endif	
 	dir_layout->addStretch();
 
 
@@ -354,7 +425,7 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 
 	QWidget * performance = new QWidget( ws );
-	performance->setFixedSize( 360, 200 );
+	performance->setFixedSize( 360, 240 );
 	QVBoxLayout * perf_layout = new QVBoxLayout( performance );
 	perf_layout->setSpacing( 0 );
 	perf_layout->setMargin( 0 );
@@ -367,7 +438,7 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 	ledCheckBox * disable_ch_act_ind = new ledCheckBox(
 				tr( "Disable channel activity indicators" ),
-						ui_fx_tw, NULL, eng(), NULL );
+							ui_fx_tw, NULL, NULL );
 	disable_ch_act_ind->move( 10, 20 );
 	disable_ch_act_ind->setChecked( m_disableChActInd );
 	connect( disable_ch_act_ind, SIGNAL( toggled( bool ) ),
@@ -376,7 +447,7 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 	ledCheckBox * manual_ch_piano = new ledCheckBox(
 			tr( "Only press keys on channel-piano manually" ),
-						ui_fx_tw, NULL, eng(), NULL );
+							ui_fx_tw, NULL, NULL );
 	manual_ch_piano->move( 10, 40 );
 	manual_ch_piano->setChecked( m_manualChPiano );
 	connect( manual_ch_piano, SIGNAL( toggled( bool ) ),
@@ -384,8 +455,45 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 
 
+	tabWidget * smp_supp_tw = new tabWidget( tr( "SMP support" ).toUpper(),
+								performance );
+	smp_supp_tw->setFixedHeight( 200 );
+
+	QLabel * par_level_lbl = new QLabel( tr( "Parallelizing level" ),
+								smp_supp_tw );
+	par_level_lbl->move( 10, 15 );
+	par_level_lbl->setFixedSize( 120, 16 );
+	lcdSpinBox * par_level = new lcdSpinBox( 1, 16, 2, smp_supp_tw, NULL,
+									NULL );
+	par_level->setValue( m_parLevel );
+	connect( par_level, SIGNAL( valueChanged( int ) ),
+			this, SLOT( setParallelizingLevel( int ) ) );
+
+	par_level->move( 140, 20 );
+
+	QLabel * smp_help = new QLabel(
+		tr( "If you have a machine with more then one processor "
+			"(e.g. dual-core systems) you should use a "
+			"parallelizing-level above 1 which means that LMMS "
+			"will try to split up sound-processing into several "
+			"threads which should should be run on several cores "
+			"by the underlaying operating-system.\n"
+			"Please note that in some cases parallelizing won't "
+			"work with small buffer-sizes. If you experience "
+			"problems (i.e. lot of xruns), try to increase buffer-"
+			"size." ), smp_supp_tw );
+	smp_help->setFixedSize( performance->width() - 20, 140 );
+#ifndef QT3
+	smp_help->setWordWrap( TRUE );
+#else
+	smp_help->setAlignment( Qt::WordBreak );
+#endif
+	smp_help->move( 10, 55 );
+
 
 	perf_layout->addWidget( ui_fx_tw );
+	perf_layout->addSpacing( 15 );
+	perf_layout->addWidget( smp_supp_tw );
 	perf_layout->addStretch();
 
 
@@ -450,22 +558,32 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
 				it != m_audioIfaceSetupWidgets.end(); ++it )
 	{
-#ifdef QT4
-		it.value()->hide();
-		asw_layout->addWidget( it.value() );
-#else
-		it.data()->hide();
-		asw_layout->addWidget( it.data() );
+		m_audioIfaceNames[tr( it.key()
+#ifndef QT3
+						.toAscii()
 #endif
+								)] = it.key();
+	}
+	for( trMap::iterator it = m_audioIfaceNames.begin();
+				it != m_audioIfaceNames.end(); ++it )
+	{
+#ifdef QT4
+		QWidget * audioWidget = m_audioIfaceSetupWidgets[it.value()];
+#else
+		QWidget * audioWidget = m_audioIfaceSetupWidgets[it.data()];
+#endif
+		audioWidget->hide();
+		asw_layout->addWidget( audioWidget );
 		m_audioInterfaces->addItem( it.key() );
 	}
 #ifdef QT4
 	m_audioInterfaces->setCurrentIndex( m_audioInterfaces->findText(
-					eng()->getMixer()->audioDevName() ) );
+			tr( engine::getMixer()->audioDevName().toAscii() ) ) );
 #else
-	m_audioInterfaces->setCurrentText( eng()->getMixer()->audioDevName() );
+	m_audioInterfaces->setCurrentText(
+				tr( engine::getMixer()->audioDevName() ) );
 #endif
-	m_audioIfaceSetupWidgets[eng()->getMixer()->audioDevName()]->show();
+	m_audioIfaceSetupWidgets[engine::getMixer()->audioDevName()]->show();
 
 	connect( m_audioInterfaces, SIGNAL( activated( const QString & ) ),
 		this, SLOT( audioInterfaceChanged( const QString & ) ) );
@@ -530,23 +648,33 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 	for( mswMap::iterator it = m_midiIfaceSetupWidgets.begin();
 				it != m_midiIfaceSetupWidgets.end(); ++it )
 	{
-#ifdef QT4
-		it.value()->hide();
-		msw_layout->addWidget( it.value() );
-#else
-		msw_layout->addWidget( it.data() );
-		it.data()->hide();
+		m_midiIfaceNames[tr( it.key()
+#ifndef QT3
+						.toAscii()
 #endif
+								)] = it.key();
+	}
+	for( trMap::iterator it = m_midiIfaceNames.begin();
+				it != m_midiIfaceNames.end(); ++it )
+	{
+#ifdef QT4
+		QWidget * midiWidget = m_midiIfaceSetupWidgets[it.value()];
+#else
+		QWidget * midiWidget = m_midiIfaceSetupWidgets[it.data()];
+#endif
+		midiWidget->hide();
+		msw_layout->addWidget( midiWidget );
 		m_midiInterfaces->addItem( it.key() );
 	}
 
 #ifdef QT4
 	m_midiInterfaces->setCurrentIndex( m_midiInterfaces->findText(
-					eng()->getMixer()->midiClientName() ) );
+		tr( engine::getMixer()->midiClientName().toAscii() ) ) );
 #else
-	m_midiInterfaces->setCurrentText( eng()->getMixer()->midiClientName() );
+	m_midiInterfaces->setCurrentText(
+				tr( engine::getMixer()->midiClientName() ) );
 #endif
-	m_midiIfaceSetupWidgets[eng()->getMixer()->midiClientName()]->show();
+	m_midiIfaceSetupWidgets[engine::getMixer()->midiClientName()]->show();
 
 	connect( m_midiInterfaces, SIGNAL( activated( const QString & ) ),
 		this, SLOT( midiInterfaceChanged( const QString & ) ) );
@@ -622,6 +750,7 @@ setupDialog::setupDialog( engine * _engine, configTabs _tab_to_open ) :
 
 setupDialog::~setupDialog()
 {
+	engine::getProjectJournal()->setJournalling( TRUE );
 }
 
 
@@ -632,9 +761,9 @@ void setupDialog::accept( void )
 	configManager::inst()->setValue( "mixer", "framesperaudiobuffer",
 					QString::number( m_bufferSize ) );
 	configManager::inst()->setValue( "mixer", "audiodev",
-					m_audioInterfaces->currentText() );
+			m_audioIfaceNames[m_audioInterfaces->currentText()] );
 	configManager::inst()->setValue( "mixer", "mididev",
-					m_midiInterfaces->currentText() );
+			m_midiIfaceNames[m_midiInterfaces->currentText()] );
 	configManager::inst()->setValue( "tooltips", "disabled",
 					QString::number( m_disableToolTips ) );
 	configManager::inst()->setValue( "knobs", "classicalusability",
@@ -647,16 +776,24 @@ void setupDialog::accept( void )
 				QString::number( m_noMsgAfterSetup ) );
 	configManager::inst()->setValue( "app", "displaydbv",
 				QString::number( m_displaydBV ) );
+	configManager::inst()->setValue( "app", "nommpz",
+				QString::number( m_noMMPZ ) );
 	configManager::inst()->setValue( "ui",
 					"disablechannelactivityindicators",
 					QString::number( m_disableChActInd ) );
 	configManager::inst()->setValue( "ui", "manualchannelpiano",
 					QString::number( m_manualChPiano ) );
+	configManager::inst()->setValue( "mixer", "parallelizinglevel",
+						QString::number( m_parLevel ) );
 
 	configManager::inst()->setWorkingDir( m_workingDir );
 	configManager::inst()->setVSTDir( m_vstDir );
 	configManager::inst()->setArtworkDir( m_artworkDir );
 	configManager::inst()->setFLDir( m_flDir );
+	configManager::inst()->setLADSPADir( m_ladDir );
+#ifdef HAVE_STK_H
+	configManager::inst()->setSTKDir( m_stkDir );
+#endif	
 
 	// tell all audio-settings-widget to save their settings
 	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
@@ -697,6 +834,21 @@ void setupDialog::accept( void )
 
 void setupDialog::setBufferSize( int _value )
 {
+	const int step = DEFAULT_BUFFER_SIZE / 64;
+	if( _value > step && _value % step )
+	{
+		int mod_value = _value % step;
+		if( mod_value < step / 2 )
+		{
+			m_bufSizeSlider->setValue( _value - mod_value );
+		}
+		else
+		{
+			m_bufSizeSlider->setValue( _value + step - mod_value );
+		}
+		return;
+	}
+
 	if( m_bufSizeSlider->value() != _value )
 	{
 		m_bufSizeSlider->setValue( _value );
@@ -706,7 +858,7 @@ void setupDialog::setBufferSize( int _value )
 	m_bufSizeLbl->setText( tr( "Frames: %1\nLatency: %2 ms" ).arg(
 					m_bufferSize ).arg(
 						1000.0f * m_bufferSize /
-						eng()->getMixer()->sampleRate(),
+					engine::getMixer()->sampleRate(),
 						0, 'f', 1 ) );
 }
 
@@ -788,6 +940,14 @@ void setupDialog::toggleDisplaydBV( bool _enabled )
 
 
 
+void setupDialog::toggleNoMMPZ( bool _enabled )
+{
+	m_noMMPZ = _enabled;
+}
+
+
+
+
 void setupDialog::toggleDisableChActInd( bool _disabled )
 {
 	m_disableChActInd = _disabled;
@@ -803,6 +963,13 @@ void setupDialog::toggleManualChPiano( bool _enabled )
 
 
 
+void setupDialog::setParallelizingLevel( int _level )
+{
+	m_parLevel = _level;
+}
+
+
+
 
 void setupDialog::openWorkingDir( void )
 {
@@ -908,6 +1075,54 @@ void setupDialog::openFLDir( void )
 
 
 
+void setupDialog::openLADSPADir( void )
+{
+#ifdef QT4
+	QString new_dir = QFileDialog::getExistingDirectory( this,
+				tr( "Choose LADSPA plugin directory" ),
+							m_ladDir );
+#else
+	QString new_dir = QFileDialog::getExistingDirectory( m_ladDir, 0, 0,
+			tr( "Choose LADSPA plugin directory" ), TRUE );
+#endif
+	if( new_dir != QString::null )
+	{
+		if( m_ladLineEdit->text() == "" )
+		{
+			m_ladLineEdit->setText( new_dir );
+		}
+		else
+		{
+			m_ladLineEdit->setText( m_ladLineEdit->text() + ":" +
+								new_dir );
+		}
+	}
+}
+
+
+
+
+void setupDialog::openSTKDir( void )
+{
+#ifdef HAVE_STK_H
+#ifdef QT4
+	QString new_dir = QFileDialog::getExistingDirectory( this,
+				tr( "Choose STK rawwave directory" ),
+							m_ladDir );
+#else
+	QString new_dir = QFileDialog::getExistingDirectory( m_ladDir, 0, 0,
+			tr( "Choose STK rawwave directory" ), TRUE );
+#endif
+	if( new_dir != QString::null )
+	{
+		m_stkLineEdit->setText( new_dir );
+	}
+#endif
+}
+
+
+
+
 void setupDialog::setFLDir( const QString & _fd )
 {
 	m_flDir = _fd;
@@ -916,6 +1131,24 @@ void setupDialog::setFLDir( const QString & _fd )
 
 
 
+void setupDialog::setLADSPADir( const QString & _fd )
+{
+	m_ladDir = _fd;
+}
+
+
+
+
+void setupDialog::setSTKDir( const QString & _fd )
+{
+#ifdef HAVE_STK_H
+	m_stkDir = _fd;
+#endif
+}
+
+
+
+
 void setupDialog::audioInterfaceChanged( const QString & _iface )
 {
 	for( aswMap::iterator it = m_audioIfaceSetupWidgets.begin();
@@ -928,7 +1161,7 @@ void setupDialog::audioInterfaceChanged( const QString & _iface )
 #endif
 	}
 
-	m_audioIfaceSetupWidgets[_iface]->show();
+	m_audioIfaceSetupWidgets[m_audioIfaceNames[_iface]]->show();
 }
 
 
@@ -966,7 +1199,7 @@ void setupDialog::midiInterfaceChanged( const QString & _iface )
 #endif
 	}
 
-	m_midiIfaceSetupWidgets[_iface]->show();
+	m_midiIfaceSetupWidgets[m_midiIfaceNames[_iface]]->show();
 }
 
 
