@@ -3,7 +3,7 @@
 /*
  * song_editor.cpp - basic window for editing song
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -40,7 +40,6 @@
 #include <QtCore/QFile>
 #include <QtCore/QFileInfo>
 #include <QtGui/QAction>
-#include <QtGui/QApplication>
 #include <QtGui/QButtonGroup>
 #include <QtGui/QFileDialog>
 #include <QtGui/QKeyEvent>
@@ -51,7 +50,6 @@
 
 #else
 
-#include <qapplication.h>
 #include <qfile.h>
 #include <qmessagebox.h>
 #include <qfiledialog.h>
@@ -69,6 +67,7 @@
 
 
 #include "song_editor.h"
+#include "automatable_object_templates.h"
 #include "automatable_slider.h"
 #include "bb_editor.h"
 #include "rename_dialog.h"
@@ -79,6 +78,7 @@
 #include "instrument_track.h"
 #include "mmp.h"
 #include "midi_client.h"
+#include "note_play_handle.h"
 #include "timeline.h"
 #include "pattern.h"
 #include "piano_roll.h"
@@ -101,8 +101,7 @@
 
 
 
-songEditor::songEditor( engine * _engine ) :
-	trackContainer( _engine ),
+songEditor::songEditor( void ) :
 	m_fileName( "" ),
 	m_oldFileName( "" ),
 	m_exporting( FALSE ),
@@ -113,14 +112,13 @@ songEditor::songEditor( engine * _engine ) :
 	m_trackToPlay( NULL ),
 	m_patternToPlay( NULL ),
 	m_loopPattern( FALSE ),
-	m_scrollBack( FALSE ),
-	m_destroyed( FALSE )
+	m_scrollBack( FALSE )
 {
 	setWindowTitle( tr( "Song-Editor" ) );
 	setWindowIcon( embed::getIconPixmap( "songeditor" ) );
 
 	QWidget * w = ( parentWidget() != NULL ) ? parentWidget() : this;
-	if( eng()->getMainWindow()->workspace() != NULL )
+	if( engine::getMainWindow()->workspace() != NULL )
 	{
 		resize( 680, 300 );
 		w->move( 10, 10 );
@@ -146,7 +144,7 @@ songEditor::songEditor( engine * _engine ) :
 	timeLine * tl = new timeLine( TRACK_OP_WIDTH +
 					DEFAULT_SETTINGS_WIDGET_WIDTH, 32,
 					pixelsPerTact(), m_playPos[PLAY_SONG],
-					m_currentPosition, cw, eng() );
+					m_currentPosition, cw );
 	connect( this, SIGNAL( positionChanged( const midiTime & ) ),
 				m_playPos[PLAY_SONG].m_timeLine,
 			SLOT( updatePosition( const midiTime & ) ) );
@@ -156,15 +154,17 @@ songEditor::songEditor( engine * _engine ) :
 	m_automation_track = track::create( track::AUTOMATION_TRACK, this );
 
 	// add some essential widgets to global tool-bar 
-	QWidget * tb = eng()->getMainWindow()->toolBar();
+	QWidget * tb = engine::getMainWindow()->toolBar();
 
-	eng()->getMainWindow()->addSpacingToToolBar( 10 );
+	engine::getMainWindow()->addSpacingToToolBar( 10 );
 
 	m_bpmSpinBox = new lcdSpinBox( MIN_BPM, MAX_BPM, 3, tb, tr( "Tempo" ),
-						eng(), m_automation_track );
+							m_automation_track );
 	m_bpmSpinBox->setLabel( tr( "TEMPO/BPM" ) );
 	connect( m_bpmSpinBox, SIGNAL( valueChanged( int ) ), this,
 						SLOT( setTempo( int ) ) );
+	connect( m_bpmSpinBox, SIGNAL( manualChange() ), this,
+						SLOT( setModified() ) );
 	toolTip::add( m_bpmSpinBox, tr( "tempo of song" ) );
 
 #ifdef QT4
@@ -179,30 +179,34 @@ songEditor::songEditor( engine * _engine ) :
 			"should be played within a minute (or how many tacts "
 			"should be played within four minutes)." ) );
 
-	int col = eng()->getMainWindow()->addWidgetToToolBar( m_bpmSpinBox, 0 );
+	int col = engine::getMainWindow()->addWidgetToToolBar( m_bpmSpinBox,
+									0 );
 
 
 	toolButton * hq_btn = new toolButton( embed::getIconPixmap( "hq_mode" ),
 						tr( "High quality mode" ),
 						NULL, NULL, tb );
 	hq_btn->setCheckable( TRUE );
-	connect( hq_btn, SIGNAL( toggled( bool ) ), eng()->getMixer(),
+	connect( hq_btn, SIGNAL( toggled( bool ) ), engine::getMixer(),
 					SLOT( setHighQuality( bool ) ) );
 	hq_btn->setFixedWidth( 42 );
-	eng()->getMainWindow()->addWidgetToToolBar( hq_btn, 1, col );
+	engine::getMainWindow()->addWidgetToToolBar( hq_btn, 1, col );
 
 
 	toolButton * cp_btn = new toolButton( embed::getIconPixmap( "auto_limit" ),
 					      tr( "Auto limiter" ),
 					      NULL, NULL, tb );
 	cp_btn->setCheckable( TRUE );
-	connect( cp_btn, SIGNAL( toggled( bool ) ), eng()->getMixer(),
+	connect( cp_btn, SIGNAL( toggled( bool ) ), engine::getMixer(),
 		 SLOT( setClipScaling( bool ) ) );
 	cp_btn->setFixedWidth( 30 );
-	eng()->getMainWindow()->addWidgetToToolBar( cp_btn, 1, col + 1 );
+	engine::getMainWindow()->addWidgetToToolBar( cp_btn, 1, col + 1 );
 
 
-	eng()->getMainWindow()->addSpacingToToolBar( 10 );
+	engine::getMainWindow()->addSpacingToToolBar( 10 );
+
+	connect( engine::getMixer(), SIGNAL( sampleRateChanged() ), this,
+					SLOT( updateFramesPerTact64th() ) );
 
 
 
@@ -210,8 +214,8 @@ songEditor::songEditor( engine * _engine ) :
 	master_vol_lbl->setPixmap( embed::getIconPixmap( "master_volume" ) );
 
 	m_masterVolumeSlider = new automatableSlider( tb, tr( "Master volume" ),
-						eng(), m_automation_track );
-	m_masterVolumeSlider->setOrientation( Vertical );
+							m_automation_track );
+	m_masterVolumeSlider->setOrientation( Qt::Vertical );
 	m_masterVolumeSlider->setRange( 0, 200 );
 	m_masterVolumeSlider->setPageStep( 1 );
 	m_masterVolumeSlider->setInitValue( 100 );
@@ -237,19 +241,19 @@ songEditor::songEditor( engine * _engine ) :
 	m_mvsStatus->setTitle( tr( "Master volume" ) );
 	m_mvsStatus->setPixmap( embed::getIconPixmap( "master_volume" ) );
 
-	eng()->getMainWindow()->addWidgetToToolBar( master_vol_lbl );
-	eng()->getMainWindow()->addWidgetToToolBar( m_masterVolumeSlider );
+	engine::getMainWindow()->addWidgetToToolBar( master_vol_lbl );
+	engine::getMainWindow()->addWidgetToToolBar( m_masterVolumeSlider );
 
 
-	eng()->getMainWindow()->addSpacingToToolBar( 10 );
+	engine::getMainWindow()->addSpacingToToolBar( 10 );
 
 	QLabel * master_pitch_lbl = new QLabel( tb );
 	master_pitch_lbl->setPixmap( embed::getIconPixmap( "master_pitch" ) );
 	master_pitch_lbl->setFixedHeight( 64 );
 
 	m_masterPitchSlider = new automatableSlider( tb, tr( "Master pitch" ),
-						eng(), m_automation_track );
-	m_masterPitchSlider->setOrientation( Vertical );
+							m_automation_track );
+	m_masterPitchSlider->setOrientation( Qt::Vertical );
 	m_masterPitchSlider->setRange( -12, 12 );
 	m_masterPitchSlider->setPageStep( 1 );
 	m_masterPitchSlider->setInitValue( 0 );
@@ -274,10 +278,10 @@ songEditor::songEditor( engine * _engine ) :
 	m_mpsStatus->setTitle( tr( "Master pitch" ) );
 	m_mpsStatus->setPixmap( embed::getIconPixmap( "master_pitch" ) );
 
-	eng()->getMainWindow()->addWidgetToToolBar( master_pitch_lbl );
-	eng()->getMainWindow()->addWidgetToToolBar( m_masterPitchSlider );
+	engine::getMainWindow()->addWidgetToToolBar( master_pitch_lbl );
+	engine::getMainWindow()->addWidgetToToolBar( m_masterPitchSlider );
 
-	eng()->getMainWindow()->addSpacingToToolBar( 10 );
+	engine::getMainWindow()->addSpacingToToolBar( 10 );
 
 	// create widget for visualization- and cpu-load-widget
 	QWidget * vc_w = new QWidget( tb );
@@ -287,12 +291,12 @@ songEditor::songEditor( engine * _engine ) :
 
 	//vcw_layout->addStretch();
 	vcw_layout->addWidget( new visualizationWidget(
-			embed::getIconPixmap( "output_graph" ), vc_w, eng() ) );
+			embed::getIconPixmap( "output_graph" ), vc_w ) );
 
-	vcw_layout->addWidget( new cpuloadWidget( vc_w, eng() ) );
+	vcw_layout->addWidget( new cpuloadWidget( vc_w ) );
 	vcw_layout->addStretch();
 
-	eng()->getMainWindow()->addWidgetToToolBar( vc_w );
+	engine::getMainWindow()->addWidgetToToolBar( vc_w );
 
 
 	// create own toolbar
@@ -393,7 +397,7 @@ songEditor::songEditor( engine * _engine ) :
 	zoom_lbl->setPixmap( embed::getIconPixmap( "zoom" ) );
 
 	// setup zooming-stuff
-	m_zoomingComboBox = new comboBox( m_toolBar, NULL, eng(), NULL );
+	m_zoomingComboBox = new comboBox( m_toolBar, NULL, NULL );
 	m_zoomingComboBox->setFixedSize( 80, 22 );
 	m_zoomingComboBox->move( 580, 4 );
 	for( int i = 0; i < 7; ++i )
@@ -446,8 +450,8 @@ songEditor::songEditor( engine * _engine ) :
 
 songEditor::~songEditor()
 {
+	m_playing = FALSE;
 	delete m_automation_track;
-	m_destroyed = TRUE;
 }
 
 
@@ -510,13 +514,13 @@ void songEditor::resizeEvent( QResizeEvent * _re )
 void songEditor::keyPressEvent( QKeyEvent * _ke )
 {
 	if( /*_ke->modifiers() & Qt::ShiftModifier*/
-		eng()->getMainWindow()->isShiftPressed() == TRUE &&
+		engine::getMainWindow()->isShiftPressed() == TRUE &&
 						_ke->key() == Qt::Key_Insert )
 	{
 		insertBar();
 	}
 	else if(/* _ke->modifiers() & Qt::ShiftModifier &&*/
-			eng()->getMainWindow()->isShiftPressed() == TRUE &&
+			engine::getMainWindow()->isShiftPressed() == TRUE &&
 						_ke->key() == Qt::Key_Delete )
 	{
 		removeBar();
@@ -577,7 +581,7 @@ void songEditor::scrolled( int _new_pos )
 
 void songEditor::wheelEvent( QWheelEvent * _we )
 {
-	if( eng()->getMainWindow()->isCtrlPressed() == TRUE )
+	if( engine::getMainWindow()->isCtrlPressed() == TRUE )
 	{
 		if( _we->delta() > 0 )
 		{
@@ -599,7 +603,7 @@ void songEditor::wheelEvent( QWheelEvent * _we )
 		// and make sure, all TCO's are resized and relocated
 		realignTracks( TRUE );
 	} 
-	else if( eng()->getMainWindow()->isShiftPressed() == TRUE )
+	else if( engine::getMainWindow()->isShiftPressed() == TRUE )
 	{
 		m_leftRightScroll->setValue( m_leftRightScroll->value() -
 							_we->delta() / 30 );
@@ -628,8 +632,7 @@ void songEditor::masterVolumeChanged( int _new_val )
 			QPoint( m_masterVolumeSlider->width() + 2, -2 ) );
 		m_mvsStatus->setVisibilityTimeOut( 1000 );
 	}
-	eng()->getMixer()->setMasterGain( _new_val / 100.0f );
-	setModified();
+	engine::getMixer()->setMasterGain( _new_val / 100.0f );
 }
 
 
@@ -678,7 +681,6 @@ void songEditor::masterPitchChanged( int _new_val )
 			QPoint( m_masterPitchSlider->width() + 2, -2 ) );
 		m_mpsStatus->setVisibilityTimeOut( 1000 );
 	}
-	setModified();
 }
 
 
@@ -752,9 +754,19 @@ void songEditor::zoomingChanged( const QString & _zfac )
 
 void songEditor::setTempo( int _new_bpm )
 {
-	m_bpmSpinBox->setInitValue(
-				tLimit<bpm_t>( _new_bpm, MIN_BPM, MAX_BPM ) );
-	setModified();
+	playHandleVector & phv = engine::getMixer()->playHandles();
+	for( playHandleVector::iterator it = phv.begin(); it != phv.end();
+									++it )
+	{
+		notePlayHandle * nph = dynamic_cast<notePlayHandle *>( *it );
+		if( nph && !nph->released() )
+		{
+			nph->resize( _new_bpm );
+		}
+	}
+
+	m_bpmSpinBox->setInitValue( _new_bpm );
+	engine::updateFramesPerTact64th();
 	emit tempoChanged( _new_bpm );
 }
 
@@ -833,7 +845,7 @@ void songEditor::doActions( void )
 				updateTimeLinePosition();
 
 				// remove all note-play-handles that are active
-				eng()->getMixer()->clear();
+				engine::getMixer()->clear();
 
 				break;
 			}
@@ -921,7 +933,7 @@ void songEditor::processNextBuffer( void )
 			// at song-start we have to reset the LFOs
 			if( m_playPos[PLAY_SONG] == 0 )
 			{
-				envelopeAndLFOWidget::resetLFO( eng() );
+				envelopeAndLFOWidget::resetLFO();
 			}
 			break;
 
@@ -930,11 +942,10 @@ void songEditor::processNextBuffer( void )
 			break;
 
 		case PLAY_BB:
-			if( eng()->getBBEditor()->numOfBBs() > 0 )
+			if( engine::getBBEditor()->numOfBBs() > 0 )
 			{
-				tco_num = eng()->getBBEditor()->currentBB();
-				tv.push_back( bbTrack::findBBTrack( tco_num,
-								eng() ) );
+				tco_num = engine::getBBEditor()->currentBB();
+				tv.push_back( bbTrack::findBBTrack( tco_num ) );
 			}
 			break;
 
@@ -959,9 +970,11 @@ void songEditor::processNextBuffer( void )
 
 	// check for looping-mode and act if neccessary
 	timeLine * tl = m_playPos[m_playMode].m_timeLine;
-	if( tl != NULL && m_exporting == FALSE && tl->loopPointsEnabled() &&
-		!( m_playMode == PLAY_PATTERN &&
-		  		m_patternToPlay->freezing() == TRUE ) )
+	bool check_loop = tl != NULL && m_exporting == FALSE &&
+				tl->loopPointsEnabled() &&
+				!( m_playMode == PLAY_PATTERN &&
+					m_patternToPlay->freezing() == TRUE );
+	if( check_loop )
 	{
 		if( m_playPos[m_playMode] < tl->loopBegin() ||
 					m_playPos[m_playMode] >= tl->loopEnd() )
@@ -970,99 +983,122 @@ void songEditor::processNextBuffer( void )
 						tl->loopBegin().getTact() );
 			m_playPos[m_playMode].setTact64th(
 						tl->loopBegin().getTact64th() );
-			// force reset of current-frame-var afterwards
-			m_playPos[m_playMode].setCurrentFrame( 0 );
 		}
 	}
 
 	f_cnt_t total_frames_played = 0;
-	f_cnt_t frames_per_tact = static_cast<f_cnt_t>( framesPerTact() );
-	if( m_playPos[m_playMode].currentFrame() == 0 &&
-		m_playPos[m_playMode].getTact64th() > 0 )
-	{
-		m_playPos[m_playMode].setCurrentFrame(
-					m_playPos[m_playMode].getTact64th() *
-							frames_per_tact / 64 );
-	}
+	float frames_per_tact64th = engine::framesPerTact64th();
 
-	while( total_frames_played < eng()->getMixer()->framesPerAudioBuffer() )
+	while( total_frames_played
+				< engine::getMixer()->framesPerPeriod() )
 	{
-		f_cnt_t played_frames = eng()->getMixer()->framesPerAudioBuffer() -
-							total_frames_played;
+		f_cnt_t played_frames = engine::getMixer()
+				->framesPerPeriod() - total_frames_played;
 
-		// did we play a whole tact?
-		if( m_playPos[m_playMode].currentFrame() >= frames_per_tact )
+		float current_frame = m_playPos[m_playMode].currentFrame();
+		// did we play a 64th of a tact?
+		if( current_frame >= frames_per_tact64th )
 		{
-			// per default we just continue playing even if
-			// there's no more stuff to play (song-play-mode)
-			int max_tact = m_playPos[m_playMode].getTact() + 2;
-
-			// then decide whether to go over to next tact or to 
-			// loop back to first tact
-			if( m_playMode == PLAY_BB )
+			int tact64th = m_playPos[m_playMode].getTact64th()
+				+ (int)( current_frame / frames_per_tact64th );
+			// did we play a whole tact?
+			if( tact64th >= 64 )
 			{
-				max_tact =
-					eng()->getBBEditor()->lengthOfCurrentBB();
-			}
-			else if( m_playMode == PLAY_PATTERN &&
+				// per default we just continue playing even if
+				// there's no more stuff to play
+				// (song-play-mode)
+				int max_tact = m_playPos[m_playMode].getTact()
+									+ 2;
+
+				// then decide whether to go over to next tact
+				// or to loop back to first tact
+				if( m_playMode == PLAY_BB )
+				{
+					max_tact = engine::getBBEditor()
+							->lengthOfCurrentBB();
+				}
+				else if( m_playMode == PLAY_PATTERN &&
 					m_loopPattern == TRUE &&
 					tl != NULL &&
 					tl->loopPointsEnabled() == FALSE )
-			{
-				max_tact = m_patternToPlay->length().getTact();
+				{
+					max_tact = m_patternToPlay->length()
+								.getTact();
+				}
+				if( m_playPos[m_playMode].getTact() + 1
+								< max_tact )
+				{
+					// next tact
+					m_playPos[m_playMode].setTact(
+						m_playPos[m_playMode].getTact()
+									+ 1 );
+				}
+				else
+				{
+					// first tact
+					m_playPos[m_playMode].setTact( 0 );
+				}
 			}
-			if( m_playPos[m_playMode].getTact() + 1 < max_tact )
+			m_playPos[m_playMode].setTact64th( tact64th % 64 );
+
+			if( check_loop )
 			{
-				// next tact
-				m_playPos[m_playMode].setTact(
-					m_playPos[m_playMode].getTact() + 1 );
+				if( m_playPos[m_playMode] >= tl->loopEnd() )
+				{
+					m_playPos[m_playMode].setTact(
+						tl->loopBegin().getTact() );
+					m_playPos[m_playMode].setTact64th(
+						tl->loopBegin().getTact64th() );
+				}
 			}
-			else
-			{
-				// first tact
-				m_playPos[m_playMode].setTact( 0 );
-			}
-			m_playPos[m_playMode].setCurrentFrame( 0 );
+
+			current_frame = fmodf( current_frame,
+							frames_per_tact64th );
+			m_playPos[m_playMode].setCurrentFrame( current_frame );
 		}
-		// or do we have some samples left in this tact but this are 
+
+		f_cnt_t last_frames = (f_cnt_t)frames_per_tact64th
+						- (f_cnt_t)current_frame;
+		// skip last frame fraction
+		if( last_frames == 0 )
+		{
+			++total_frames_played;
+			m_playPos[m_playMode].setCurrentFrame( current_frame
+								+ 1.0f );
+			continue;
+		}
+		// do we have some samples left in this tact64th but this are
 		// less then samples we have to play?
-		else if( frames_per_tact - m_playPos[m_playMode].currentFrame()
-				< eng()->getMixer()->framesPerAudioBuffer() )
+		if( last_frames < played_frames )
 		{
-			// then set played_samples to remaining samples, the 
+			// then set played_samples to remaining samples, the
 			// rest will be played in next loop
-			played_frames = frames_per_tact -
-					m_playPos[m_playMode].currentFrame();
+			played_frames = last_frames;
 		}
 
-		if( m_playMode == PLAY_SONG )
+		if( (f_cnt_t)current_frame == 0 )
 		{
-			m_automation_track->play( m_playPos[m_playMode],
-					m_playPos[m_playMode].currentFrame(),
-					played_frames, total_frames_played,
-					tco_num );
-		}
+			if( m_playMode == PLAY_SONG )
+			{
+				m_automation_track->play( m_playPos[m_playMode],
+						played_frames,
+						total_frames_played, tco_num );
+			}
 
-		// loop through all tracks and play them if they're not muted
-		for( trackVector::iterator it = tv.begin(); it != tv.end();
-									++it )
-		{
-			if( ( *it )->muted() == FALSE )
+			// loop through all tracks and play them
+			for( trackVector::iterator it = tv.begin();
+							it != tv.end(); ++it )
 			{
 				( *it )->play( m_playPos[m_playMode],
-					m_playPos[m_playMode].currentFrame(),
-					played_frames, total_frames_played,
-					tco_num );
+						played_frames,
+						total_frames_played, tco_num );
 			}
 		}
 
 		// update frame-counters
 		total_frames_played += played_frames;
 		m_playPos[m_playMode].setCurrentFrame( played_frames +
-					m_playPos[m_playMode].currentFrame() );
-		m_playPos[m_playMode].setTact64th(
-					( m_playPos[m_playMode].currentFrame() *
-						64 / frames_per_tact) % 64 );
+								current_frame );
 	}
 
 	if( m_exporting == FALSE )
@@ -1091,7 +1127,7 @@ void songEditor::play( void )
 		if( m_playMode != PLAY_SONG )
 		{
 			// make sure, bb-editor updates/resets it play-button
-			eng()->getBBEditor()->stop();
+			engine::getBBEditor()->stop();
 			//pianoRoll::inst()->stop();
 		}
 		else
@@ -1174,8 +1210,7 @@ void songEditor::setPlayPos( tact _tact_num, tact64th _t_64th, playModes
 {
 	m_playPos[_play_mode].setTact( _tact_num );
 	m_playPos[_play_mode].setTact64th( _t_64th );
-	m_playPos[_play_mode].setCurrentFrame( static_cast<f_cnt_t>(
-					_t_64th * framesPerTact() / 64.0f ) );
+	m_playPos[_play_mode].setCurrentFrame( 0.0f );
 	if( _play_mode == m_playMode )
 	{
 		updateTimeLinePosition();
@@ -1201,10 +1236,6 @@ void songEditor::updateTimeLinePosition( void )
 
 void songEditor::stop( void )
 {
-	if( m_destroyed )
-	{
-		return;
-	}
 	m_actions.push_back( ACT_STOP_PLAY );
 #ifdef QT4
 	m_playButton->setIcon( embed::getIconPixmap( "play" ) );
@@ -1316,21 +1347,17 @@ void songEditor::addSampleTrack( void )
 
 
 
-float songEditor::framesPerTact( void ) const
+bpm_t songEditor::getTempo( void )
 {
-	// when fooling around with tempo while playing, we sometimes get
-	// 0 here which leads to FP-exception, so handle it separately
-	const int bpm = tMax( 1, m_bpmSpinBox->value() );
-	return( eng()->getMixer()->sampleRate() * 60.0f * BEATS_PER_TACT /
-								bpm );
+	return( m_bpmSpinBox->value() );
 }
 
 
 
 
-bpm_t songEditor::getTempo( void )
+automationPattern * songEditor::tempoAutomationPattern( void )
 {
-	return( m_bpmSpinBox->value() );
+	return( m_bpmSpinBox->getAutomationPattern() );
 }
 
 
@@ -1349,7 +1376,7 @@ bool songEditor::mayChangeProject( void )
 #else
 		information
 #endif
-				( eng()->getMainWindow(),
+				( engine::getMainWindow(),
 						tr( "Project not saved" ),
 						tr( "The current project was "
 							"modified since last "
@@ -1370,12 +1397,12 @@ bool songEditor::mayChangeProject( void )
 				QMessageBox::Yes,
 				QMessageBox::No,
 				QMessageBox::Cancel,
-				eng()->getMainWindow() );
+				engine::getMainWindow() );
 	int answer = mb.exec();
 
 	if( answer == QMessageBox::Yes )
 	{
-		return( eng()->getMainWindow()->saveProject() );
+		return( engine::getMainWindow()->saveProject() );
 	}
 	else if( answer == QMessageBox::No )
 	{
@@ -1390,43 +1417,30 @@ bool songEditor::mayChangeProject( void )
 
 void songEditor::clearProject( void )
 {
-	eng()->getProjectJournal()->setJournalling( FALSE );
+	engine::getProjectJournal()->setJournalling( FALSE );
 
 	if( m_playing )
 	{
-		// stop play, because it's dangerous that play-routines try to
-		// access non existing data (as you can see in the next lines,
-		// all data is cleared!)
 		stop();
 	}
 
-	// make sure all running notes are cleared, otherwise the whole
-	// thing will end up in a SIGSEGV...
-	eng()->getMixer()->clear( TRUE );
-	while( eng()->getMixer()->haveNoRunningNotes() == FALSE )
-	{
-#ifdef QT4
-		QApplication::processEvents( QEventLoop::AllEvents );
-#else
-		qApp->processEvents();
-#endif
-	}
-
+	engine::getMixer()->lock();
 	clearAllTracks();
 
-	eng()->getAutomationEditor()->setCurrentPattern( NULL );
+	engine::getAutomationEditor()->setCurrentPattern( NULL );
 	m_bpmSpinBox->getAutomationPattern()->clear();
 	m_masterVolumeSlider->clearAutomationValues();
 	m_masterPitchSlider->clearAutomationValues();
 
-	eng()->getBBEditor()->clearAllTracks();
+	engine::getBBEditor()->clearAllTracks();
+	engine::getMixer()->unlock();
 
-	eng()->getProjectNotes()->clear();
+	engine::getProjectNotes()->clear();
 
-	eng()->getProjectJournal()->clearInvalidJournallingObjects();
-	eng()->getProjectJournal()->clearJournal();
+	engine::getProjectJournal()->clearInvalidJournallingObjects();
+	engine::getProjectJournal()->clearJournal();
 
-	eng()->getProjectJournal()->setJournalling( TRUE );
+	engine::getProjectJournal()->setJournalling( TRUE );
 }
 
 
@@ -1454,20 +1468,20 @@ void songEditor::createNewProject( void )
 
 	clearProject();
 
-	eng()->getProjectJournal()->setJournalling( FALSE );
+	engine::getProjectJournal()->setJournalling( FALSE );
 
 	track * t;
-	t = track::create( track::CHANNEL_TRACK, this );
+	t = track::create( track::INSTRUMENT_TRACK, this );
 	dynamic_cast< instrumentTrack * >( t )->loadInstrument(
 					"tripleoscillator" );
 	track::create( track::SAMPLE_TRACK, this );
-	t = track::create( track::CHANNEL_TRACK, eng()->getBBEditor() );
+	t = track::create( track::INSTRUMENT_TRACK, engine::getBBEditor() );
 	dynamic_cast< instrumentTrack * >( t )->loadInstrument(
 						"tripleoscillator" );
 	track::create( track::BB_TRACK, this );
 
 	m_loadingProject = TRUE;
-	setTempo( DEFAULT_BPM );
+	m_bpmSpinBox->setInitValue( DEFAULT_BPM );
 	m_masterVolumeSlider->setInitValue( 100 );
 	m_masterPitchSlider->setInitValue( 0 );
 	m_loadingProject = FALSE;
@@ -1476,9 +1490,9 @@ void songEditor::createNewProject( void )
 
 	m_modified = FALSE;
 
-	eng()->getMainWindow()->resetWindowTitle( "" );
+	engine::getMainWindow()->resetWindowTitle( "" );
 
-	eng()->getProjectJournal()->setJournalling( TRUE );
+	engine::getProjectJournal()->setJournalling( TRUE );
 
 }
 
@@ -1502,7 +1516,7 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 {
 	clearProject();
 
-	eng()->getProjectJournal()->setJournalling( FALSE );
+	engine::getProjectJournal()->setJournalling( FALSE );
 
 	m_fileName = _file_name;
 	m_oldFileName = _file_name;
@@ -1519,52 +1533,14 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 	}
 
 	// get the header information from the DOM
-	QDomNode node = mmp.head().firstChild();
-	while( !node.isNull() )
-	{
-		if( node.isElement() )
-		{
-			if( node.nodeName() == "bpm" &&
-		node.toElement().attribute( "value" ).toInt() > 0 )
-			{
-				setTempo( node.toElement().attribute( "value"
-								).toInt() );
-			}
-			else if( node.nodeName() == "mastervol" )
-			{
-				if( node.toElement().attribute( "value"
-								).toInt() > 0 )
-				{
-					m_masterVolumeSlider->setInitValue(
-						node.toElement().attribute(
-							"value" ).toInt() );
-				}
-				else
-				{
-					m_masterVolumeSlider->setInitValue(
-							DEFAULT_VOLUME );
-				}
-			}
-			else if( node.nodeName() == "masterpitch" )
-			{
-				m_masterPitchSlider->setInitValue(
-					-node.toElement().attribute( "value"
-								).toInt() );
-			}
-			else if( node.nodeName()
-					== automationPattern::classNodeName() )
-			{
-				m_bpmSpinBox->loadSettings( mmp.head(), "bpm" );
-				m_masterVolumeSlider->loadSettings( mmp.head(),
-								"mastervol" );
-				m_masterPitchSlider->loadSettings( mmp.head(),
-								"masterpitch" );
-			}
-		}
-		node = node.nextSibling();
-	}
+	m_bpmSpinBox->loadSettings( mmp.head(), "bpm" );
+	m_masterVolumeSlider->loadSettings( mmp.head(), "mastervol" );
+	m_masterPitchSlider->loadSettings( mmp.head(), "masterpitch" );
 
-	node = mmp.content().firstChild();
+	// reset loop-point-state
+	m_playPos[PLAY_SONG].m_timeLine->toggleLoopPoints( 0 );
+
+	QDomNode node = mmp.content().firstChild();
 	while( !node.isNull() )
 	{
 		if( node.isElement() )
@@ -1575,21 +1551,21 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 					restoreState( node.toElement() );
 			}
 			else if( node.nodeName() ==
-					eng()->getPianoRoll()->nodeName() )
+					engine::getPianoRoll()->nodeName() )
 			{
-				eng()->getPianoRoll()->restoreState(
+				engine::getPianoRoll()->restoreState(
 							node.toElement() );
 			}
 			else if( node.nodeName() ==
-				eng()->getAutomationEditor()->nodeName() )
+				engine::getAutomationEditor()->nodeName() )
 			{
-				eng()->getAutomationEditor()->restoreState(
+				engine::getAutomationEditor()->restoreState(
 							node.toElement() );
 			}
 			else if( node.nodeName() ==
-					eng()->getProjectNotes()->nodeName() )
+					engine::getProjectNotes()->nodeName() )
 			{
-				( (journallingObject *)( eng()->
+				( (journallingObject *)( engine::
 							getProjectNotes() ) )->
 					restoreState( node.toElement() );
 			}
@@ -1608,9 +1584,12 @@ void FASTCALL songEditor::loadProject( const QString & _file_name )
 
 	m_loadingProject = FALSE;
 
-	eng()->getMainWindow()->resetWindowTitle( "" );
+	configManager::inst()->addRecentlyOpenedProject( _file_name );
 
-	eng()->getProjectJournal()->setJournalling( TRUE );
+	engine::getMainWindow()->resetWindowTitle( "" );
+	engine::getMainWindow()->updateRecentlyOpenedProjectsMenu();
+
+	engine::getProjectJournal()->setJournalling( TRUE );
 }
 
 
@@ -1628,9 +1607,9 @@ bool songEditor::saveProject( void )
 
 	( (journallingObject *)( this ) )->saveState( mmp, mmp.content() );
 
-	eng()->getPianoRoll()->saveState( mmp, mmp.content() );
-	eng()->getAutomationEditor()->saveState( mmp, mmp.content() );
-	( (journallingObject *)( eng()->getProjectNotes() ) )->saveState( mmp,
+	engine::getPianoRoll()->saveState( mmp, mmp.content() );
+	engine::getAutomationEditor()->saveState( mmp, mmp.content() );
+	( (journallingObject *)( engine::getProjectNotes() ) )->saveState( mmp,
 								mmp.content() );
 	m_playPos[PLAY_SONG].m_timeLine->saveState( mmp, mmp.content() );
 
@@ -1644,7 +1623,9 @@ bool songEditor::saveProject( void )
 							).arg( m_fileName ),
 				embed::getIconPixmap( "project_save", 24, 24 ),
 									2000 );
-		eng()->getMainWindow()->resetWindowTitle( "" );
+		configManager::inst()->addRecentlyOpenedProject( m_fileName );
+		engine::getMainWindow()->resetWindowTitle( "" );
+		engine::getMainWindow()->updateRecentlyOpenedProjectsMenu();
 	}
 	else
 	{
@@ -1714,7 +1695,7 @@ void songEditor::exportProject( void )
 	}
  	base_filename += fileEncodeDevices[0].m_extension;
 
-	QFileDialog efd( eng()->getMainWindow() );
+	QFileDialog efd( engine::getMainWindow() );
 	efd.setFileMode( QFileDialog::AnyFile );
 
 	int idx = 0;
@@ -1752,7 +1733,7 @@ void songEditor::exportProject( void )
 		const QString export_file_name = efd.selectedFile();
 #endif
 		if( QFileInfo( export_file_name ).exists() == TRUE &&
-			QMessageBox::warning( eng()->getMainWindow(),
+			QMessageBox::warning( engine::getMainWindow(),
 						tr( "File already exists" ),
 						tr( "The file \"%1\" already "
 							"exists. Do you want "
@@ -1768,13 +1749,22 @@ void songEditor::exportProject( void )
 			return;
 		}
 		exportProjectDialog epd( export_file_name,
-						eng()->getMainWindow(), eng() );
+						engine::getMainWindow() );
 		epd.exec();
 	}
 }
 
 
 
+
+void songEditor::updateFramesPerTact64th( void )
+{
+	engine::updateFramesPerTact64th();
+}
+
+
+
+
 #include "song_editor.moc"
 
 
