@@ -3,7 +3,7 @@
 /*
  * midi_alsa_seq.cpp - ALSA-sequencer-client
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -43,6 +43,7 @@
 
 #include "midi_alsa_seq.h"
 #include "config_mgr.h"
+#include "engine.h"
 #include "gui_templates.h"
 #include "song_editor.h"
 #include "midi_port.h"
@@ -52,18 +53,11 @@
 #ifdef ALSA_SUPPORT
 
 
-midiALSASeq::midiALSASeq( engine * _engine ) :
-#ifndef QT4
-	QObject(),
-#endif
-	midiClient( _engine ),
-	QThread(),
+midiALSASeq::midiALSASeq( void ) :
 	m_seqHandle( NULL ),
 	m_queueID( -1 ),
 	m_quit( FALSE ),
-	m_portListUpdateTimer( this ),
-	m_readablePorts(),
-	m_writeablePorts()
+	m_portListUpdateTimer( this )
 {
 	int err;
 	if( ( err = snd_seq_open( &m_seqHandle,
@@ -84,13 +78,13 @@ midiALSASeq::midiALSASeq( engine * _engine ) :
 	snd_seq_queue_tempo_t * tempo;
 	snd_seq_queue_tempo_alloca( &tempo );
 	snd_seq_queue_tempo_set_tempo( tempo, 6000000 /
-					eng()->getSongEditor()->getTempo() );
+					engine::getSongEditor()->getTempo() );
 	snd_seq_queue_tempo_set_ppq( tempo, 16 );
 	snd_seq_set_queue_tempo( m_seqHandle, m_queueID, tempo );
 
 	snd_seq_start_queue( m_seqHandle, m_queueID, NULL );
-	changeQueueTempo( eng()->getSongEditor()->getTempo() );
-	connect( eng()->getSongEditor(), SIGNAL( tempoChanged( bpm_t ) ),
+	changeQueueTempo( engine::getSongEditor()->getTempo() );
+	connect( engine::getSongEditor(), SIGNAL( tempoChanged( bpm_t ) ),
 			this, SLOT( changeQueueTempo( bpm_t ) ) );
 
 	// initial list-update
@@ -101,6 +95,12 @@ midiALSASeq::midiALSASeq( engine * _engine ) :
 	// we check for port-changes every second
 	m_portListUpdateTimer.start( 1000 );
 
+	// use a pipe to detect shutdown
+	if( pipe( m_pipe ) == -1 )
+	{
+		perror( __FILE__ ": pipe" );
+	}
+
 	start( 
 #if QT_VERSION >= 0x030505
 	    	QThread::IdlePriority 
@@ -116,8 +116,9 @@ midiALSASeq::~midiALSASeq()
 	if( isRunning() )
 	{
 		m_quit = TRUE;
+		// wake up input queue
+		write( m_pipe[1], "\n", 1 );
 		wait( 1000 );
-		terminate();
 
 		snd_seq_stop_queue( m_seqHandle, m_queueID, NULL );
 		snd_seq_free_queue( m_seqHandle, m_queueID );
@@ -185,10 +186,11 @@ void midiALSASeq::processOutEvent( const midiEvent & _me,
 						_me.velocity() );
 			break;
 
-		case PITCH_BEND:
-			snd_seq_ev_set_pitchbend( &ev,
+		case CONTROL_CHANGE:
+			snd_seq_ev_set_controller( &ev,
 						_port->outputChannel(),
-						_me.m_data.m_param[0] - 8192 );
+						_me.m_data.m_param[0],
+						_me.m_data.m_param[1] );
 			break;
 
 		case PROGRAM_CHANGE:
@@ -203,6 +205,12 @@ void midiALSASeq::processOutEvent( const midiEvent & _me,
 						_me.m_data.m_param[0] );
 			break;
 
+		case PITCH_BEND:
+			snd_seq_ev_set_pitchbend( &ev,
+						_port->outputChannel(),
+						_me.m_data.m_param[0] - 8192 );
+			break;
+
 		default:
 			printf( "ALSA-sequencer: unhandled output event %d\n",
 							(int) _me.m_type );
@@ -437,8 +445,32 @@ void midiALSASeq::subscribeWriteablePort( midiPort * _port,
 
 void midiALSASeq::run( void )
 {
+	// watch the pipe and sequencer input events
+	int pollfd_count = snd_seq_poll_descriptors_count( m_seqHandle,
+								POLLIN );
+	struct pollfd * pollfd_set = new struct pollfd[pollfd_count + 1];
+	snd_seq_poll_descriptors( m_seqHandle, pollfd_set + 1, pollfd_count,
+								POLLIN );
+	pollfd_set[0].fd = m_pipe[0];
+	pollfd_set[0].events = POLLIN;
+	++pollfd_count;
+
 	while( m_quit == FALSE )
 	{
+		if( poll( pollfd_set, pollfd_count, -1 ) == -1 )
+		{
+			// gdb may interrupt the poll
+			if( errno == EINTR )
+			{
+				continue;
+			}
+			perror( __FILE__ ": poll" );
+		}
+		// shutdown?
+		if( pollfd_set[0].revents )
+		{
+			break;
+		}
 		snd_seq_event_t * ev;
 		snd_seq_event_input( m_seqHandle, &ev );
 
@@ -488,6 +520,38 @@ void midiALSASeq::run( void )
 							), midiTime() );
 				break;
 
+			case SND_SEQ_EVENT_CONTROLLER:
+				dest->processInEvent( midiEvent( CONTROL_CHANGE,
+						ev->data.control.channel,
+						ev->data.control.param,
+						ev->data.control.value ),
+								midiTime() );
+				break;
+
+			case SND_SEQ_EVENT_PGMCHANGE:
+				dest->processInEvent( midiEvent( PROGRAM_CHANGE,
+						ev->data.control.channel,
+						ev->data.control.param,
+						ev->data.control.value ),
+								midiTime() );
+				break;
+
+			case SND_SEQ_EVENT_CHANPRESS:
+				dest->processInEvent( midiEvent(
+							CHANNEL_PRESSURE,
+						ev->data.control.channel,
+						ev->data.control.param,
+						ev->data.control.value ),
+								midiTime() );
+				break;
+
+			case SND_SEQ_EVENT_PITCHBEND:
+				dest->processInEvent( midiEvent( PITCH_BEND,
+						ev->data.control.channel,
+						ev->data.control.value + 8192,
+							0 ), midiTime() );
+				break;
+
 			case SND_SEQ_EVENT_SENSING:
 			case SND_SEQ_EVENT_CLOCK:
 				break;
@@ -500,6 +564,7 @@ void midiALSASeq::run( void )
 
 	}
 
+	delete[] pollfd_set;
 }
 
 
