@@ -3,7 +3,7 @@
 /*
  * midi_client.cpp - base-class for MIDI-clients like ALSA-sequencer-client
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * This file partly contains code from Fluidsynth, Peter Hanappe
  *
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -40,8 +40,7 @@
 
 
 
-midiClient::midiClient( engine * _engine ) :
-	engineObject( _engine )
+midiClient::midiClient( void )
 {
 }
 
@@ -88,6 +87,7 @@ void midiClient::removePort( midiPort * _port )
 							_port );
 	if( it != m_midiPorts.end() )
 	{
+		delete *it;
 		m_midiPorts.erase( it );
 	}
 }
@@ -130,8 +130,7 @@ void midiClient::subscribeWriteablePort( midiPort * , const QString & , bool)
 
 
 
-midiClientRaw::midiClientRaw( engine * _engine ) :
-	midiClient( _engine )
+midiClientRaw::midiClientRaw( void )
 {
 }
 
