@@ -3,7 +3,7 @@
 /*
  * bb_track.cpp - implementation of class bbTrack and bbTCO
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -19,8 +19,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -45,12 +45,14 @@
 
 
 #include "bb_track.h"
-#include "song_editor.h"
 #include "bb_editor.h"
-#include "gui_templates.h"
-#include "name_label.h"
 #include "embed.h"
+#include "engine.h"
+#include "gui_templates.h"
+#include "mixer.h"
+#include "name_label.h"
 #include "rename_dialog.h"
+#include "song_editor.h"
 #include "templates.h"
 
 
@@ -68,7 +70,7 @@ bbTCO::bbTCO( track * _track, const QColor & _c ) :
 #ifndef QT4
 	setBackgroundMode( Qt::NoBackground );
 #endif
-	tact t = eng()->getBBEditor()->lengthOfBB(
+	tact t = engine::getBBEditor()->lengthOfBB(
 					bbTrack::numOfBBTrack( getTrack() ) );
 	if( t > 0 )
 	{
@@ -165,7 +167,7 @@ void bbTCO::paintEvent( QPaintEvent * )
 	}
 #endif
 
-	tact t = eng()->getBBEditor()->lengthOfBB( bbTrack::numOfBBTrack(
+	tact t = engine::getBBEditor()->lengthOfBB( bbTrack::numOfBBTrack(
 								getTrack() ) );
 	if( length() > 64 && t > 0 )
 	{
@@ -248,10 +250,10 @@ void bbTCO::loadSettings( const QDomElement & _this )
 
 void bbTCO::openInBBEditor( bool )
 {
-	eng()->getBBEditor()->setCurrentBB( bbTrack::numOfBBTrack(
+	engine::getBBEditor()->setCurrentBB( bbTrack::numOfBBTrack(
 								getTrack() ) );
-	eng()->getBBEditor()->show();
-	eng()->getBBEditor()->setFocus();
+	engine::getBBEditor()->show();
+	engine::getBBEditor()->setFocus();
 }
 
 
@@ -289,10 +291,40 @@ void bbTCO::changeName( void )
 void bbTCO::changeColor( void )
 {
 	QColor _new_color = QColorDialog::getColor( m_color );
-	if( _new_color.isValid() && _new_color != m_color )
+	if( !_new_color.isValid() )
+	{
+		return;
+	}
+	if( isSelected() )
+	{
+		vvector<selectableObject *> selected =
+				engine::getSongEditor()->selectedObjects();
+		for( vvector<selectableObject *>::iterator it =
+							selected.begin();
+						it != selected.end(); ++it )
+		{
+			bbTCO * bb_tco = dynamic_cast<bbTCO *>( *it );
+			if( bb_tco )
+			{
+				bb_tco->setColor( _new_color );
+			}
+		}
+	}
+	else
+	{
+		setColor( _new_color );
+	}
+}
+
+
+
+
+void bbTCO::setColor( QColor _new_color )
+{
+	if( _new_color != m_color )
 	{
 		m_color = _new_color;
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 		update();
 	}
 }
@@ -313,7 +345,7 @@ bbTrack::bbTrack( trackContainer * _tc ) :
 	s_infoMap[this] = bbNum;
 
 	m_trackLabel = new nameLabel( tr( "Beat/Baseline %1" ).arg( bbNum ),
-					getTrackSettingsWidget(), eng() );
+						getTrackSettingsWidget() );
 	m_trackLabel->setPixmap( embed::getIconPixmap( "bb_track" ) );
 	m_trackLabel->setGeometry( 1, 1, DEFAULT_SETTINGS_WIDGET_WIDTH - 2,
 									29 );
@@ -321,13 +353,13 @@ bbTrack::bbTrack( trackContainer * _tc ) :
 	connect( m_trackLabel, SIGNAL( clicked() ),
 			this, SLOT( clickedTrackLabel() ) );
 	connect( m_trackLabel, SIGNAL( nameChanged() ),
-			eng()->getBBEditor(), SLOT( updateComboBox() ) );
+			engine::getBBEditor(), SLOT( updateComboBox() ) );
 	connect( m_trackLabel, SIGNAL( pixmapChanged() ),
-			eng()->getBBEditor(), SLOT( updateComboBox() ) );
+			engine::getBBEditor(), SLOT( updateComboBox() ) );
 
 
-	eng()->getBBEditor()->setCurrentBB( bbNum );
-	eng()->getBBEditor()->updateComboBox();
+	engine::getBBEditor()->setCurrentBB( bbNum );
+	engine::getBBEditor()->updateComboBox();
 
 	_tc->updateAfterTrackAdd();
 }
@@ -337,8 +369,10 @@ bbTrack::bbTrack( trackContainer * _tc ) :
 
 bbTrack::~bbTrack()
 {
+	engine::getMixer()->removePlayHandles( this );
+
 	csize bb = s_infoMap[this];
-	eng()->getBBEditor()->removeBB( bb );
+	engine::getBBEditor()->removeBB( bb );
 	for( infoMap::iterator it = s_infoMap.begin(); it != s_infoMap.end();
 									++it )
 	{
@@ -355,7 +389,7 @@ bbTrack::~bbTrack()
 #endif
 	}
 	s_infoMap.remove( this );
-	eng()->getBBEditor()->updateComboBox();
+	engine::getBBEditor()->updateComboBox();
 }
 
 
@@ -368,25 +402,25 @@ track::trackTypes bbTrack::type( void ) const
 
 
 
-// play _frames frames of given TCO within starting with _start/_start_frame
+// play _frames frames of given TCO within starting with _start
 bool FASTCALL bbTrack::play( const midiTime & _start,
-						const f_cnt_t _start_frame,
-						const fpab_t _frames,
-						const f_cnt_t _frame_base,
+						const fpp_t _frames,
+						const f_cnt_t _offset,
 							Sint16 _tco_num )
 {
+	sendMidiTime( _start );
+
 	if( _tco_num >= 0 )
 	{
-		return( eng()->getBBEditor()->play( _start, _start_frame,
-							_frames,
-							_frame_base,
+		return( engine::getBBEditor()->play( _start, _frames,
+							_offset,
 							s_infoMap[this] ) );
 	}
 
 	vlist<trackContentObject *> tcos;
-	getTCOsInRange( tcos, _start, _start +static_cast<Sint32>( _frames *
-			64 / eng()->getSongEditor()->framesPerTact() ) );
-	
+	getTCOsInRange( tcos, _start, _start + static_cast<Sint32>( _frames /
+						engine::framesPerTact64th() ) );
+
 	if ( tcos.size() == 0 )
 	{
 		return( FALSE );
@@ -406,9 +440,9 @@ bool FASTCALL bbTrack::play( const midiTime & _start,
 	}
 	if( _start - lastPosition < lastLen )
 	{
-		return( eng()->getBBEditor()->play( _start - lastPosition,
-							_start_frame, _frames,
-							_frame_base,
+		return( engine::getBBEditor()->play( _start - lastPosition,
+							_frames,
+							_offset,
 							s_infoMap[this] ) );
 	}
 	return( FALSE );
@@ -444,14 +478,28 @@ void bbTrack::saveTrackSpecificSettings( QDomDocument & _doc,
 	_this.setAttribute( "name", m_trackLabel->text() );
 	_this.setAttribute( "icon", m_trackLabel->pixmapFile() );
 /*	_this.setAttribute( "current", s_infoMap[this] ==
-					eng()->getBBEditor()->currentBB() );*/
+					engine::getBBEditor()->currentBB() );*/
 	if( s_infoMap[this] == 0 &&
-			_this.parentNode().nodeName() != "clone" &&
+			_this.parentNode().parentNode().nodeName() != "clone" &&
 			_this.parentNode().nodeName() != "journaldata" )
 	{
-		( (journallingObject *)( eng()->getBBEditor() ) )->saveState(
+		( (journallingObject *)( engine::getBBEditor() ) )->saveState(
 								_doc, _this );
 	}
+
+	int track_num = 0;
+	trackVector tracks = engine::getBBEditor()->tracks();
+	for( trackVector::iterator it = tracks.begin(); it != tracks.end();
+							++it, ++track_num )
+	{
+		if( automationDisabled( *it ) )
+		{
+			QDomElement disabled = _doc.createElement(
+							"automation-disabled" );
+			disabled.setAttribute( "track", track_num );
+			_this.appendChild( disabled );
+		}
+	}
 }
 
 
@@ -464,33 +512,49 @@ void bbTrack::loadTrackSpecificSettings( const QDomElement & _this )
 	{
 		m_trackLabel->setPixmapFile( _this.attribute( "icon" ) );
 	}
-	if( _this.firstChild().isElement() )
+	engine::getBBEditor()->updateComboBox();
+
+	QDomNode node = _this.namedItem( trackContainer::classNodeName() );
+	if( node.isElement() )
 	{
-		( (journallingObject *)( eng()->getBBEditor() ) )->restoreState(
-					_this.firstChild().toElement() );
+		( (journallingObject *)engine::getBBEditor() )->restoreState(
+							node.toElement() );
 	}
 /*	doesn't work yet because bbTrack-ctor also sets current bb so if
 	bb-tracks are created after this function is called, this doesn't
 	help at all....
 	if( _this.attribute( "current" ).toInt() )
 	{
-		eng()->getBBEditor()->setCurrentBB( s_infoMap[this] );
+		engine::getBBEditor()->setCurrentBB( s_infoMap[this] );
 	}*/
+
+	trackVector tracks = engine::getBBEditor()->tracks();
+	node = _this.firstChild();
+	while( !node.isNull() )
+	{
+		if( node.isElement()
+				&& node.nodeName() == "automation-disabled" )
+		{
+			disableAutomation( tracks[node.toElement().attribute(
+							"track" ).toInt()] );
+		}
+		node = node.nextSibling();
+	}
 }
 
 
 
 
 // return pointer to bbTrack specified by _bb_num
-bbTrack * bbTrack::findBBTrack( csize _bb_num, engine * _engine )
+bbTrack * bbTrack::findBBTrack( csize _bb_num )
 {
 	for( infoMap::iterator it = s_infoMap.begin(); it != s_infoMap.end();
 									++it )
 	{
 #ifdef QT4
-		if( it.value() == _bb_num && it.key()->eng() == _engine )
+		if( it.value() == _bb_num )
 #else
-		if( it.data() == _bb_num && it.key()->eng() == _engine )
+		if( it.data() == _bb_num )
 #endif
 		{
 			return( it.key() );
@@ -521,9 +585,8 @@ void bbTrack::swapBBTracks( track * _track1, track * _track2 )
 	if( t1 != NULL && t2 != NULL )
 	{
 		qSwap( s_infoMap[t1], s_infoMap[t2] );
-		_track1->eng()->getBBEditor()->swapBB( s_infoMap[t1],
-								s_infoMap[t2] );
-		_track1->eng()->getBBEditor()->setCurrentBB( s_infoMap[t1] );
+		engine::getBBEditor()->swapBB( s_infoMap[t1], s_infoMap[t2] );
+		engine::getBBEditor()->setCurrentBB( s_infoMap[t1] );
 	}
 }
 
@@ -532,8 +595,8 @@ void bbTrack::swapBBTracks( track * _track1, track * _track2 )
 
 void bbTrack::clickedTrackLabel( void )
 {
-	eng()->getBBEditor()->setCurrentBB( s_infoMap[this] );
-	eng()->getBBEditor()->show();
+	engine::getBBEditor()->setCurrentBB( s_infoMap[this] );
+	engine::getBBEditor()->show();
 }
 
 
