@@ -3,8 +3,8 @@
 /*
  * pattern.cpp - implementation of class pattern which holds notes
  *
- * Copyright (c) 2004-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
- * Copyright (c) 2005 Danny McRae <khjklujn/at/yahoo.com>
+ * Copyright (c) 2004-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Danny McRae <khjklujn/at/yahoo.com>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -59,6 +59,7 @@
 #include "templates.h"
 #include "gui_templates.h"
 #include "embed.h"
+#include "engine.h"
 #include "piano_roll.h"
 #include "track_container.h"
 #include "rename_dialog.h"
@@ -68,7 +69,6 @@
 #include "tooltip.h"
 #include "bb_editor.h"
 #include "string_pair_drag.h"
-#include "buffer_allocator.h"
 #include "main_window.h"
 
 
@@ -88,7 +88,6 @@ pattern::pattern ( instrumentTrack * _instrument_track ) :
 	m_patternType( BEAT_PATTERN ),
 	m_name( _instrument_track->name() ),
 	m_steps( DEFAULT_STEPS_PER_TACT ),
-	m_frozenPatternMutex(),
 	m_frozenPattern( NULL ),
 	m_freezing( FALSE ),
 	m_freezeAborted( FALSE )
@@ -107,7 +106,6 @@ pattern::pattern( const pattern & _pat_to_copy ) :
 	m_patternType( _pat_to_copy.m_patternType ),
 	m_name( "" ),
 	m_steps( _pat_to_copy.m_steps ),
-	m_frozenPatternMutex(),
 	m_frozenPattern( NULL ),
 	m_freezeAborted( FALSE )
 {
@@ -125,16 +123,16 @@ pattern::pattern( const pattern & _pat_to_copy ) :
 
 pattern::~pattern()
 {
-	if( eng()->getPianoRoll()->currentPattern() == this )
+	if( engine::getPianoRoll()->currentPattern() == this )
 	{
-		eng()->getPianoRoll()->setCurrentPattern( NULL );
+		engine::getPianoRoll()->setCurrentPattern( NULL );
 		// we have to have the song-editor to stop playing if it played
 		// us before
-		if( eng()->getSongEditor()->playing() &&
-			eng()->getSongEditor()->playMode() ==
+		if( engine::getSongEditor()->playing() &&
+			engine::getSongEditor()->playMode() ==
 						songEditor::PLAY_PATTERN )
 		{
-			eng()->getSongEditor()->playPattern( NULL );
+			engine::getSongEditor()->playPattern( NULL );
 		}
 	}
 
@@ -146,9 +144,10 @@ pattern::~pattern()
 
 	m_notes.clear();
 
-	m_frozenPatternMutex.lock();
-	delete m_frozenPattern;
-	m_frozenPatternMutex.unlock();
+	if( m_frozenPattern )
+	{
+		sharedObject::unref( m_frozenPattern );
+	}
 }
 
 
@@ -244,9 +243,10 @@ note * pattern::addNote( const note & _new_note, const bool _quant_pos )
 	note * new_note = new note( _new_note );
 	if( _quant_pos )
 	{
-		new_note->quantizePos( eng()->getPianoRoll()->quantization() );
+		new_note->quantizePos( engine::getPianoRoll()->quantization() );
 	}
 
+	engine::getMixer()->lock();
 	if( m_notes.size() == 0 || m_notes.back()->pos() <= new_note->pos() )
 	{
 		m_notes.push_back( new_note );
@@ -269,6 +269,7 @@ note * pattern::addNote( const note & _new_note, const bool _quant_pos )
 
 		m_notes.insert( it, new_note );
 	}
+	engine::getMixer()->unlock();
 
 	checkType();
 	update();
@@ -284,6 +285,7 @@ note * pattern::addNote( const note & _new_note, const bool _quant_pos )
 
 void pattern::removeNote( const note * _note_to_del )
 {
+	engine::getMixer()->lock();
 	noteVector::iterator it = m_notes.begin();
 	while( it != m_notes.end() )
 	{
@@ -295,6 +297,7 @@ void pattern::removeNote( const note * _note_to_del )
 		}
 		++it;
 	}
+	engine::getMixer()->unlock();
 
 	checkType();
 	update();
@@ -322,44 +325,20 @@ note * pattern::rearrangeNote( const note * _note_to_proc,
 
 void pattern::clearNotes( void )
 {
+	engine::getMixer()->lock();
 	for( noteVector::iterator it = m_notes.begin(); it != m_notes.end();
 									++it )
 	{
 		delete *it;
 	}
-
 	m_notes.clear();
+	engine::getMixer()->unlock();
+
 	checkType();
 	update();
-	if( eng()->getPianoRoll()->currentPattern() == this )
+	if( engine::getPianoRoll()->currentPattern() == this )
 	{
-		eng()->getPianoRoll()->update();
-	}
-}
-
-
-
-
-note * pattern::noteAt( int _note_num )
-{
-	if( (csize) _note_num < m_notes.size() )
-	{
-		return( m_notes[_note_num] );
-	}
-	return( NULL );
-}
-
-
-
-
-void pattern::setNoteAt( int _note_num, note _new_note )
-{
-	if( static_cast<csize>( _note_num ) < m_notes.size() )
-	{
-		delete m_notes[_note_num];
-		m_notes[_note_num] = new note( _new_note );
-		checkType();
-		update();
+		engine::getPianoRoll()->update();
 	}
 }
 
@@ -396,20 +375,6 @@ void pattern::checkType( void )
 
 
 
-void pattern::playFrozenData( sampleFrame * _ab, const f_cnt_t _start_frame,
-							const fpab_t _frames )
-{
-	m_frozenPatternMutex.lock();
-	if( m_frozenPattern != NULL )
-	{
-		m_frozenPattern->play( _ab, _start_frame, _frames );
-	}
-	m_frozenPatternMutex.unlock();
-}
-
-
-
-
 void pattern::saveSettings( QDomDocument & _doc, QDomElement & _this )
 {
 	_this.setAttribute( "type", m_patternType );
@@ -470,7 +435,7 @@ void pattern::loadSettings( const QDomElement & _this )
 		if( node.isElement() &&
 			!node.toElement().attribute( "metadata" ).toInt() )
 		{
-			note * n = new note( eng() );
+			note * n = new note;
 			n->restoreState( node.toElement() );
 			m_notes.push_back( n );
 		}
@@ -484,6 +449,7 @@ void pattern::loadSettings( const QDomElement & _this )
 	}
 
 	ensureBeatNotes();
+	checkType();
 /*	if( _this.attribute( "frozen" ).toInt() )
 	{
 		freeze();
@@ -514,9 +480,9 @@ void pattern::openInPianoRoll( void )
 
 void pattern::openInPianoRoll( bool )
 {
-	eng()->getPianoRoll()->setCurrentPattern( this );
-	eng()->getPianoRoll()->show();
-	eng()->getPianoRoll()->setFocus();
+	engine::getPianoRoll()->setCurrentPattern( this );
+	engine::getPianoRoll()->show();
+	engine::getPianoRoll()->setFocus();
 }
 
 
@@ -550,7 +516,7 @@ void pattern::changeName( void )
 
 void pattern::freeze( void )
 {
-	if( eng()->getSongEditor()->playing() )
+	if( engine::getSongEditor()->playing() )
 	{
 		QMessageBox::information( 0, tr( "Cannot freeze pattern" ),
 						tr( "The pattern currently "
@@ -561,33 +527,6 @@ void pattern::freeze( void )
 						QMessageBox::Ok );
 		return;
 	}
-	if( m_instrumentTrack->muted() || muted() )
-	{
-		if( QMessageBox::
-#if QT_VERSION >= 0x030200		
-				 question
-#else
-				 information
-#endif				 
-		
-					    ( 0, tr( "Pattern muted" ),
-						tr( "The track this pattern "
-							"belongs to or the "
-							"pattern itself is "
-							"currently muted "
-							"therefore "
-							"freezing makes no "
-							"sense! Do you still "
-							"want to continue?" ),
-						QMessageBox::Yes,
-						QMessageBox::No |
-						QMessageBox::Default |
-						QMessageBox::Escape ) ==
-			QMessageBox::No )
-		{
-			return;
-		}
-	}
 
 	// already frozen?
 	if( m_frozenPattern != NULL )
@@ -607,10 +546,8 @@ void pattern::unfreeze( void )
 {
 	if( m_frozenPattern != NULL )
 	{
-		m_frozenPatternMutex.lock();
-		delete m_frozenPattern;
+		sharedObject::unref( m_frozenPattern );
 		m_frozenPattern = NULL;
-		m_frozenPatternMutex.unlock();
 		update();
 	}
 }
@@ -719,13 +656,24 @@ void pattern::constructContextMenu( QMenu * _cm )
 						this, SLOT( changeName() ) );
 	_cm->addSeparator();
 
-	_cm->addAction( embed::getIconPixmap( "freeze" ),
-		( m_frozenPattern != NULL )? tr( "Refreeze" ) : tr( "Freeze" ),
+	bool freeze_separator = FALSE;
+	if( !( m_instrumentTrack->muted() || muted() ) )
+	{
+		_cm->addAction( embed::getIconPixmap( "freeze" ),
+			m_frozenPattern ? tr( "Refreeze" ) : tr( "Freeze" ),
 						this, SLOT( freeze() ) );
-	_cm->addAction( embed::getIconPixmap( "unfreeze" ), tr( "Unfreeze" ),
-						this, SLOT( unfreeze() ) );
-
-	_cm->addSeparator();
+		freeze_separator = TRUE;
+	}
+	if( m_frozenPattern )
+	{
+		_cm->addAction( embed::getIconPixmap( "unfreeze" ),
+				tr( "Unfreeze" ), this, SLOT( unfreeze() ) );
+		freeze_separator = TRUE;
+	}
+	if( freeze_separator )
+	{
+		_cm->addSeparator();
+	}
 
 #ifdef QT4
 	QMenu * add_step_menu = _cm->addMenu(
@@ -813,21 +761,21 @@ void pattern::mousePressEvent( QMouseEvent * _me )
 		{
 			n->setLength( -64 );
 		}
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 		update();
-		if( eng()->getPianoRoll()->currentPattern() == this )
+		if( engine::getPianoRoll()->currentPattern() == this )
 		{
-			eng()->getPianoRoll()->update();
+			engine::getPianoRoll()->update();
 		}
 	}
 	else if( m_frozenPattern != NULL && _me->button() == Qt::LeftButton &&
-			eng()->getMainWindow()->isShiftPressed() == TRUE )
+			engine::getMainWindow()->isShiftPressed() == TRUE )
 	{
 		QString s;
 		new stringPairDrag( "sampledata",
 					m_frozenPattern->toBase64( s ),
-					    embed::getIconPixmap( "freeze" ),
-					    this, eng() );
+					embed::getIconPixmap( "freeze" ),
+					this );
 	}
 	else
 	{
@@ -877,11 +825,11 @@ void pattern::wheelEvent( QWheelEvent * _we )
 				n->setLength( 0 );
 			}
 		}
-		eng()->getSongEditor()->setModified();
+		engine::getSongEditor()->setModified();
 		update();
-		if( eng()->getPianoRoll()->currentPattern() == this )
+		if( engine::getPianoRoll()->currentPattern() == this )
 		{
-			eng()->getPianoRoll()->update();
+			engine::getPianoRoll()->update();
 		}
 		_we->accept();
 	}
@@ -1079,7 +1027,7 @@ void pattern::paintEvent( QPaintEvent * )
 		for( noteVector::iterator it = m_notes.begin();
 						it != m_notes.end(); ++it )
 		{
-			Sint16 no = it - m_notes.begin();
+			Sint16 no = ( *it )->pos() / 4;
 			Sint16 x = TCO_BORDER_WIDTH + static_cast<int>( no *
 								w / steps );
 			Sint16 y = height() - s_stepBtnOff->height() - 1;
@@ -1160,7 +1108,7 @@ void pattern::ensureBeatNotes( void )
 		}
 		if( found == FALSE )
 		{
-			addNote( note( eng(), midiTime( 0 ), midiTime( i *
+			addNote( note( midiTime( 0 ), midiTime( i *
 							BEATS_PER_TACT ) ) );
 		}
 	}
@@ -1171,15 +1119,31 @@ void pattern::ensureBeatNotes( void )
 
 void pattern::updateBBTrack( void )
 {
-	if( getTrack()->getTrackContainer() == eng()->getBBEditor() )
+	if( getTrack()->getTrackContainer() == engine::getBBEditor() )
 	{
-		eng()->getBBEditor()->updateBBTrack( this );
+		engine::getBBEditor()->updateBBTrack( this );
 	}
 }
 
 
 
 
+bool pattern::empty( void )
+{
+	for( noteVector::iterator it = m_notes.begin(); it != m_notes.end();
+									++it )
+	{
+		if( ( *it )->length() != 0 )
+		{
+			return( FALSE );
+		}
+	}
+	return( TRUE );
+}
+
+
+
+
 
 
 
@@ -1287,8 +1251,6 @@ void patternFreezeStatusDialog::updateProgress( void )
 
 
 patternFreezeThread::patternFreezeThread( pattern * _pattern ) :
-	QThread(),
-	engineObject( _pattern->eng() ),
 	m_pattern( _pattern )
 {
 	// create status-dialog
@@ -1312,23 +1274,21 @@ patternFreezeThread::~patternFreezeThread()
 
 void patternFreezeThread::run( void )
 {
-	bufferAllocator::disableAutoCleanup( TRUE );
-
 	// create and install audio-sample-recorder
 	bool b;
 	// we cannot create local copy, because at a later stage
 	// mixer::restoreAudioDevice(...) deletes old audio-dev and thus
 	// audioSampleRecorder would be destroyed two times...
 	audioSampleRecorder * freeze_recorder = new audioSampleRecorder(
-						eng()->getMixer()->sampleRate(),
+					engine::getMixer()->sampleRate(),
 							DEFAULT_CHANNELS, b,
-							eng()->getMixer() );
-	eng()->getMixer()->setAudioDevice( freeze_recorder,
-					eng()->getMixer()->highQuality() );
+							engine::getMixer() );
+	engine::getMixer()->setAudioDevice( freeze_recorder,
+					engine::getMixer()->highQuality() );
 
 	// prepare stuff for playing correct things later
-	eng()->getSongEditor()->playPattern( m_pattern, FALSE );
-	songEditor::playPos & ppp = eng()->getSongEditor()->getPlayPos(
+	engine::getSongEditor()->playPattern( m_pattern, FALSE );
+	songEditor::playPos & ppp = engine::getSongEditor()->getPlayPos(
 						songEditor::PLAY_PATTERN );
 	ppp.setTact( 0 );
 	ppp.setTact64th( 0 );
@@ -1346,27 +1306,30 @@ void patternFreezeThread::run( void )
 		freeze_recorder->processNextBuffer();
 		m_statusDlg->setProgress( ppp * 100 / m_pattern->length() );
 	}
+	m_statusDlg->setProgress( 100 );
+	// render tails
+	while( engine::getMixer()->hasPlayHandles() &&
+					m_pattern->m_freezeAborted == FALSE )
+	{
+		freeze_recorder->processNextBuffer();
+	}
 
 
 	m_pattern->m_freezing = FALSE;
 
 	// reset song-editor settings
-	eng()->getSongEditor()->stop();
+	engine::getSongEditor()->stop();
 	ppp.m_timeLineUpdate = TRUE;
 
 	// create final sample-buffer if freezing was successful
 	if( m_pattern->m_freezeAborted == FALSE )
 	{
-		m_pattern->m_frozenPatternMutex.lock();
 		freeze_recorder->createSampleBuffer(
 						&m_pattern->m_frozenPattern );
-		m_pattern->m_frozenPatternMutex.unlock();
 	}
 
-	bufferAllocator::disableAutoCleanup( FALSE );
-
 	// restore original audio-device
-	eng()->getMixer()->restoreAudioDevice();
+	engine::getMixer()->restoreAudioDevice();
 
 	m_statusDlg->setProgress( -1 );	// we're finished
 
