@@ -4,7 +4,7 @@
  * cpuload_widget.cpp - widget for displaying CPU-load (partly based on
  *                      Hydrogen's CPU-load-widget)
  *
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -41,12 +41,12 @@
 
 #include "cpuload_widget.h"
 #include "embed.h"
+#include "engine.h"
 #include "mixer.h"
 
 
-cpuloadWidget::cpuloadWidget( QWidget * _parent, engine * _engine ) :
+cpuloadWidget::cpuloadWidget( QWidget * _parent ) :
 	QWidget( _parent ),
-	engineObject( _engine ),
 	m_currentLoad( 0 ),
 	m_temp(),
 	m_background( embed::getIconPixmap( "cpuload_bg" ) ),
@@ -106,7 +106,7 @@ void cpuloadWidget::paintEvent( QPaintEvent *  )
 void cpuloadWidget::updateCpuLoad()
 {
 	// smooth load-values a bit
-	Uint8 new_load = ( m_currentLoad + eng()->getMixer()->cpuLoad() ) / 2;
+	Uint8 new_load = ( m_currentLoad + engine::getMixer()->cpuLoad() ) / 2;
 	if( new_load != m_currentLoad )
 	{
 		m_currentLoad = new_load;
