@@ -3,8 +3,8 @@
 /*
  * tempo_sync_knob.cpp - adds bpm to ms conversion for knob class
  *
- * Copyright (c) 2005 Danny McRae <khjklujn/at/yahoo.com>
- * Copyright (c) 2005-2006 Tobias Doerffel <tobydox/at/users.sourceforge.net>
+ * Copyright (c) 2005-2007 Danny McRae <khjklujn/at/yahoo.com>
+ * Copyright (c) 2005-2007 Tobias Doerffel <tobydox/at/users.sourceforge.net>
  * 
  * This file is part of Linux MultiMedia Studio - http://lmms.sourceforge.net
  *
@@ -20,8 +20,8 @@
  *
  * You should have received a copy of the GNU General Public
  * License along with this program (see COPYING); if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA.
  *
  */
 
@@ -47,25 +47,30 @@
 
 
 #include "tempo_sync_knob.h"
-#include "song_editor.h"
+#include "automatable_object_templates.h"
 #include "embed.h"
-
-
+#include "main_window.h"
+#include "meter_dialog.h"
+#include "song_editor.h"
 
 
 tempoSyncKnob::tempoSyncKnob( int _knob_num, QWidget * _parent,
-					const QString & _name,
-					engine * _engine, track * _track,
-					float _scale ) :
-	knob( _knob_num, _parent, _name, _engine, _track ),
+							const QString & _name,
+							track * _track,
+							float _scale ) :
+	knob( _knob_num, _parent, _name, _track ),
 	m_tempoSyncMode( NO_SYNC ),
 	m_scale( _scale ),
 	m_tempoSyncIcon( embed::getIconPixmap( "tempo_sync" ) ),
 	m_tempoSyncDescription( tr( "Tempo Sync" ) ),
 	m_tempoLastSyncMode( NO_SYNC )
 {
-	connect( eng()->getSongEditor(), SIGNAL( tempoChanged( bpm_t ) ),
-		 this, SLOT( calculateTempoSyncTime( bpm_t ) ) );
+	connect( engine::getSongEditor(), SIGNAL( tempoChanged( bpm_t ) ),
+			this, SLOT( calculateTempoSyncTime( bpm_t ) ) );
+	m_custom = new meterDialog( engine::getMainWindow()->workspace(),
+								_track );
+	m_custom->hide();
+	m_custom->setWindowTitle( "Meter" );
 }
 
 
@@ -73,6 +78,10 @@ tempoSyncKnob::tempoSyncKnob( int _knob_num, QWidget * _parent,
 
 tempoSyncKnob::~tempoSyncKnob()
 {
+	if( m_custom )
+	{
+		m_custom->deleteLater();
+	}
 }
 
 
@@ -105,77 +114,136 @@ void tempoSyncKnob::contextMenuEvent( QContextMenuEvent * )
 							m_hintTextAfterValue ),
 				this, SLOT( pasteValue() ) );
 	contextMenu.addSeparator();
+	
+	float limit = 60000.0f / ( engine::getSongEditor()->getTempo() *
+								m_scale );
+	
 #ifdef QT4
 	QMenu * syncMenu = contextMenu.addMenu( m_tempoSyncIcon,
 						m_tempoSyncDescription );
+#else
+	QMenu * syncMenu = new QMenu( this );
+#endif
+	if( limit / 8.0f <= maxValue() )
+	{
+#ifdef QT4
 	connect( syncMenu, SIGNAL( triggered( QAction * ) ),
 			this, SLOT( setTempoSync( QAction * ) ) );
 	syncMenu->addAction( embed::getIconPixmap( "note_none" ),
 				tr( "No Sync" ) )->setData( (int) NO_SYNC );
-	syncMenu->addAction( embed::getIconPixmap( "note_double_whole" ),
+	if( limit / 0.125f <= maxValue() )
+	{
+		syncMenu->addAction( embed::getIconPixmap( "note_double_whole" ),
 				tr( "Eight beats" ) )->setData(
 						(int) DOUBLE_WHOLE_NOTE );
-	syncMenu->addAction( embed::getIconPixmap( "note_whole" ),
+	}
+	if( limit / 0.25f <= maxValue() )
+	{
+		syncMenu->addAction( embed::getIconPixmap( "note_whole" ),
 					tr( "Whole note" ) )->setData(
 						(int) WHOLE_NOTE );
-	syncMenu->addAction( embed::getIconPixmap( "note_half" ),
+	}
+	if( limit / 0.5f <= maxValue() )
+	{
+		syncMenu->addAction( embed::getIconPixmap( "note_half" ),
 					tr( "Half note" ) )->setData(
 						(int) HALF_NOTE );
-	syncMenu->addAction( embed::getIconPixmap( "note_quarter" ),
+	}
+	if( limit <= maxValue() )
+	{
+		syncMenu->addAction( embed::getIconPixmap( "note_quarter" ),
 					tr( "Quarter note" ) )->setData(
 						(int) QUARTER_NOTE );
-	syncMenu->addAction( embed::getIconPixmap( "note_eighth" ),
+	}
+	if( limit / 2.0f <= maxValue() )
+	{
+		syncMenu->addAction( embed::getIconPixmap( "note_eighth" ),
 					tr( "8th note" ) )->setData(
 						(int) EIGHTH_NOTE );
-	syncMenu->addAction( embed::getIconPixmap( "note_sixteenth" ),
+	}
+	if( limit / 4.0f <= maxValue() )
+	{
+		syncMenu->addAction( embed::getIconPixmap( "note_sixteenth" ),
 					tr( "16th note" ) )->setData(
 					(int) SIXTEENTH_NOTE );
+	}
 	syncMenu->addAction( embed::getIconPixmap( "note_thirtysecond" ),
 					tr( "32nd note" ) )->setData(
 					(int) THIRTYSECOND_NOTE );
+	syncMenu->addAction( embed::getIconPixmap( "dont_know" ),
+				tr( "Custom..." ),
+				this, SLOT( showCustom( void ) )
+						)->setData( (int) CUSTOM );
 #else
-	QMenu * syncMenu = new QMenu( this );
 	int menuId;
 	menuId = syncMenu->addAction( embed::getIconPixmap( "note_none" ),
 					tr( "No Sync" ),
 					this, SLOT( setTempoSync( int ) ) );
 	syncMenu->setItemParameter( menuId, ( int ) NO_SYNC );
-	menuId = syncMenu->addAction( embed::getIconPixmap(
-							"note_double_whole" ),
+	if( limit / 0.125f <= maxValue() )
+	{
+		menuId = syncMenu->addAction( 
+				embed::getIconPixmap( "note_double_whole" ),
 					tr( "Eight beats" ),
 					this, SLOT( setTempoSync( int ) ) );
-	syncMenu->setItemParameter( menuId, ( int ) DOUBLE_WHOLE_NOTE );
-	menuId = syncMenu->addAction( embed::getIconPixmap( "note_whole" ),
+		syncMenu->setItemParameter( menuId, ( int ) DOUBLE_WHOLE_NOTE );
+	}
+	if( limit / 0.25f <= maxValue() )
+	{
+		menuId = syncMenu->addAction( 
+				embed::getIconPixmap( "note_whole" ),
 					tr( "Whole note" ),
 					this, SLOT( setTempoSync( int ) ) );
-	syncMenu->setItemParameter( menuId, ( int ) WHOLE_NOTE );
-	menuId = syncMenu->addAction( embed::getIconPixmap( "note_half" ),
+		syncMenu->setItemParameter( menuId, ( int ) WHOLE_NOTE );
+	}
+	if( limit / 0.5f <= maxValue() )
+	{
+		menuId = syncMenu->addAction( 
+				embed::getIconPixmap( "note_half" ),
 					tr( "Half note" ),
 					this, SLOT( setTempoSync( int ) ) );
-	syncMenu->setItemParameter( menuId, ( int ) HALF_NOTE );
-	menuId = syncMenu->addAction( embed::getIconPixmap( "note_quarter" ),
+		syncMenu->setItemParameter( menuId, ( int ) HALF_NOTE );
+	}
+	if( limit <= maxValue() )
+	{
+		menuId = syncMenu->addAction( 
+				embed::getIconPixmap( "note_quarter" ),
 					tr( "Quarter note" ),
 			     		this, SLOT( setTempoSync( int ) ) );
-	syncMenu->setItemParameter( menuId, ( int ) QUARTER_NOTE );
-	menuId = syncMenu->addAction( embed::getIconPixmap( "note_eighth" ),
+		syncMenu->setItemParameter( menuId, ( int ) QUARTER_NOTE );
+	}
+	if( limit / 2.0f <= maxValue() )
+	{
+		menuId = syncMenu->addAction( 
+				embed::getIconPixmap( "note_eighth" ),
 					tr( "8th note" ),
 					this, SLOT( setTempoSync( int ) ) );
-	syncMenu->setItemParameter( menuId, ( int ) EIGHTH_NOTE );
-	menuId = syncMenu->addAction( embed::getIconPixmap( "note_sixteenth" ),
+		syncMenu->setItemParameter( menuId, ( int ) EIGHTH_NOTE );
+	}
+	if( limit / 4.0f <= maxValue() )
+	{
+		menuId = syncMenu->addAction( 
+				embed::getIconPixmap( "note_sixteenth" ),
 					tr( "16th note" ),
 					this, SLOT( setTempoSync( int ) ) );
-	syncMenu->setItemParameter( menuId, ( int ) SIXTEENTH_NOTE );
+		syncMenu->setItemParameter( menuId, ( int ) SIXTEENTH_NOTE );
+	}
 	menuId = syncMenu->addAction( embed::getIconPixmap(
 							"note_thirtysecond" ),
 					tr( "32nd note" ),
 					this, SLOT( setTempoSync( int ) ) );
 	syncMenu->setItemParameter( menuId, ( int ) THIRTYSECOND_NOTE );
+	menuId = syncMenu->addAction( embed::getIconPixmap( "dont_know" ),
+					tr( "Custom..." ),
+					this, SLOT( showCustom( void ) ) );
+	syncMenu->setItemParameter( menuId, ( int ) CUSTOM );
 	
 	contextMenu.addMenu( m_tempoSyncIcon, m_tempoSyncDescription,
 								syncMenu );
 #endif
 
 	contextMenu.addSeparator();
+	}
 	
 	contextMenu.addAction( embed::getIconPixmap( "automation" ),
 					tr( "&Open in automation editor" ),
@@ -183,7 +251,7 @@ void tempoSyncKnob::contextMenuEvent( QContextMenuEvent * )
 					SLOT( openInAutomationEditor() ) );
 	contextMenu.addSeparator();
 	contextMenu.addAction( tr( "Connect to MIDI-device" ), this,
-						SLOT( connectToMidiDevice() ) );
+					SLOT( connectToMidiDevice() ) );
 	contextMenu.addSeparator();
 	contextMenu.addAction( embed::getIconPixmap( "help" ), tr( "&Help" ),
 						this, SLOT( displayHelp() ) );
@@ -198,7 +266,7 @@ void tempoSyncKnob::contextMenuEvent( QContextMenuEvent * )
 void tempoSyncKnob::mouseMoveEvent( QMouseEvent * _me )
 {
 	m_tempoSyncMode = NO_SYNC;
-	calculateTempoSyncTime( eng()->getSongEditor()->getTempo() );
+	calculateTempoSyncTime( engine::getSongEditor()->getTempo() );
 	knob::mouseMoveEvent( _me );
 }
 
@@ -209,7 +277,7 @@ void tempoSyncKnob::wheelEvent( QWheelEvent * _we )
 {
 	knob::wheelEvent( _we );
 	m_tempoSyncMode = NO_SYNC;
-	calculateTempoSyncTime( eng()->getSongEditor()->getTempo() );
+	calculateTempoSyncTime( engine::getSongEditor()->getTempo() );
 }
 
 
@@ -231,9 +299,8 @@ void tempoSyncKnob::setTempoSync( QAction * ) { }
 
 void tempoSyncKnob::setTempoSync( int _note_type )
 {
-	m_tempoSyncMode = ( tempoSyncMode ) _note_type;
-	calculateTempoSyncTime( eng()->getSongEditor()->getTempo() );
-	eng()->getSongEditor()->setModified();
+	setSyncMode( ( tempoSyncMode ) _note_type );
+	engine::getSongEditor()->setModified();
 }
 
 
@@ -247,76 +314,138 @@ void tempoSyncKnob::calculateTempoSyncTime( bpm_t _bpm )
 	{
 		switch( m_tempoSyncMode )
 		{
+			case CUSTOM:
+				m_tempoSyncDescription = tr( "Custom " ) + 
+						"(" +
+				QString::number( m_custom->getNumerator() ) +
+						"/" +
+				QString::number( m_custom->getDenominator() ) +
+						")";
+				conversionFactor = 
+			static_cast<float>( m_custom->getDenominator() ) /
+			static_cast<float>( m_custom->getNumerator() );
+				break;
 			case DOUBLE_WHOLE_NOTE:
 				m_tempoSyncDescription = tr(
 						"Synced to Eight Beats" );
-				m_tempoSyncIcon = embed::getIconPixmap(
-							"note_double_whole" );
 				conversionFactor = 0.125;
 				break;
 			case WHOLE_NOTE:
 				m_tempoSyncDescription = tr(
 						"Synced to Whole Note" );
-				m_tempoSyncIcon = embed::getIconPixmap(
-								"note_whole" );
 				conversionFactor = 0.25;
 				break;
 			case HALF_NOTE:
 				m_tempoSyncDescription = tr(
 							"Synced to Half Note" );
-				m_tempoSyncIcon = embed::getIconPixmap(
-								"note_half" );
 				conversionFactor = 0.5;
 				break;
 			case QUARTER_NOTE:
 				m_tempoSyncDescription = tr(
 						"Synced to Quarter Note" );
-				m_tempoSyncIcon = embed::getIconPixmap(
-							"note_quarter" );
 				conversionFactor = 1.0;
 				break;
 			case EIGHTH_NOTE:
 				m_tempoSyncDescription = tr(
 							"Synced to 8th Note" );
-				m_tempoSyncIcon = embed::getIconPixmap(
-								"note_eighth" );
 				conversionFactor = 2.0;
 				break;
 			case SIXTEENTH_NOTE:
 				m_tempoSyncDescription = tr(
 							"Synced to 16th Note" );
-				m_tempoSyncIcon = embed::getIconPixmap(
-							"note_sixteenth" );
 				conversionFactor = 4.0;
 				break;
 			case THIRTYSECOND_NOTE:
 				m_tempoSyncDescription = tr(
 							"Synced to 32nd Note" );
+				conversionFactor = 8.0;
+				break;
+			default: ;
+		}
+		bool journalling = testAndSetJournalling( FALSE );
+		setValue( 60000.0 / ( _bpm * conversionFactor * m_scale ) );
+		setJournalling( journalling );
+	}
+	else
+	{
+		m_tempoSyncDescription = tr( "Tempo Sync" );
+	}
+	
+	if( m_tempoSyncMode != m_tempoLastSyncMode )
+	{
+		switch( m_tempoSyncMode )
+		{
+			case NO_SYNC:
+				m_tempoSyncIcon = embed::getIconPixmap(
+								"tempo_sync" );
+				break;
+			case CUSTOM:
+				m_tempoSyncIcon = embed::getIconPixmap(
+								"dont_know" );
+				break;
+			case DOUBLE_WHOLE_NOTE:
+				m_tempoSyncIcon = embed::getIconPixmap(
+							"note_double_whole" );
+				break;
+			case WHOLE_NOTE:
+				m_tempoSyncIcon = embed::getIconPixmap(
+								"note_whole" );
+				break;
+			case HALF_NOTE:
+				m_tempoSyncIcon = embed::getIconPixmap(
+								"note_half" );
+				break;
+			case QUARTER_NOTE:
+				m_tempoSyncIcon = embed::getIconPixmap(
+							"note_quarter" );
+				break;
+			case EIGHTH_NOTE:
+				m_tempoSyncIcon = embed::getIconPixmap(
+								"note_eighth" );
+				break;
+			case SIXTEENTH_NOTE:
+				m_tempoSyncIcon = embed::getIconPixmap(
+							"note_sixteenth" );
+				break;
+			case THIRTYSECOND_NOTE:
 				m_tempoSyncIcon = embed::getIconPixmap(
 							"note_thirtysecond" );
-				conversionFactor = 8.0;
 				break;
 			default:
 				printf( "tempoSyncKnob::calculateTempoSyncTime"
 						": invalid tempoSyncMode" );
 				break;
 		}
-		setValue( 60000.0 / ( _bpm * conversionFactor * m_scale ) );
-	}
-	else
-	{
-		m_tempoSyncDescription = tr( "Tempo Sync" );
-		m_tempoSyncIcon = embed::getIconPixmap( "tempo_sync" );
-	}
-	
-	if( m_tempoSyncMode != m_tempoLastSyncMode )
-	{
+
 		emit syncModeChanged( m_tempoSyncMode );
 		emit syncDescriptionChanged( m_tempoSyncDescription );
 		emit syncIconChanged();
+
+		m_tempoLastSyncMode = m_tempoSyncMode;
 	}
-	
-	m_tempoLastSyncMode = m_tempoSyncMode;
+}
+
+
+
+
+void tempoSyncKnob::saveSettings( QDomDocument & _doc, QDomElement & _this,
+							const QString & _name )
+{
+ 	_this.setAttribute( "syncmode", ( int ) getSyncMode() );
+	automatableObject<float>::saveSettings( _doc, _this, _name );
+	m_custom->saveSettings( _doc, _this, _name );
+}
+
+
+
+
+void tempoSyncKnob::loadSettings( const QDomElement & _this,
+							const QString & _name )
+{
+ 	setSyncMode( ( tempoSyncMode ) _this.attribute(
+ 						"syncmode" ).toInt() );
+	automatableObject<float>::loadSettings( _this, _name );
+	m_custom->loadSettings( _this, _name );
 }
 
 
@@ -332,8 +461,24 @@ tempoSyncKnob::tempoSyncMode tempoSyncKnob::getSyncMode( void )
 
 void tempoSyncKnob::setSyncMode( tempoSyncMode _new_mode )
 {
-	m_tempoSyncMode = _new_mode;
-	calculateTempoSyncTime( eng()->getSongEditor()->getTempo() );
+	if( m_tempoSyncMode != _new_mode )
+	{
+		m_tempoSyncMode = _new_mode;
+		if( _new_mode == CUSTOM )
+		{
+			connect( m_custom, SIGNAL( numeratorChanged( int ) ),
+					this, SLOT( updateCustom( int ) ) );
+			connect( m_custom, SIGNAL( denominatorChanged( int ) ),
+					this, SLOT( updateCustom( int ) ) );
+		}
+		else
+		{
+			m_custom->hide();
+			disconnect( m_custom, 0,
+					this, SLOT( updateCustom( int ) ) );
+		}
+	}
+	calculateTempoSyncTime( engine::getSongEditor()->getTempo() );
 }
 
 
@@ -350,7 +495,7 @@ float tempoSyncKnob::getScale( void )
 void tempoSyncKnob::setScale( float _new_scale )
 {
 	m_scale = _new_scale;
-	calculateTempoSyncTime( eng()->getSongEditor()->getTempo() );
+	calculateTempoSyncTime( engine::getSongEditor()->getTempo() );
 	emit scaleChanged( _new_scale );
 }
 
@@ -390,6 +535,23 @@ void tempoSyncKnob::setSyncIcon( const QPixmap & _new_icon )
 
 
 
+
+void tempoSyncKnob::updateCustom( int )
+{
+	setSyncMode( CUSTOM );
+}
+
+
+
+
+void tempoSyncKnob::showCustom( void )
+{
+	m_custom->show();
+	setTempoSync( CUSTOM );
+}
+
+
+
 #ifndef QT4
 #undef addSeparator
 #endif
